/*
 *  AActivateApplication.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Dec 16 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "XApplicationIdentifier.h"

enum class AlreadyActivatedActionType {
	kNormal,
	kSwitchToLast,
	kHide,
	kQuit,
	kDEFAULT = kNormal,
};
#define kAlreadyActivatedActionTypeNamesArray {CFSTR("Normal"), CFSTR("SwitchToLast"), CFSTR("Hide"), CFSTR("Quit"), NULL}


class AActivateApplication
	:	public HBaseAction
{
private:
	
	XApplicationIdentifier		mApplication;
	Boolean						mReopenWindows = false;
	Boolean						mAllWindows = true;
	AlreadyActivatedActionType	mAlreadyActivatedActionType = AlreadyActivatedActionType::kDEFAULT;
	
#if KMF_IS_ENGINE
	int				mActionState;
#endif

public:
	
    AActivateApplication() = default;
	AActivateApplication( const AActivateApplication& inAction ) = default;
    virtual ~AActivateApplication() = default;
	
	virtual AActivateApplication*	Clone() const { return new AActivateApplication( *this ); }
	virtual AActivateApplication*	Create() const { return new AActivateApplication(); }
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues();
#endif
	
	virtual	CFStringRef				CopyActionDescriptionString() const;
 	virtual	XIcon					GetActionDescriptionXIcon() const;
	
	virtual HActionType				GetActionType() const { return HActionType::kActivateApplication; }

    virtual const XApplicationIdentifier&	GetApplication() const { return mApplication; }
	virtual void							SetApplication( const XApplicationIdentifier& inApp ) { mApplication = inApp; }
	
	void							SetReopenWindows( Boolean inValue ) { mReopenWindows = inValue; }
	Boolean							GetReopenWindows() const { return mReopenWindows; }

	void							SetAllWindows( Boolean inValue ) { mAllWindows = inValue; }
	Boolean							GetAllWindows() const { return mAllWindows; }

	void							SetAlreadyActivatedActionType( AlreadyActivatedActionType inValue ) { mAlreadyActivatedActionType = inValue; }
	AlreadyActivatedActionType		GetAlreadyActivatedActionType() const { return mAlreadyActivatedActionType; }

	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return 20*kEventDurationSecond; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
	Boolean							IsActionCompleted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual BOOL					CanBeInstigatedByClickAction() const { return YES; }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
};

