/*
 *  AAlert.h
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Tue 17 Jun 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"

#if KMF_IS_ENGINE
@class KMAlertController;
#endif

class AAlert
	:	public HBaseAction
{

private:

    CFContainer<CFStringRef>		mTitle;
    CFContainer<CFStringRef>		mText;

#if KMF_IS_ENGINE
	KMAlertController*				mAlert;
#endif

public:

    AAlert();
	AAlert( const AAlert& inAction );
    virtual ~AAlert();

	virtual AAlert*					Clone() const { return new AAlert( *this ); }
	virtual AAlert*					Create() const { return new AAlert(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kAlert; }

	void							SetTitle( CFStringRef inString ) { mTitle = inString->Copy(); }
	CFStringRef						CopyTitle() const { return mTitle->Copy(); }
	NSString*						GetTitle() const { return mTitle->GetNS(); }

	void							SetText( CFStringRef inString ) { mText = inString->Copy(); }
	CFStringRef						CopyText() const { return mText->Copy(); }
	NSString*						GetText() const { return mText->GetNS(); }

	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return 3*kEventDurationMinute; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;

	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

