/*
 *  AAlert.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Tue 17 Jun 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "AAlert.h"
#import "NSObjectExtensions.h"
#import "NSStringExtensions.h"
#import "KMVariables.h"

#if KMF_IS_ENGINE
#import "KMAlert.h"
#import "TokenText.h"
#import "HExecuteActionListManager.h"
#import "KMVariables.h"
#import "KMNotifications.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AAlertEditor.h"
#import "TemplateEngine.h"
#endif

AAlert::AAlert()
	:	HBaseAction(),
		mTitle( CFSTR("kAlertDefaultTitle")->CopyLocalized() ),
		mText( CFSTR("kAlertDefaultText")->CopyLocalized() )
{
#if KMF_IS_ENGINE
	mAlert = nil;
#endif
}

AAlert::AAlert( const AAlert& inAction )
	:	HBaseAction( inAction ),
		mTitle( inAction.mTitle ),
		mText( inAction.mText )
{
#if KMF_IS_ENGINE
	mAlert = nil;
#endif
}

AAlert::~AAlert()
{
#if KMF_IS_ENGINE
	[mAlert Kill];
#endif
}

CFStringRef
AAlert::CopyActionDescriptionString() const
{
	return CFString::CreateWithFormat( CFCS( CFSTR("hkAlertTitle")->CopyLocalized() ), CFStringRef(mTitle) );
}

#if KMF_IS_ENGINE

float
AAlert::DoAction()
{
	float time = 5;

	KMVariables::Set( kAlertResultButtonVariable, @"", KMNotifications::kLoadVariableFromEngineAction );
	
	CFContainer<CFStringRef> mDisplayText = TokenText::CopyResult( mText, time );
	mAlert = [KMAlertController newAlertController];
	mAlert.title = mTitle->GetNS();
	mAlert.explanation = mDisplayText->GetNS();
	mAlert.button1Text = [@"AlertContinueButton" localized];
	mAlert.button2Text = [@"AlertStopButton" localized];
	[mAlert Show];
	return time;
}

Boolean
AAlert::IsActionCompleted()
{
	if ( [mAlert IsComplete] ) {
		if ( mAlert.resultButton == 2 ) {
			HExecuteActionListManager::CancelThisExecutionGroup();
			KMVariables::Set( kAlertResultButtonVariable, mAlert.button2Text, KMNotifications::kLoadVariableFromEngineAction );
		} else {
			KMVariables::Set( kAlertResultButtonVariable, mAlert.button1Text, KMNotifications::kLoadVariableFromEngineAction );
		}
		[mAlert Kill];
		mAlert = nil;
	}
	return mAlert == nil;
}

void
AAlert::ActionAborted()
{
	[mAlert Kill];
	mAlert = nil;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AAlert::ActionController() const
{
	return [AAlertEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
 {{ text }}
 Store button pressed in variable ‘{{ result | htmlencode }}’.
)RAW";

NSString*
AAlert::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
							@"text" : UserTextForHTML( mText->GetNS() ),
							@"result" : kAlertResultButtonVariable,
						};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
AAlert::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Title"), CFStringRef(mTitle) );
	dictionary->Set( CFSTR("Text"), CFStringRef(mText) );
}

void
AAlert::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef newTitle;
	if ( dictionary->Get( CFSTR("Title"), newTitle ) ) {
		mTitle = newTitle->Copy();
	}

	CFStringRef newText;
	XAssert( dictionary->Get( CFSTR("Text"), newText ) );
	mText = newText->Copy();
}
