//
//  AApplicationLauncher.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 7 Sep 2010.
//  Copyright (c) 2010 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "XApplicationList.h"
#import "CopyAllApplications.h"

class AApplicationLauncher
	:	public HBaseAction
{

private:

#if KMF_IS_ENGINE
	NSUUID*							mUUID = [NSUUID UUID];
#endif

	NSColor*						mColor;
	Boolean							mSwitcherUnderMouse = false;
	Boolean							mHideOthers = false;
	SInt16							mApplicationMask = CopyAllApplications::kAll;
	Boolean							mExcludeRunning = false;
	XApplicationList				mIncludedApps;
	XApplicationList				mExcludedApps;

public:

    AApplicationLauncher();
	AApplicationLauncher( const AApplicationLauncher& inAction ) = default;
    virtual ~AApplicationLauncher() = default;

	virtual AApplicationLauncher*		Clone() const { return new AApplicationLauncher( *this ); }
	virtual AApplicationLauncher*		Create() const { return new AApplicationLauncher(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kApplicationLauncher; }

#if KMF_IS_ENGINE
    NSString*						GetUID() const { return [mUUID UUIDString]; }
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	NSColor*						GetSwitcherColor() const { return mColor; }
	void							SetSwitcherColor( NSColor* inColor );

	Boolean							GetSwitcherUnderMouse() const { return mSwitcherUnderMouse; }
	void							SetSwitcherUnderMouse( Boolean inSwitcherUnderMouse ) { mSwitcherUnderMouse = inSwitcherUnderMouse; }
	
	Boolean							GetHideOthers() const { return mHideOthers; }
	void							SetHideOthers( Boolean inHideOthers ) { mHideOthers = inHideOthers; }
	
	NSInteger						GetApplicationMask() const { return mApplicationMask; }
	void							SetApplicationMask( NSInteger inApplicationMask ) { mApplicationMask = inApplicationMask; }
	
	Boolean							GetExcludeRunning() const { return mExcludeRunning; }
	void							SetExcludeRunning( Boolean inExcludeRunning ) { mExcludeRunning = inExcludeRunning; }
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
	
	const XApplicationList&			GetIncludedApps() const { return mIncludedApps; }
	void							SetIncludedApps( const XApplicationList& inIncludedApps ) { mIncludedApps = inIncludedApps; }
	
	const XApplicationList&			GetExcludedApps() const { return mExcludedApps; }
	void							SetExcludedApps( const XApplicationList& inExcludedApps ) { mExcludedApps = inExcludedApps; }
	
};

