//
//  AApplicationLauncher.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 7 Sep 2010.
//  Copyright (c) 2010 Stairways Software. All rights reserved.
//

#import "AApplicationLauncher.h"
#import "CopyAllApplications.h"
#import "NSColorExtensions.h"

#if KMF_IS_ENGINE
#import "ALHUDController.h"
#import "HExecuteActionListManager.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AApplicationLauncherEditor.h"
#import "NSImageExtensions.h"
#import "TemplateEngine.h"
#endif

static const RGBColor kDefaultColor = {0, 128, 128};

enum {
	kDefaultAlpha = 70,
};

AApplicationLauncher::AApplicationLauncher()
	:	HBaseAction(),
		mColor( [NSColor colorWithCalibratedRed:kDefaultColor.red/255.0 green:kDefaultColor.green/255.0 blue:kDefaultColor.blue/255.0 alpha:kDefaultAlpha/100.0] )
{
}

#if KMF_IS_ENGINE

float
AApplicationLauncher::DoAction()
{
	return [ALHUDController TriggerFired:this modifiers:HExecuteActionListManager::GetTrippedModifiers()] ? 10 : 0.5;
}

#endif

void AApplicationLauncher::SetSwitcherColor( NSColor* inColor )
{
	mColor = [[inColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] copy];
}

#if KMF_EDITOR_WINDOW

Class
AApplicationLauncher::ActionController() const
{
	return [AApplicationLauncherEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
 <span class="IconTitle">Color: </span><img src={{ color | inlineimage: 16 }} alt="Swatch" width="16" height="16" /><br />
 {% if placeundermouse %}Place launcher under mouse.<br />{% /if %}
 {% if hideothers %}Hide other applications when launching.<br />{% /if %}
 Display {{ display | htmlencode }}.<br />
 {% if includecount > 0 %}
 Always include these applications:<br />
	{{ include }}
 {% /if %}
 {% if excludecount > 0 %}
 Always exclude these applications:<br />
	{{ exclude }}
 {% /if %}
)RAW";

NSString*
AApplicationLauncher::GetActionExtraHTML() const
{
	NSMutableArray* display = [NSMutableArray new];
	if ( mApplicationMask & CopyAllApplications::kApplications ) [display addObject:@"applications"];
	if ( mApplicationMask & CopyAllApplications::kUtilities ) [display addObject:@"utilities"];
	if ( mApplicationMask & CopyAllApplications::kRunning ) [display addObject:@"running applications"];
	if ( mApplicationMask & CopyAllApplications::kRecent ) [display addObject:@"recently quit applications"];
	if ( display.count == 0 ) [display addObject:@"nothing"];

	NSDictionary *variables = @{
									@"placeundermouse" : @(mSwitcherUnderMouse),
									@"hideothers" : @(mHideOthers),
									@"color" : [NSImage imageWithSwatch:mColor],
									@"display" : [display componentsJoinedByString:@", "],
									@"includecount" : @(mIncludedApps.GetCount()),
									@"include" : mIncludedApps.GetDisplayHTMLContent(),
									@"excludecount" : @(mExcludedApps.GetCount()),
									@"exclude" : mExcludedApps.GetDisplayHTMLContent(),
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
AApplicationLauncher::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	[mColor saveWithAlphaToDictionary:dictionary];
	dictionary->SetBoolean( CFSTR("SwitcherUnderMouse"), mSwitcherUnderMouse );
	dictionary->SetBoolean( CFSTR("HideOthers"), mHideOthers );
	dictionary->SetNumber( CFSTR("ApplicationMask"), mApplicationMask );
	dictionary->SetBoolean( CFSTR("ExcludeRunning"), mExcludeRunning );
	SafeCFPropertyListRef apps = mIncludedApps.CopyToPropertyList();
	dictionary->Set( CFSTR("IncludedApplications"), apps );
	CFRelease( apps );
	apps = mExcludedApps.CopyToPropertyList();
	dictionary->Set( CFSTR("ExcludedApplications"), apps );
	CFRelease( apps );
}

void
AApplicationLauncher::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mColor = [[NSColor colorFromDictionary:dictionary defaultColor:kDefaultColor defaultAlpha:kDefaultAlpha] copy];
	dictionary->GetBoolean( CFSTR("SwitcherUnderMouse"), mSwitcherUnderMouse, false );
	dictionary->GetBoolean( CFSTR("HideOthers"), mHideOthers, false );
	dictionary->GetBoolean( CFSTR("ExcludeRunning"), mExcludeRunning, false );
	if ( !dictionary->Get( CFSTR("ApplicationMask"), mApplicationMask ) ) {
		mApplicationMask = CopyAllApplications::kAll;
	}
	SafeCFPropertyListRef apps = (SafeCFPropertyListRef)dictionary->Get( CFSTR("IncludedApplications") );
	mIncludedApps.LoadFromPropertyList( apps );
	apps = (SafeCFPropertyListRef)dictionary->Get( CFSTR("ExcludedApplications") );
	mExcludedApps.LoadFromPropertyList( apps );
}

