//
//  AApplyStyle.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 5 Apr 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ClipboardTarget.h"
#import "StringRange.h"

// Attributes
#define kApplyStyleFontNameKey @"KMFontName"
#define kApplyStyleFontSizeKey @"KMFontSize"
#define kApplyStyleFontSmallerKey @"KMSmaller"
#define kApplyStyleFontBiggerKey @"KMBigger"
// Colors are all savable string values

class AApplyStyle
	:	public HBaseAction
{

private:

	ClipboardTarget					mTarget;
	StringRangeType					mStringRange = StringRangeType::kAll;
	NSString*						mFirst = @"2";
	NSString*						mSecond = @"4";
	NSDictionary*					mAttributes = [NSDictionary new];

public:

    AApplyStyle() = default;
	AApplyStyle( const AApplyStyle& inAction ) = default;
    virtual ~AApplyStyle() = default;

	virtual AApplyStyle*			Clone() const { return new AApplyStyle( *this ); }
	virtual AApplyStyle*			Create() const { return new AApplyStyle(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kApplyStyle; }
	virtual CFStringRef				CopyActionDescriptionString() const;

#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); }
#endif

    float							Apply( NSMutableAttributedString* s, bool applyToAll ) const;

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	bool							NeedFirst() const { return StringRange::CountTypeValues( mStringRange ) > 0; }
	bool							NeedSecond() const { return StringRange::CountTypeValues( mStringRange ) > 1; }

	NSString*						GetGenericNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }

	StringRangeType					GetStringRange() const { return mStringRange; }
	void							SetStringRange( StringRangeType inStringRange ) { mStringRange = inStringRange; }

	NSString*						GetFirst() const { return mFirst; }
	void							SetFirst( NSString* inFirst ) { mFirst = [inFirst copy]; }

	NSString*						GetSecond() const { return mSecond; }
	void							SetSecond( NSString* inSecond ) { mSecond = [inSecond copy]; }

	NSDictionary*					GetAttributes() const { return mAttributes; }
	void							SetAttributes( NSDictionary* inAttributes ) { mAttributes = [inAttributes copy]; }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
	
};

