/*
 *  AApplyTextFactory.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Thu 15 May 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "AApplyTextFactory.h"
#import "FileReference.h"

#if KMF_IS_ENGINE
#import "XClipboardHistory.h"
#import "XClipboardDataSet.h"
#import "NSStringExtensions.h"
#import "NSAppleScriptExtensions.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AApplyTextFactoryEditor.h"
#import "NSStringExtensions.h"
#import "TemplateEngine.h"
#endif

CFStringRef
AApplyTextFactory::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> theNameStr = (CFStringRef)CFBridgingRetain(FileReference::PathToName( mPath->GetNS() ));
	CFContainer<CFStringRef> theFormatStr = CFSTR("kmApplyTextFactoryTitleFormat")->CopyLocalized();
	return CFString::CreateWithFormat( theFormatStr, CFStringRef(theNameStr) );
}

#if KMF_IS_ENGINE

float
AApplyTextFactory::DoAction()
{
	float time = 10;

	NSString* factoryPath = FileReference::PathToActualPath( mPath->GetNS() );
	if ( factoryPath ) {
		NSString* tempFilePath = [NSString tempFileWithBasename:@"Keyboard-Maestro-Clipboard" extension:@"txt"];
		
		const XClipboardDataSet* clip = mTarget.GetClipboardDataSet();
		if ( clip ) {
			CFContainer<CFStringRef> clipboardText = clip->CopyIfText();
			if ( clipboardText && [clipboardText->GetNS() writeToFile:tempFilePath atomically:NO encoding:NSUTF8StringEncoding error:NULL] ) {

//				(void)[NSAppleScript executeWithSource:@"set tf to POSIX file %@  \r set cf to POSIX file %@ \r tell application \"BBEdit\" to apply text factory tf to  saving yes" param1:factoryPath param2:tempFilePath];
				(void)[NSAppleScript executeWithSource:@"tell application \"BBEdit\" to apply text factory POSIX file %@ to POSIX file %@ saving yes" param1:factoryPath param2:tempFilePath];

				NSString* result = [NSString stringWithContentsOfFile:tempFilePath encoding:NSUTF8StringEncoding error:NULL];
				[tempFilePath deleteFile];

				if ( result ) {
					mTarget.WriteClipboardDataSet( XClipboardDataSet( CFBridgingToCFString(result) ) );
				}
			}
		}
	}
	
	return time;
}


#endif

#if KMF_EDITOR_WINDOW

Class
AApplyTextFactory::ActionController() const
{
	return [AApplyTextFactoryEditor class];
}

NSString*
AApplyTextFactory::GetActionHTMLDescriptionString() const
{
	NSString* format = [@"kmApplyTextFactoryHTMLDescriptionTitleFormat" localized];
	return [NSString stringWithFormat:format, mTarget.GetDisplayName()];
}

static NSString* kActionDisplayHTML = @R"RAW(
Text factory: {{ file | htmlencode }}<br />
)RAW";

NSString*
AApplyTextFactory::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"file" : mPath->GetNS(),
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
AApplyTextFactory::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	FileReference::SavePathToDictionary( dictionary, mPath->GetNS(), CFSTR("Path") );
	mTarget.SaveToDictionary( dictionary, CFSTR("Target") );
}

void
AApplyTextFactory::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	FileReference::LoadPathFromDictionary( dictionary, mPath, CFSTR("NewFile"), CFSTR("Path") );
	mTarget.InitializeFromDictionary( dictionary, CFSTR("Target") );
}
