/*
 *  ACancel.h
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Wed 12 Nov 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"

// WARNING: enum valued used in menu selection in action editor
enum class CancelActionType {
	kCancelAllMacros,
	kCancelAllOtherMacros,
	kCancelThisMacro,
	kCancelJustThisMacro,
	kBreakFromLoop,
	kDEFAULT = kCancelThisMacro,
};
#define kCancelActionTypeNamesArray {CFSTR("CancelAllMacros"), CFSTR("CancelAllOtherMacros"), CFSTR("CancelThisMacro"), CFSTR("CancelJustThisMacro"), CFSTR("BreakFromLoop"), NULL}

class ACancel
	:	public HBaseAction
{
private:

	CancelActionType				mAction = CancelActionType::kDEFAULT;

public:
    ACancel() = default;
    ACancel( const CancelActionType& inActionType ) : mAction(inActionType) { }
    ACancel( const ACancel& inAction ) = default;
    virtual ~ACancel() = default;

	virtual ACancel*				Clone() const { return new ACancel( *this ); }
	virtual ACancel*				Create() const { return new ACancel(); }
	
	virtual CFStringRef				CopyActionTitle() const;

    virtual HActionType				GetActionType() const { return HActionType::kCancel; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif
	
	CancelActionType				GetCancelAction() const { return mAction; }
	void							SetCancelAction( CancelActionType inAction ) { mAction = inAction; }
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

