/*
 *  ACancel.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Wed 12 Nov 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "KMDefines.h"
#import "ACancel.h"

#if KMF_IS_ENGINE
#import "HExecuteActionListManager.h"
#import "Growl/GrowlApplicationBridge.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ACancelEditor.h"
#endif

static const CFStringRef kCancelActionTypeTypeNames[] = kCancelActionTypeNamesArray;

CFStringRef
ACancel::CopyActionTitle() const
{
	CFStringRef key = kCancelActionTypeTypeNames[int(mAction)];
	return key->CopyLocalized();
}

#if KMF_IS_ENGINE

float
ACancel::DoAction()
{
	Boolean growl = false;
	switch ( mAction ) {
		case CancelActionType::kCancelAllMacros:
			HExecuteActionListManager::CancelAllExecution();
			growl = true;
			break;
		case CancelActionType::kCancelAllOtherMacros:
			HExecuteActionListManager::CancelAllOtherExecutionGroups();
			growl = true;
			break;
		case CancelActionType::kCancelThisMacro:
			HExecuteActionListManager::CancelThisExecutionGroup();
			growl = false;
			break;
		case CancelActionType::kCancelJustThisMacro:
			HExecuteActionListManager::CancelJustThisMacro();
			growl = false;
			break;
		case CancelActionType::kBreakFromLoop:
			HExecuteActionListManager::CancelJustThisLoop();
			growl = false;
			break;
	}
	if ( growl ) {
		[GrowlApplicationBridge 
		 notifyWithTitle:CFBridgingToNS(kmEngineName) 
		 description:GetActionDescriptionString()
		 notificationName:@"Macro Cancelled"
		 iconData:nil priority:0 isSticky:NO clickContext:nil];
	}
	return 10;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ACancel::ActionController() const
{
	return [ACancelEditor class];
}

#endif

#pragma mark streaming

void
ACancel::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Action"), kCancelActionTypeTypeNames[int(mAction)] );
}

void
ACancel::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	// Not the default because the upgrade from 4.x is to default to Cancel All Macros
	mAction = dictionary->GetEnumClass( CFSTR("Action"), kCancelActionTypeTypeNames, CancelActionType::kCancelAllMacros );
}
