/*
 *  AClipboardSwitcher.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Dec 16 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "AClipboardSwitcher.h"
#import "CSClipboardDomain.h"
#import "ClipboardSwitcherWindowController.h"
#import "XClipboardDataSet.h"
#import "SystemDefinedKeys.h"
#import "KMIconRegistry.h"

#if KMF_IS_ENGINE
#import "UCGRemoteControl.h"
#import "MMacroDomain.h"
#import "KMNotifications.h"
#import "XClipboardHistory.h"
#import "KMPreferences.h"
#endif

AClipboardSwitcher::AClipboardSwitcher( ClipboardSwitcherType inType )
	:	HBaseAction(),
		mType( inType )
{
}

AClipboardSwitcher::AClipboardSwitcher( const AClipboardSwitcher& inAction )
	:	HBaseAction( inAction ),
		mType( inAction.mType )
{
}

AClipboardSwitcher::~AClipboardSwitcher()
{
}

HActionType
AClipboardSwitcher::GetActionType() const
{
	HActionType theType;
	
	switch ( mType ) {
		case ClipboardSwitcherType::kCut:
			theType = HActionType::kClipboardSwitcherCut;
			break;
		case ClipboardSwitcherType::kCopy:
			theType = HActionType::kClipboardSwitcherCopy;
			break;
		case ClipboardSwitcherType::kPaste:
			theType = HActionType::kClipboardSwitcherPaste;
			break;
	}
	
	return theType;
}


CFStringRef
AClipboardSwitcher::CopyActionTitle() const
{
	CFStringRef		theString = CFSTR("Clipboard Switcher");
	
	switch (mType) {
		case ClipboardSwitcherType::kCut:
			theString = CFSTR("hkClipboardSwitcherCut")->CopyLocalized();
			break;
		case ClipboardSwitcherType::kCopy:
			theString = CFSTR("hkClipboardSwitcherCopy")->CopyLocalized();
			break;
		case ClipboardSwitcherType::kPaste:
			theString = CFSTR("hkClipboardSwitcherPaste")->CopyLocalized();
			break;
	}
	
	return theString;
}

#if KMF_IS_ENGINE

float
AClipboardSwitcher::DoAction()
{
#if KMF_HAS_CLIPBOARD_SWITCHERS
	ClipboardNamedSwitcherToggleOpen( mType );
#endif
	return 10;
}

#endif

#pragma mark streaming

void
AClipboardSwitcher::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
}

void
AClipboardSwitcher::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
}
