/*
 *  AClipboardSwitcherMacroAction.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Apr 27 2003.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "ClipboardReference.h"
#if KMF_IS_ENGINE
#import "XClipboardDataSet.h"
#endif

enum class CSMacroActionType {
	kCut,
	kCopy,
	kPaste,
	kDEFAULT = kPaste,
};
#define kCSMacroActionTypeNamesArray {CFSTR("Cut"), CFSTR("Copy"), CFSTR("Paste"), NULL}

class AClipboardSwitcherMacroAction
	:	public HBaseAction
{
private:

	CSMacroActionType				mAction = CSMacroActionType::kDEFAULT;
	ClipboardReference				mClipboardReference;
	
#if KMF_IS_ENGINE
	XClipboardDataSet*				mOldScrap;
	EventTime						mActionTime;
	int								mCopyPasteState;
#endif

public:

    AClipboardSwitcherMacroAction() = default;
	AClipboardSwitcherMacroAction( const AClipboardSwitcherMacroAction& inAction ) = default;
	AClipboardSwitcherMacroAction( CSMacroActionType inMacroAction );
    virtual ~AClipboardSwitcherMacroAction() = default;
	
	virtual AClipboardSwitcherMacroAction*		Clone() const { return new AClipboardSwitcherMacroAction( *this ); }
	virtual AClipboardSwitcherMacroAction*		Create() const { return new AClipboardSwitcherMacroAction(); }
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mClipboardReference.UpdateClipboardReference(); }
#endif
	
    virtual HActionType				GetActionType() const { return HActionType::kClipboardSwitcher; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual NSString*				GetNewActionTitle() const;

	NSString*						GetClipboardUID() const { return mClipboardReference.GetClipboardUID(); }
	void							SetClipboardUID( NSString* inUID ) { mClipboardReference.SetClipboardUID( inUID ); }
	
	CSMacroActionType				GetAction() const;
	void							SetAction(const CSMacroActionType& inAction );
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

