/*
 *  AComment.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Tue 1 Jul 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "AComment.h"
#if KMF_EDITOR_WINDOW
#import "ACommentEditor.h"
#import "NSObjectExtensions.h"
#import "NSStringExtensions.h"
#endif

AComment::AComment()
	:	HBaseAction()
{
	mTitle = CFSTR("kCommentDefaultTitle")->CopyLocalized();
	mText = CFSTR("kCommentDefaultText")->CopyLocalized();
}

AComment::AComment( const AComment& inAction )
	:	HBaseAction( inAction ),
		mTitle( inAction.mTitle ),
		mText( inAction.mText )
{
}

AComment::~AComment()
{
}

CFStringRef
AComment::CopyActionDescriptionString() const
{
	return CFString::CreateWithFormat( CFCS( CFSTR("hkCommentTitle")->CopyLocalized() ), CFStringRef(mTitle) );
}

#if KMF_EDITOR_WINDOW

Class
AComment::ActionController() const
{
	return [ACommentEditor class];
}

NSString*
AComment::GetActionExtraHTML() const
{
	return UserTextForHTML( mText->GetNS() );
}

#endif

#pragma mark streaming

void
AComment::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Title"), CFStringRef(mTitle) );
	dictionary->Set( CFSTR("Text"), CFStringRef(mText) );
}

void
AComment::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef newTitle;
	if ( dictionary->Get( CFSTR("Title"), newTitle ) ) {
		mTitle = newTitle->Copy();
	}

	CFStringRef newText;
	XAssert( dictionary->Get( CFSTR("Text"), newText ) );
	mText = newText->Copy();
}
