//
//  ACompositeImage.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 10 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ClipboardTarget.h"
#import "StyledTextSource.h"

class ACompositeImage
	:	public HBaseAction
{
private:

	ClipboardTarget					mTarget;
    CFContainer<CFStringRef>		mLeft;
    CFContainer<CFStringRef>		mTop;
    CFContainer<CFStringRef>		mWidth; // Optional
    CFContainer<CFStringRef>		mHeight; // Optional
	StyledTextSource				mStyledText;
	ClipboardTarget					mSource;
	Boolean							mUseText = false;

protected:

public:

    ACompositeImage();
	ACompositeImage( const ACompositeImage& inAction ) = default;
    virtual ~ACompositeImage() = default;

	virtual ACompositeImage*		Clone() const { return new ACompositeImage( *this ); }
	virtual ACompositeImage*		Create() const { return new ACompositeImage(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kCompositeImage; }
	
	virtual CFStringRef				CopyActionDescriptionString() const;

#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); mSource.UpdateToDefaultValues(); }
#endif

	NSString*						GetGenericTargetNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericTargetNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }
	
	void							SetLeft( CFStringRef inString ) { mLeft = inString->Copy(); }
	void							SetLeft( double inValue ) { mLeft = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyLeft() const { return mLeft->Copy(); }
	NSString*						GetLeft() const { return mLeft->GetNS(); }
	
	void							SetTop( CFStringRef inString ) { mTop = inString->Copy(); }
	void							SetTop( double inValue ) { mTop = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyTop() const { return mTop->Copy(); }
	NSString*						GetTop() const { return mTop->GetNS(); }
	
	void							SetWidth( CFStringRef inString ) { mWidth = inString->Copy(); }
	void							SetWidth( double inValue ) { mWidth = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyWidth() const { return mWidth->Copy(); }
	NSString*						GetWidth() const { return mWidth->GetNS(); }
	
	void							SetHeight( CFStringRef inString ) { mHeight = inString->Copy(); }
	void							SetHeight( double inValue ) { mHeight = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyHeight() const { return mHeight->Copy(); }
	NSString*						GetHeight() const { return mHeight->GetNS(); }

	Boolean							GetUseText() { return mUseText; }
	void							SetUseText( Boolean inUseText ) { mUseText = inUseText; }

	void							SetPlainText( NSString* inString ) { mStyledText.SetPlainText( inString ); }
	void							SetStyledText( NSAttributedString* inString ) { mStyledText.SetStyledText( inString ); }

	NSString*						GetSourceString() const { return mStyledText.GetString(); }
	NSAttributedString*				GetSourceAttributedString() const { return mStyledText.GetAttributedString(); }
	
	NSString*						GetGenericSourceNamedClipboardUID() const { return mSource.GetGenericNamedClipboardUID(); }
	void							SetGenericSourceNamedClipboardUID( NSString* inUID ) { mSource.SetGenericNamedClipboardUID( inUID ); }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;

	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

