//
//  ACopyClipboard.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 10 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "ACopyClipboard.h"
#import "XException.h"
#import "KMDefines.h"
#import "XErrorHandler.h"
#import "NSStringRegularExpressions.h"
#import "Calculate.h"
#import "TokenText.h"
#import "CSClipboardDomain.h"
#import "NSStringExtensions.h"
#import "NSImageExtensions.h"

#if KMF_EDITOR_WINDOW
#import "ACopyClipboardEditor.h"
#endif
#if KMF_IS_ENGINE
#import "KMNotifications.h"
#import "XClipboardDataSet.h"
#import "XClipboardHistory.h"
#endif

static CFStringRef kDefaultLeft = CFSTR("100");
static CFStringRef kDefaultTop = CFSTR("100");
static CFStringRef kDefaultWidth = CFSTR("0");
static CFStringRef kDefaultHeight = CFSTR("0");

ACopyClipboard::ACopyClipboard()
	:	HBaseAction()
{
	CFContainer<CFStringRef> name = CFSTR("DefaultClipboardName")->CopyLocalized();
	CSClipboardDataSet* clip = CSClipboardDomain::GetClipboardDomain()->GetClipDataSetByDisplayName( name );
	if ( !clip && CSClipboardDomain::GetClipboardDomain()->ClipListSize() > 0 ) {
		clip = CSClipboardDomain::GetClipboardDomain()->GetClipDataSet( 0 );
	}
	if ( clip ) {
		mSource.SetClipboardUID( clip->GetUID() );
	}
}

CFStringRef
ACopyClipboard::CopyActionDescriptionString() const
{
	if ( mSource == mTarget ) {
		return CFSTR("aCopyClipboardNothing")->CopyLocalized();
	}
	CFContainer<CFStringRef> format = CFSTR("aCopyClipboard")->CopyLocalized();
	return CFString::CreateWithFormat( format, mSource.GetDisplayName(), mTarget.GetDisplayName() );
}

#if KMF_IS_ENGINE

float
ACopyClipboard::DoAction()
{	
	const XClipboardDataSet* sourceclip = mSource.GetClipboardDataSet();
	if ( sourceclip ) {
		mTarget.WriteClipboardDataSet( *sourceclip );
	}	
	return 10;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ACopyClipboard::ActionController() const
{
	return [ACopyClipboardEditor class];
}

#endif

#pragma mark streaming

void
ACopyClipboard::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	mSource.SaveToDictionary( dictionary, CFSTR("Source") );
	mTarget.SaveToDictionary( dictionary, CFSTR("Target") );
}

void
ACopyClipboard::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mSource.InitializeFromDictionary( dictionary, CFSTR("Source") );
	mTarget.InitializeFromDictionary( dictionary, CFSTR("Target") );
}
