//
//  ADebuggerAction.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 12 Mar 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"

enum class DebuggerActionType {
	FIRSTENUM(kStart),
	kFinish,
	kToggle,
	kNewMacrosPaused,
	kNewMacrosRun,
	kBreakpoint,
	kBreakpointOthers,
	kBreakpointEvery,
	kStepOverOthers,
	kStepIntoOthers,
	kStepOutOthers,
	kContinue,
	LASTENUM(kContinueEvery),
	kDEFAULT = kStart,
};
// Warning: Also in Localizable.strings
#define kDebuggerActionNamesArray {CFSTR("Start"), CFSTR("Finish"), CFSTR("Toggle"), CFSTR("NewMacrosPaused"), CFSTR("NewMacrosRun"), CFSTR("Breakpoint"), CFSTR("BreakpointOthers"), CFSTR("BreakpointEvery"), CFSTR("StepOverOthers"), CFSTR("StepIntoOthers"), CFSTR("StepOutOthers"), CFSTR("Continue"), CFSTR("ContinueEvery"), NULL}

class ADebuggerAction
	:	public HBaseAction
{
private:

	DebuggerActionType				mDebuggerAction = DebuggerActionType::kDEFAULT;

public:

    ADebuggerAction() = default;
	ADebuggerAction( const ADebuggerAction& inAction ) = default;
	ADebuggerAction( const DebuggerActionType& inActionType ) : mDebuggerAction(inActionType) { }
	virtual ~ADebuggerAction() = default;

	virtual ADebuggerAction*		Clone() const { return new ADebuggerAction( *this ); }
	virtual ADebuggerAction*		Create() const { return new ADebuggerAction(); }
	
	virtual	CFStringRef				CopyActionTitle() const;
	virtual	XIcon					GetActionDescriptionXIcon() const;

	virtual NSString*				GetNewActionTitle() const { return NSCS(CopyActionDescriptionString()); }

    virtual HActionType				GetActionType() const { return HActionType::kDebuggerAction; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	void							SetDebuggerAction( DebuggerActionType inValue ) { mDebuggerAction = inValue; }
	DebuggerActionType				GetDebuggerAction() const { return mDebuggerAction; }
};

