//
//  ADebuggerAction.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 12 Mar 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "ADebuggerAction.h"
#import "NSStringExtensions.h"
#import "KMIconRegistry.h"
#import "XIcon.h"
#if KMF_IS_ENGINE
#import "HExecuteActionListManager.h"
#import "MacroDebugger.h"
#endif

static const CFStringRef kDebuggerActionNames[] = kDebuggerActionNamesArray;

CFStringRef
ADebuggerAction::CopyActionTitle() const
{
	NSString* theTitleKey = [NSString stringWithFormat:@"hkDebuggerAction%@Title", kDebuggerActionNames[int(mDebuggerAction)]];
	return (CFStringRef)CFBridgingRetain([theTitleKey localized]);
}

XIcon
ADebuggerAction::GetActionDescriptionXIcon() const
{
	switch ( GetDebuggerAction() ) {
		case DebuggerActionType::kStart:
			return XIcon( KMIconRegistry::kDebuggerContinue );;
		case DebuggerActionType::kFinish:
			return XIcon( KMIconRegistry::kDebuggerContinue );;
		case DebuggerActionType::kToggle:
			return XIcon( KMIconRegistry::kDebuggerContinue );;
		case DebuggerActionType::kNewMacrosPaused:
			return XIcon( KMIconRegistry::kDebuggerPause );;
		case DebuggerActionType::kNewMacrosRun:
			return XIcon( KMIconRegistry::kDebuggerContinue );;
		case DebuggerActionType::kBreakpoint:
			return XIcon( KMIconRegistry::kDebuggerPause );;
		case DebuggerActionType::kBreakpointOthers:
			return XIcon( KMIconRegistry::kDebuggerPause );;
		case DebuggerActionType::kBreakpointEvery:
			return XIcon( KMIconRegistry::kDebuggerPause );;
		case DebuggerActionType::kContinue:
			return XIcon( KMIconRegistry::kDebuggerContinue );;
		case DebuggerActionType::kStepOverOthers:
			return XIcon( KMIconRegistry::kDebuggerStepOver );;
		case DebuggerActionType::kStepIntoOthers:
			return XIcon( KMIconRegistry::kDebuggerStepInto );;
		case DebuggerActionType::kStepOutOthers:
			return XIcon( KMIconRegistry::kDebuggerStepOut );;
		case DebuggerActionType::kContinueEvery:
			return XIcon( KMIconRegistry::kDebuggerContinue );;
	}
}

#if KMF_IS_ENGINE

float
ADebuggerAction::DoAction()
{
	float time = 0;
	switch ( GetDebuggerAction() ) {
		case DebuggerActionType::kStart:
			MacroDebugger::StartDebugging();
			break;
		case DebuggerActionType::kFinish:
			MacroDebugger::StopDebugging();
			break;
		case DebuggerActionType::kToggle:
			MacroDebugger::ToggleDebugging();
			break;
		case DebuggerActionType::kNewMacrosPaused:
			MacroDebugger::SetNewMacrosPaused( true );
			break;
		case DebuggerActionType::kNewMacrosRun:
			MacroDebugger::SetNewMacrosPaused( false );
			break;
		case DebuggerActionType::kBreakpoint:
			MacroDebugger::StartDebuggingThis( HExecuteActionListManager::GetThisExecutionGroup() );
			break;
		case DebuggerActionType::kBreakpointOthers:
			MacroDebugger::StartDebuggingOthers( HExecuteActionListManager::GetThisExecutionGroup() );
			break;
		case DebuggerActionType::kBreakpointEvery:
			MacroDebugger::StartDebuggingEvery();
			break;
		case DebuggerActionType::kContinue:
			HExecuteActionListManager::SetExecutionDebugState( HExecuteActionListManager::GetThisExecutionGroup(), ExecutingEntryDebugState::kRunning );
			break;
		case DebuggerActionType::kStepOverOthers:
			HExecuteActionListManager::SetExecutionDebugStateAll( ExecutingEntryDebugState::kStepOver, HExecuteActionListManager::GetThisExecutionGroup() );
			break;
		case DebuggerActionType::kStepIntoOthers:
			HExecuteActionListManager::SetExecutionDebugStateAll( ExecutingEntryDebugState::kStepInto, HExecuteActionListManager::GetThisExecutionGroup() );
			break;
		case DebuggerActionType::kStepOutOthers:
			HExecuteActionListManager::SetExecutionDebugStateAll( ExecutingEntryDebugState::kStepOut, HExecuteActionListManager::GetThisExecutionGroup() );
			break;
		case DebuggerActionType::kContinueEvery:
			HExecuteActionListManager::SetExecutionPausedAll( false );
			break;
	}
	return time;
}

#endif

#pragma mark streaming

void
ADebuggerAction::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("DebuggerAction"), kDebuggerActionNames[int(mDebuggerAction)] );
}

void
ADebuggerAction::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mDebuggerAction = dictionary->GetEnumClass( CFSTR("DebuggerAction"), kDebuggerActionNames, DebuggerActionType::kDEFAULT );
}

