//
//  ADeletePastClipboard.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 1 Apr 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "ADeletePastClipboard.h"
#import "CSClipboardDomain.h"
#import "XClipboardHistory.h"
#import "Calculate.h"
#if KMF_EDITOR_WINDOW
#import "ADeletePastClipboardEditor.h"
#endif
#if KMF_IS_ENGINE
#import "XErrorHandler.h"
#endif

const CFStringRef kDefaultPastClipboard = CFSTR("1");

ADeletePastClipboard::ADeletePastClipboard()
	:	HBaseAction(),
		mPastClipboard( kDefaultPastClipboard )
{
}

ADeletePastClipboard::ADeletePastClipboard( const ADeletePastClipboard& inAction )
	:	HBaseAction( inAction ),
		mPastClipboard( inAction.mPastClipboard )
{
}

ADeletePastClipboard::~ADeletePastClipboard()
{
}

CFStringRef
ADeletePastClipboard::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> titleFormat = CFSTR("aDeletePastClipboardFormat")->CopyLocalized();
	return CFString::CreateWithFormat( titleFormat, mPastClipboard->GetNS() );
}

#if KMF_IS_ENGINE

float
ADeletePastClipboard::DoAction()
{
	float time = 0;
	
	double doubleindex;
	if ( !Calculate::CalculateValue( mPastClipboard->GetNS(), doubleindex ) ) {
		XErrorHandlerMaybeLogFormat1("always", CFSTR("Delete Past Clipboard %@ failed"), mPastClipboard.Get() );
		doubleindex = -1;
	}

	SInt32 index = lround(doubleindex);
	SInt32 count = XClipboardHistory::Count();
	if ( 0 <= index && index < count ) {
		XClipboardHistory::Delete( index );
	}
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ADeletePastClipboard::ActionController() const
{
	return [ADeletePastClipboardEditor class];
}

#endif

#pragma mark streaming

void
ADeletePastClipboard::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("PastExpression"), CFStringRef(mPastClipboard) );
}

void
ADeletePastClipboard::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("PastExpression"), value, CFStringRef(NULL) );
	mPastClipboard = value ? value->Copy() : NULL;
	
	SInt32 index;
	if ( !mPastClipboard && dictionary->Get( CFSTR("Past"), index ) ) {
		mPastClipboard = CFString::CreateWithNumber( index );
	}
	
	if ( !mPastClipboard ) {
		mPastClipboard = kDefaultPastClipboard;
	}
}
