//
//  ADrawImage.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 10 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ClipboardTarget.h"
#import "ArrowStrings.h"

typedef enum { // WARNING: tag valued in ADrawImage.xib match this enum
	kDrawImageActionLine,
	kDrawImageActionRectangle,
	kDrawImageActionOval,
	kDrawImageActionDEFAULT = kDrawImageActionLine,
} DrawImageActionType;
#define kDrawImageActionTypeNamesArray {CFSTR("Line"), CFSTR("Rectangle"), CFSTR("Oval"), NULL}

class ADrawImage
	:	public HBaseAction
{
private:

	DrawImageActionType				mDrawAction;
	ClipboardTarget					mTarget;
    CFContainer<CFStringRef>		mLeft;
    CFContainer<CFStringRef>		mTop;
    CFContainer<CFStringRef>		mWidth; // or left2
    CFContainer<CFStringRef>		mHeight; // or top2
	NSColor*						mFillColor;
	NSColor*						mStrokeColor;
    CFContainer<CFStringRef>		mThickness;
    CFContainer<CFStringRef>		mCorner;

	virtual	CFStringRef				CopyActionDescriptionShortString() const;

protected:

public:

    ADrawImage();
	ADrawImage( const ADrawImage& inAction ) = default;
    virtual ~ADrawImage() = default;

	virtual ADrawImage*				Clone() const { return new ADrawImage( *this ); }
	virtual ADrawImage*				Create() const { return new ADrawImage(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kDrawImage; }
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); }
#endif
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

	DrawImageActionType				GetDrawAction() const { return mDrawAction; }
	void							SetDrawAction( DrawImageActionType inDrawAction ) { mDrawAction = inDrawAction; }
	
	const ArrowString::Style*		GetArrowStyles() const; // changes whenever DrawAction changes

	NSString*						GetGenericTargetNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericTargetNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }
	
	void							SetLeft( CFStringRef inString ) { mLeft = inString->Copy(); }
	void							SetLeft( double inValue ) { mLeft = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyLeft() const { return mLeft->Copy(); }
	NSString*						GetLeft() const { return mLeft->GetNS(); }
	
	void							SetTop( CFStringRef inString ) { mTop = inString->Copy(); }
	void							SetTop( double inValue ) { mTop = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyTop() const { return mTop->Copy(); }
	NSString*						GetTop() const { return mTop->GetNS(); }
	
	void							SetWidth( CFStringRef inString ) { mWidth = inString->Copy(); }
	void							SetWidth( double inValue ) { mWidth = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyWidth() const { return mWidth->Copy(); }
	NSString*						GetWidth() const { return mWidth->GetNS(); }
	
	void							SetHeight( CFStringRef inString ) { mHeight = inString->Copy(); }
	void							SetHeight( double inValue ) { mHeight = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyHeight() const { return mHeight->Copy(); }
	NSString*						GetHeight() const { return mHeight->GetNS(); }
	
	NSColor*						GetFillColor() const { return mFillColor; }
	void							SetFillColor( NSColor* inFillColor );
	
	NSColor*						GetStrokeColor() const { return mStrokeColor; }
	void							SetStrokeColor( NSColor* inStrokeColor );
	
	void							SetThickness( CFStringRef inString ) { mThickness = inString->Copy(); }
	void							SetThickness( double inValue ) { mThickness = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyThickness() const { return mThickness->Copy(); }
	NSString*						GetThickness() const { return mThickness->GetNS(); }
	
	void							SetCorner( CFStringRef inString ) { mCorner = inString->Copy(); }
	void							SetCorner( double inValue ) { mCorner = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyCorner() const { return mCorner->Copy(); }
	NSString*						GetCorner() const { return mCorner->GetNS(); }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

