//
//  AExecuteBaseScript.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 30 Apr 2010.
//  Copyright (c) 2010 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "MTextResultsWindow.h"
#import "KMVariables.h"
#import "ClipboardTarget.h"
#import "AExecuteRootScript.h"
#import "UncopyablePointer.h"

#if KMF_IS_ENGINE
#import "TempScriptHolder.h"
#endif

class AExecuteBaseScript
	:	public AExecuteRootScript
{
private:

	MTextResultsWindowKind			mDefaultDisplayKind;

	Boolean							mUseScriptText = true;
	NSString*						mScriptText = @"";
	NSString*						mScriptPath = @"";
	MTextResultsWindowKind			mDisplayKind = mDefaultDisplayKind;
	VariableContainer				mVariable = @"Variable";
	ClipboardTarget					mClipboardTarget;
	Boolean							mTrimResults = true;

#if KMF_IS_ENGINE
	UncopyablePointer<TempScriptHolder> mTempScriptHolder;
#endif
	
	AExecuteBaseScript() = delete;

protected:
	
#if KMF_IS_ENGINE
	virtual NSString*				GetScriptDisplayName() const { return mTempScriptHolder.Defined() ? mTempScriptHolder->GetDisplayName() : @"Unknown"; }
	virtual NSString*				GetLongScriptDisplayName() const;

	virtual NSString*				GetShellCommand( NSString* path ) = PURE_VIRTUAL;
	virtual NSString*				GetShellCommand();

	virtual TempScriptHolder*		CreateTempScriptHolderForText( NSString* text ) { return new TempScriptHolder( TempScriptHolder::kScript, text ); }
	virtual TempScriptHolder*		CreateTempScriptHolderForPath( NSString* path ) { return new TempScriptHolder( TempScriptHolder::kPath, path ); }

	virtual Boolean					TrimResults() { return mTrimResults; }
	virtual MTextResultsWindowKind	DisplayKind() { return mDisplayKind; }
	virtual NSString*				SaveToVariable() { return mVariable->GetNS(); }
	virtual void					WriteClipboardDataSet( const XClipboardDataSet& dataset ) { mClipboardTarget.WriteClipboardDataSet( dataset ); }

#endif
	
public:

	AExecuteBaseScript( MTextResultsWindowKind defaultDisplayKind ) : mDefaultDisplayKind( defaultDisplayKind ) { ; }
	AExecuteBaseScript( const AExecuteBaseScript& inAction ) = default;
	virtual ~AExecuteBaseScript() = default;

	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual	XIcon					GetActionDescriptionXIcon() const;

	virtual Boolean					UseScriptText() const { return mUseScriptText; }
	virtual void					SetUseScriptText( Boolean inUseScriptText ) { mUseScriptText = inUseScriptText; }
		// Warning: SetFileRef & SetScriptText also set mUseScriptText so set it last!

	virtual NSString*				GetScriptText() const { check( mScriptText ); return mScriptText; }
	virtual void					SetScriptText( NSString* inScriptText ) { check(inScriptText); mScriptText = [inScriptText copy]; mUseScriptText = true; }
	
	virtual NSString*				GetVariable() const { check( mVariable ); return mVariable->GetNS(); }
	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }

	NSString*						GetGenericNamedClipboardUID() const { return mClipboardTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mClipboardTarget.SetGenericNamedClipboardUID( inUID ); }

	virtual Boolean					GetTrimResults() const { return mTrimResults; }
	virtual void					SetTrimResults( Boolean inTrimResults ) { mTrimResults = inTrimResults; }

	NSString*						GetScriptPath() const { return mScriptPath; }
	void							SetScriptPath( NSString* path ) { check(path); mScriptPath = [path copy]; mUseScriptText = false; }

    virtual MTextResultsWindowKind	GetDisplayKind() const { return mDisplayKind; }
	virtual void					SetDisplayKind( MTextResultsWindowKind inDisplayKind ) { mDisplayKind = inDisplayKind; }

#if KMF_EDITOR_WINDOW
//	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;

	virtual	NSString*				GetActionExtraResults() const;
	
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

