//
//  AExecuteJavaScript.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Mon 5 Nov 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "AExecuteJavaScript.h"
#import "KMPreferences.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "TempScriptHolder.h"
#import "XErrorHandler.h"
#import "NSAppleScriptExtensions.h"
#endif

#if KMF_EDITOR_WINDOW
#import "AExecuteScriptEditor.h"
#endif

static const MTextResultsWindowKind kDefaultDisplayKind = MTextResultsWindowKind::kNone;

NSString*
AExecuteJavaScript::GetScriptTypeName() const
{
	static NSString* kScriptTypeNameFormat = [@"kmExecuteJavaScriptType" localized];
	return [NSString stringWithFormat:kScriptTypeNameFormat, mWebBrowserInterface->GetName()];
}

AExecuteJavaScript::AExecuteJavaScript()
	:	AExecuteBaseScript( kDefaultDisplayKind )
{
}

AExecuteJavaScript::AExecuteJavaScript( WebBrowserInterface* interface )
	:	AExecuteBaseScript( kDefaultDisplayKind ),
		mWebBrowserInterface( interface )
{
}

NSString*
AExecuteJavaScript::GetNewActionTitle() const
{
	return [NSString stringWithFormat:[@"aExecuteJavaScript_NewActionFormat" localized], mWebBrowserInterface->GetName()];
}

#if KMF_IS_ENGINE

TempScriptHolder*
AExecuteJavaScript::CreateTempScriptHolderForText( NSString* javascript )
{
	NSString* script = mWebBrowserInterface->GetDoJavascriptAppleScript( javascript, DoJavascriptNotRunning::kNotRunning );
	XErrorHandlerMaybeLogFormat1( "JavaScript", @"script is [%@]", script );
	return new TempScriptHolder( TempScriptHolder::kScript, script );
}

TempScriptHolder*
AExecuteJavaScript::CreateTempScriptHolderForPath( NSString* path )
{
	NSError* error;
	NSString* text = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:&error];
	if ( !text ) {
		XErrorHandlerMaybeLogFormat1("always", @"Execute JavaScript could not read file %@", error );
		return NULL;
	}
	return CreateTempScriptHolderForText( text );
}

NSString*
AExecuteJavaScript::GetShellCommand( NSString* path )
{
	CFContainer<CFStringRef> osascript;
	KMPreferences::Copy( KMPreferences::kOSAScriptCommandID, osascript, CFSTR("/usr/bin/osascript") );
	return [NSString stringWithFormat:@"%@ %@", osascript->GetNS(), [path shellQuotedPath]];
}

#endif

#if KMF_EDITOR_WINDOW

Class
AExecuteJavaScript::ActionController() const
{
	return [AExecuteScriptEditor class];
}

#endif

#pragma mark streaming

void
AExecuteJavaScript::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	AExecuteBaseScript::SaveToDictionary( dictionary );
	mWebBrowserInterface->SaveToDictionary( dictionary );
}

void
AExecuteJavaScript::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	AExecuteBaseScript::InitializeFromDictionary( dictionary );
	mWebBrowserInterface = WebBrowserInterface::InitializeFromDictionary( dictionary );
}
