//
//  AExecuteMacro.h"
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 19 Mar 2010.
//  Copyright (c) 2010 Stairways Software. All rights reserved.
//

#import "AExecuteMacro.h"
#import "MMacroDomain.h"
#import "MMacro.h"
#import "ARepeat.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "HExecuteActionListManager.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AExecuteMacroEditor.h"
#endif

AExecuteMacro::~AExecuteMacro()
{
#if KMF_IS_ENGINE
	HExecuteActionListManager::CancelExecution( mTrackingID );
#endif
}

CFStringRef
AExecuteMacro::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> result;
	CFContainer<CFStringRef> format = CFSTR("kmExecuteMacroTitleFormat")->CopyLocalized();
	MMacro* macro = MMacroSerializedObject::FindMacroByUID( mMacroUID );
	if ( macro ) {
		result = CFString::CreateWithFormat( format, macro->GetName() );
	} else {
		result = CopyActionTitle();
	}

	return result->Copy();
}

XIcon
AExecuteMacro::GetActionDescriptionXIcon() const
{
	XIcon result;
	MMacro* macro = MMacroSerializedObject::FindMacroByUID( mMacroUID );
	if ( macro ) {
		result = macro->GetXIcon();
	} else {
		result = HBaseAction::GetActionDescriptionXIcon();
	}
	return result;
}

#if KMF_IS_ENGINE

float
AExecuteMacro::DoAction()
{
	mTrackingID = [NSUUID UUID];
	check( mCount == 1 );

	MMacro* macro = MMacroSerializedObject::FindMacroByUID( mMacroUID );
	if ( macro ) {
		macro->DoActionList( ExecuteActionListDisplay::kHide, NULL, CFCS(CFSTR("TriggerByExecuteMacro")->CopyLocalized()), mTrackingID );
	}

	return 60;
}

Boolean
AExecuteMacro::IsActionCompleted()
{
	return !HExecuteActionListManager::IsExecuting( mTrackingID );
}

void
AExecuteMacro::ActionAborted()
{
	HExecuteActionListManager::CancelExecution( mTrackingID );
}

#endif

#if KMF_EDITOR_WINDOW

Class
AExecuteMacro::ActionController() const
{
	return [AExecuteMacroEditor class];
}

#endif

#pragma mark streaming

void
AExecuteMacro::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("MacroUID"), CFBridgingToCFString(mMacroUID) );
	
	// Obsolete
	check( mCount == 1 );
	dictionary->SetNumber( CFSTR("Count"), mCount );
}

void
AExecuteMacro::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef macroUID;
	XAssert( dictionary->Get( CFSTR("MacroUID"), macroUID ) );
	mMacroUID = macroUID->GetNS();

	// Obsolete
	if ( !dictionary->Get( CFSTR("Count"), mCount ) ) {
		mCount = 1;
	}
}

HBaseAction*
AExecuteMacro::CleanActionAfterInitialization()
{
	if ( mCount != 1 ) {
		ARepeat* repeat = new ARepeat();
		repeat->SetCount( CFBridgingToCFString([NSString stringWithLongInt:mCount]) );
		mCount = 1;
		HActionList actions;
		actions.AppendAction( Clone() );
		repeat->SetActionList( actions );
		delete this;
		return repeat;
	} else {
		return this;
	}
}
