//
//  AExecuteRootScript.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 30 Apr 2010.
//  Copyright (c) 2010 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "MTextResultsWindow.h"
#import "KMVariables.h"
#import "ClipboardTarget.h"

#if KMF_IS_ENGINE
#import "AsyncNSTask.h"
#import "TempScriptHolder.h"
class InsertText;
#endif

class AExecuteRootScript
	:	public HBaseAction
#if KMF_IS_ENGINE
		, public AsyncNSTaskInterface
#endif
{
private:

#if KMF_IS_ENGINE
	AsyncNSTask*					mTask = nil;
	CFContainer<CFDataRef>			mTheResultData;

	InsertText*						mInsertText = nil;
	
	void							ProcessResults();
#endif


protected:

	virtual NSString*				GetScriptTypeName() const = PURE_VIRTUAL;

#if KMF_IS_ENGINE
	virtual void					DisplayResults( NSString* results ); // Override to capture the results if desired

	virtual NSString*				GetLongScriptDisplayName() const = PURE_VIRTUAL;
			// Should return something like "script ’Script’" or "the text script"

	virtual NSString*				GetShellCommand() = PURE_VIRTUAL;
	virtual Boolean					StripTrailingLineFeed() = PURE_VIRTUAL;
	virtual Boolean					TrimResults() = PURE_VIRTUAL;

	virtual MTextResultsWindowKind	DisplayKind() = PURE_VIRTUAL;
	virtual NSString*				SaveToVariable() { return @""; } // override if DisplayKind() can be MTextResultsWindowKind::kSaveToVariable
	virtual void					WriteClipboardDataSet( const XClipboardDataSet& dataset ) { ; } // override if DisplayKind() can be MTextResultsWindowKind::kSaveToClipboard

	virtual NSDictionary*			GetExecutionParameters() { return nil; }
#endif
	
public:

	AExecuteRootScript() = default;
	AExecuteRootScript( const AExecuteRootScript& inAction );
	virtual ~AExecuteRootScript();

	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return kVeryLongDefaultTimeOutValue; }
	
#if KMF_IS_ENGINE
	virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
	
	virtual void					FinishedWithData( AsyncNSTask* task, NSData* data, NSData* error );
#endif

};

