/*
 *  AExecuteWorkflow.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Sun 17 Feb 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "AExecuteWorkflow.h"
#import "FileReference.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "XErrorHandler.h"
#import "KMPreferences.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AExecuteWorkflowEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
AExecuteWorkflow::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> theNameStr = (CFStringRef)CFBridgingRetain(FileReference::PathToName( mPath ));
	CFContainer<CFStringRef> theFormatStr = CFSTR("kmExecuteWorkflowTitleFormat")->CopyLocalized();
	return CFString::CreateWithFormat( theFormatStr, CFStringRef(theNameStr) );
}

#if KMF_IS_ENGINE

NSString*
AExecuteWorkflow::GetShellCommand()
{
	NSString* command = nil;
	NSString* path = FileReference::PathToActualPath( mPath );
	if ( path ) {
		CFContainer<CFStringRef> automator;
		KMPreferences::Copy( KMPreferences::kAutomatorCommandID, automator, CFSTR("/usr/bin/automator") );
		command = [NSString stringWithFormat:@"%@ %@", automator.Get(), [path shellQuotedPath]];
	}
	return command;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AExecuteWorkflow::ActionController() const
{
	return [AExecuteWorkflowEditor class];
}

NSString*
AExecuteWorkflow::GetActionHTMLDescriptionString() const
{
	return GetActionTitle();
}

static NSString* kActionDisplayHTML = @R"RAW(
 {% if path %}Path: {{ path | htmlencode }}<br />{% /if %}
)RAW";

NSString*
AExecuteWorkflow::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
							@"path" : mPath,
						};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
AExecuteWorkflow::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	FileReference::SavePathToDictionary( dictionary, mPath, CFSTR("Path") );
}

void
AExecuteWorkflow::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mPath = FileReference::LoadPathFromDictionary( dictionary, CFSTR("NewFile"), CFSTR("Path") );
}
