/*
 *  AFastUserSwitch.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Thu 10 Jul 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "AFastUserSwitch.h"
#import "KMDefines.h"

#if KMF_IS_ENGINE
#import "NSAppleScriptExtensions.h"
#endif

#if KMF_EDITOR_WINDOW
#import "AFastUserSwitchEditor.h"
#endif

AFastUserSwitch::AFastUserSwitch()
	:	HBaseAction(),
		mUserName( CFSTR("")->Copy() )
{
}

AFastUserSwitch::AFastUserSwitch( const AFastUserSwitch& inAction )
	:	HBaseAction( inAction ),
		mUserName( inAction.mUserName )
{
}

AFastUserSwitch::~AFastUserSwitch()
{
}

CFStringRef
AFastUserSwitch::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> description;

	if ( !mUserName->IsEmpty() ) {
		CFContainer<CFStringRef> format = CFSTR("aFastUserSwitch_DescriptionFormat")->CopyLocalized();
		description = CFString::CreateWithFormat( format, CFStringRef(mUserName) );
	} else {
		description = CopyActionTitle();
	}
	return description->Copy();
}

#if KMF_IS_ENGINE

float
AFastUserSwitch::DoAction()
{
	(void)[NSAppleScript executeWithSource:
@"set n to %@" "\n"
"tell application \"System Events\"" "\n"
"	if n is not full name of current user then" "\n"
"		tell the front menu bar of process \"SystemUIServer\"" "\n"
"			set menuExtras to value of attribute \"AXDescription\" of menu bar items" "\n"
"			repeat with mExtras from 1 to the length of menuExtras" "\n"
"				if item mExtras of menuExtras is \"user\" then exit repeat" "\n"
"			end repeat" "\n"
"			tell menu bar item mExtras" "\n"
"				click" "\n"
"				tell menu item n of front menu to click" "\n"
"			end tell" "\n"
"		end tell" "\n"
"	else" "\n"
"		\"Cant switch to current user\"" "\n"
"	end if" "\n"
"end tell" "\n"
									 param:mUserName->GetNS()];
	return 8;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AFastUserSwitch::ActionController() const
{
	return [AFastUserSwitchEditor class];
}

#endif

#pragma mark streaming

void
AFastUserSwitch::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("UserName"), CFStringRef(mUserName) );
}

void
AFastUserSwitch::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef newUserName;
	dictionary->Get( CFSTR("UserName"), newUserName, CFSTR("") );
	check( newUserName );
	mUserName = newUserName->Copy();
}
