//
//  AFile.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 3 Apr 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#if KMF_IS_ENGINE
#import "ThreadFinished.h"
#endif

typedef enum { // WARNING: tag/index valued in AFile.xib match this enum
	kFileOperationMove,
	kFileOperationCopy,
	kFileOperationDuplicate,
	kFileOperationTrash,
	kFileOperationDelete,
	kFileOperationRecursiveDelete,
	kFileOperationDEFAULT = kFileOperationDuplicate, 
} FileOperationType;
#define kFileOperationTypeNamesArray {CFSTR("Move"), CFSTR("Copy"), CFSTR("Duplicate"), CFSTR("Trash"), CFSTR("Delete"), CFSTR("RecursiveDelete"), NULL}

class AFile
	:	public HBaseAction
{

private:
	
	FileOperationType				mOperation;
	CFContainer<CFStringRef>		mSource;
	CFContainer<CFStringRef>		mDestination;
	
#if KMF_IS_ENGINE
	ThreadFinished::Token			mThreadToken;
#endif
	
public:

    AFile();
	AFile( const AFile& inAction );
	AFile( const FileOperationType& inOperation );
	
    virtual ~AFile();

	virtual AFile*					Clone() const { return new AFile( *this ); }
	virtual AFile*					Create() const { return new AFile(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual	XIcon					GetActionDescriptionXIcon() const;

	virtual NSString*				GetNewActionTitle() const { return NSCS(CopyActionDescriptionString()); }

    virtual HActionType				GetActionType() const { return HActionType::kFile; }
	
	FileOperationType				GetOperation() const { return mOperation; }
	void							SetOperation( FileOperationType operation ) { mOperation = operation; }
	
	CFStringRef						CopySource() const { return mSource->Copy(); }
	NSString*						GetSource() const { return mSource->GetNS(); }
	void							SetSource( CFStringRef source ) { mSource = source->Copy(); }
	
	CFStringRef						CopyDestination() const { return mDestination->Copy(); }
	NSString*						GetDestination() const { return mDestination->GetNS(); }
	void							SetDestination( CFStringRef destination ) { mDestination = destination->Copy(); }
	
	bool							UseDestination() const { return mOperation == kFileOperationMove || mOperation == kFileOperationCopy; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	Boolean							IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

