/*
 *  AFilterClipboard.h
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Fri 2 May 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "FilterText.h"
#import "ClipboardTarget.h"

const FilterText::Action kDefaultFilterTextAction = FilterText::kRemoveStyles;

class AFilterClipboard
	:	public HBaseAction
{

private:

	FilterText::Action				mAction = kDefaultFilterTextAction;
	ClipboardTarget					mTarget;

public:

    AFilterClipboard() = default;
	AFilterClipboard( const AFilterClipboard& inAction ) = default;
    virtual ~AFilterClipboard() = default;

	virtual AFilterClipboard*		Clone() const { return new AFilterClipboard( *this ); }
	virtual AFilterClipboard*		Create() const { return new AFilterClipboard(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kFilterClipboard; }
	virtual	CFStringRef				CopyActionDescriptionString() const;

	NSString*						GetGenericNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }

	void							SetAction( FilterText::Action inValue ) { mAction = inValue; }
	FilterText::Action				GetAction() const { return mAction; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); }
	virtual Class					ActionController() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

