/*
 *  AFilterClipboard.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Fri 2 May 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "AFilterClipboard.h"

#if KMF_IS_ENGINE
#import "XClipboardHistory.h"
#import "XClipboardDataSet.h"
#import "NSAttributedStringExtensions.h"
#endif

#if KMF_EDITOR_WINDOW
#import "AFilterClipboardEditor.h"
#endif

CFStringRef
AFilterClipboard::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("hkFilterClipboardTitleFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, mTarget.GetDisplayName(), FilterText::GetDescription( mAction ) );
}

#if KMF_IS_ENGINE

float
AFilterClipboard::DoAction()
{
	float time = 0;
	const XClipboardDataSet* clip = mTarget.GetClipboardDataSet();
	if ( clip ) {
		NSAttributedString* inClipboard;
		if ( mAction == FilterText::kRemoveStyles ) {
			CFContainer<CFStringRef> normalString = clip->RequireCopyText();
			inClipboard = [[NSAttributedString alloc] initWithString:normalString->GetNS()];
			inClipboard.wasPlainText = true;
		} else {
			inClipboard = clip->AttributedString( true );
		}
		bool wasPlainText = inClipboard.wasPlainText;
		NSMutableAttributedString* clipboard = [inClipboard mutableCopy];
		time = FilterText::Process( mAction, clipboard );
		// Avoid adding default formatting to plain text clipboards
		mTarget.WriteClipboardDataSet( wasPlainText, clipboard );
	}
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AFilterClipboard::ActionController() const
{
	return [AFilterClipboardEditor class];
}

#endif

#pragma mark streaming

static const CFStringRef kFilterTextNames[] = kAFilterTextNamesArray;

void
AFilterClipboard::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Action"), kFilterTextNames[mAction] );
	mTarget.SaveToDictionary( dictionary, CFSTR("Target") );
}

void
AFilterClipboard::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mAction = (FilterText::Action) dictionary->GetEnumerated( CFSTR("Action"), kFilterTextNames, kDefaultFilterTextAction );
	mTarget.InitializeFromDictionary( dictionary, CFSTR("Target") );
}
