//
//  AFilterVariable.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 10 May 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "AFilterVariable.h"

#if KMF_IS_ENGINE
#import "KMVariables.h"
#import "KMNotifications.h"
#endif

#if KMF_EDITOR_WINDOW
#import "AFilterVariableEditor.h"
#endif

const FilterText::Action kDefaultAction = FilterText::kCapitalize;

AFilterVariable::AFilterVariable()
	:	HBaseAction(),
		mVariable( CFSTR("Variable")->Copy() ),
		mAction( kDefaultAction )
{
}

AFilterVariable::AFilterVariable( const AFilterVariable& inAction )
	:	HBaseAction( inAction ),
		mVariable( inAction.mVariable ),
		mAction( inAction.mAction )
{
}

AFilterVariable::~AFilterVariable()
{
}

CFStringRef
AFilterVariable::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("hkFilterVariableTitleFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, CFStringRef(mVariable), FilterText::GetDescription( mAction ) );
}


CFStringRef
AFilterVariable::CopyTransformationResult( NSUInteger* errorPos, float* outTime ) const
{
	float time;
	NSString* value = KMVariables::GetWithPassword( mVariable->GetNS() );
	CFContainer<CFStringRef> result = FilterText::CopyResult( mAction, CFBridgingToCFString(value), time );
	if ( errorPos ) *errorPos = 0;
	if ( outTime ) *outTime = time;
	return result->Copy();
}

#if KMF_IS_ENGINE

float
AFilterVariable::DoAction()
{
	NSUInteger errorPos;
	float time;
	
	NSString* result = GetTransformationResult( &errorPos, &time );
	check( result ); // FilterText cannot fail
	KMVariables::Set( mVariable->GetNS(), result, KMNotifications::kLoadVariableFromEngineAction );
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AFilterVariable::ActionController() const
{
	return [AFilterVariableEditor class];
}

#endif

#pragma mark streaming

static const CFStringRef kFilterTextNames[] = kAFilterTextNamesArray;

void
AFilterVariable::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Variable"), CFStringRef(mVariable) );
	dictionary->Set( CFSTR("Action"), kFilterTextNames[mAction] );
}

void
AFilterVariable::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Variable"), value, CFSTR("Variable") );
	check( value );
	mVariable = value->Copy();
	mAction = (FilterText::Action) dictionary->GetEnumerated( CFSTR("Action"), kFilterTextNames, kDefaultAction );
}
