//
//  AFindImage.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 3 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"
#import "ThreadFinished.h"
#import "MatchImage.h"

const Boolean kFindImageDefaultUnique = true;

class AFindImage
	:	public HBaseAction
{
private:

	NSImage*						mImage = nil;
	UInt32							mFuzz = MatchImage::kDefaultFuzz;
	Boolean							mUnique = kFindImageDefaultUnique;
	VariableContainer				mVariable = @"Variable";
	Boolean							mDisplayMatches = false;

#if KMF_IS_ENGINE
	ThreadFinished::Token			mThreadToken = ThreadFinished::kNull;
	void							MatchCompleted( bool match, CGPoint topleft );
#endif

protected:

public:

    AFindImage() = default;
	AFindImage( const AFindImage& inAction );
    virtual ~AFindImage();

	virtual AFindImage*				Clone() const { return new AFindImage( *this ); }
	virtual AFindImage*				Create() const { return new AFindImage(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kFindImage; }
	
	NSImage*						GetImage() const { return mImage; }
	void							SetImage( NSImage* inImage );
	
	void							SetFuzz( UInt32 inFuzz ) { mFuzz = inFuzz; }
	UInt32							GetFuzz() const { return mFuzz; }
	
	Boolean							GetUnique() const { return mUnique; }
	void							SetUnique( Boolean inUnique ) { mUnique = inUnique; }

	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	virtual NSString*				GetVariable() const { return mVariable->GetNS(); }
	
	void							SetDisplayMatches( Boolean inDisplayMatches ) { mDisplayMatches = inDisplayMatches; }
	Boolean							GetDisplayMatches() const { return mDisplayMatches; }

#if KMF_IS_ENGINE
	virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

