//
//  AFor.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 18 Mar 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "CCollectionList.h"
#import "HActionList.h"
#import "KMVariables.h"

class AFor
	:	public HBaseAction
{

private:

#if KMF_IS_ENGINE
	NSUUID*							mTrackingID;
	Boolean							mExecuting;
#endif
	
	VariableContainer				mVariable;
	CCollectionList					mCollections;
	HActionList						mActionList;

public:

    AFor();
	AFor( const AFor& inAction );
    virtual ~AFor();

	virtual AFor*					Clone() const { return new AFor( *this ); }
	virtual AFor*					Create() const { return new AFor(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kFor; }
	virtual Boolean					IsLoop() const { return true; }

	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return kVeryLongDefaultTimeOutValue; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	const CCollectionList&			GetCollectionList() const { return mCollections; }
	void							SetCollectionList( const CCollectionList& inCollections ) { mCollections = inCollections; }
	
	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	virtual NSString*				GetVariable() const { return mVariable->GetNS(); }
	
	const HActionList&				GetActionList() const { return mActionList; }
	void							SetActionList( const HActionList& inActions ) { mActionList = inActions; }
	void							SetActionAtIndex( const CFIndex& inIndex, HBaseAction* inAction ) { mActionList.SetActionAtIndex( inIndex, inAction ); }
	
	virtual void					DoForEverySubAction( ActionBlock inApplier ) { mActionList.DoForEveryActionAndSubAction( inApplier ); }

	virtual BOOL					MatchesTextSearch( NSString* search ) const;
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

