//
//  AGetFileAttribute.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 11 Apr 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"
#import "VariableTransformationAction.h"

enum class FileAttributeType { // WARNING: tag valued in AGetFileAttribute.xib match this enum
	kFileType,
	kFileSize,
	kCreationDate,
	kModificationDate,
	kOwnerName,
	kOwnerID,
	kGroupName,
	kGroupID,
	kPosixPermissions,
	kExtensionHidden,
	kHFSCreatorCode,
	kHFSTypeCode,
	kParent,
	kName,
	kBaseName,
	kExtension,
	kDEFAULT = kModificationDate, 
};
#define kFileAttributeTypeNamesArray {CFSTR("FileType"), CFSTR("FileSize"), CFSTR("CreationDate"), CFSTR("ModificationDate"), CFSTR("OwnerName"), CFSTR("OwnerID"), CFSTR("GroupName"), CFSTR("GroupID"), CFSTR("PosixPermissions"), CFSTR("ExtensionHidden"), CFSTR("HFSCreatorCode"), CFSTR("HFSTypeCode"), CFSTR("Parent"), CFSTR("Name"), CFSTR("BaseName"), CFSTR("Extension"), NULL}
#define kFileAttributeNSAttributeKeyNamesArray {NSFileType, NSFileSize, NSFileCreationDate, NSFileModificationDate, NSFileOwnerAccountName, NSFileOwnerAccountID, NSFileGroupOwnerAccountName, NSFileGroupOwnerAccountID, NSFilePosixPermissions, NSFileExtensionHidden, NSFileHFSCreatorCode, NSFileHFSTypeCode, NULL, NULL, NULL, NULL, NULL}

class AGetFileAttribute
	:	public HBaseAction,
		public VariableTransformationAction
{

private:

	FileAttributeType				mAttribute;
	CFContainer<CFStringRef>		mPath;
	VariableContainer				mVariable;

public:

    AGetFileAttribute();
	AGetFileAttribute( const AGetFileAttribute& inAction );
    virtual ~AGetFileAttribute();

	virtual AGetFileAttribute*		Clone() const { return new AGetFileAttribute( *this ); }
	virtual AGetFileAttribute*		Create() const { return new AGetFileAttribute(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kGetFileAttribute; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	
	static CFStringRef				CopyAttributeName( FileAttributeType inAttribute );
	
	virtual CFStringRef				CopyTransformationResult( NSUInteger* errorPos, float* outTime ) const;

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	FileAttributeType				GetAttribute() const { return mAttribute; }
	void							SetAttribute( FileAttributeType inAttribute ) { mAttribute = inAttribute; }

	CFStringRef						CopyPath() const { return mPath->Copy(); }
	NSString*						GetPath() const { return mPath->GetNS(); }
	void							SetPath( CFStringRef inPath ) { mPath = inPath->Copy(); }
	
	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	virtual NSString*				GetVariable() const { return mVariable->GetNS(); }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
	
};

