//
//  AGetImageSize.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 2 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "AGetImageSize.h"
#import "FileReference.h"
#import "XClipboardDataSet.h"
#import "XClipboardHistory.h"
#import "CSClipboardDomain.h"
#import "NSStringRegularExpressions.h"
#import "XErrorHandler.h"

#if KMF_IS_ENGINE
#import "KMVariables.h"
#import "KMNotifications.h"
#endif

#if KMF_EDITOR_WINDOW
#import "AGetImageSizeEditor.h"
#import "TemplateEngine.h"
#endif

AGetImageSize::AGetImageSize()
	:	HBaseAction(),
		mVariable( CFSTR("Variable")->Copy() )
{
}

AGetImageSize::AGetImageSize( const AGetImageSize& inAction )
	:	HBaseAction( inAction ),
		mTarget( inAction.mTarget ),
		mVariable( inAction.mVariable )
{
}

AGetImageSize::~AGetImageSize()
{
}

CFStringRef
AGetImageSize::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("kmGetImageSize")->CopyLocalized();

	return CFString::CreateWithFormat( format, mTarget.GetDisplayName() );
}

#if KMF_IS_ENGINE

float
AGetImageSize::DoAction()
{
	float time = 5;

	NSString* result = @"";
	const XClipboardDataSet* clip = mTarget.GetClipboardDataSet();
	if ( clip ) {
		NSImage* image = clip->Image();
		if ( image ) {
			NSSize size = image.size;
			result = [NSString stringWithFormat:@"%d,%d", (int)roundf(size.width), (int)roundf(size.height)];
		}
	}
	KMVariables::Set( mVariable->GetNS(), result, KMNotifications::kLoadVariableFromEngineAction );
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AGetImageSize::ActionController() const
{
	return [AGetImageSizeEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
{{ text }}
To variable ‘{{ variable | htmlencode }}’
)RAW";

NSString*
AGetImageSize::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"variable" : mVariable.GetNS(),
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
AGetImageSize::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	mTarget.SaveToDictionary( dictionary, CFSTR("Target") );
	dictionary->Set( CFSTR("Variable"), CFStringRef(mVariable) );
}

void
AGetImageSize::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mTarget.InitializeFromDictionary( dictionary, CFSTR("Target") );
	CFStringRef value;
	dictionary->Get( CFSTR("Variable"), value, CFSTR("Variable") );
	check( value );
	mVariable = value->Copy();
}
