//
//  AGrowl.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 7 Oct 2009.
//  Copyright (c) 2009 Stairways Software. All rights reserved.
//

#import "AGrowl.h"
#if KMF_IS_ENGINE
#import "Growl/GrowlApplicationBridge.h"
#import "TokenText.h"
#import "TextResultsBrieflyWindowController.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AGrowlEditor.h"
#endif

AGrowl::AGrowl()
	:	HBaseAction()
{
	mTitle = CFSTR("kGrowlDefaultTitle")->CopyLocalized();
	mText = CFSTR("kGrowlDefaultText")->CopyLocalized();
}

AGrowl::AGrowl( const AGrowl& inAction )
	:	HBaseAction( inAction ),
		mTitle( inAction.mTitle ),
		mText( inAction.mText )
{
}

AGrowl::~AGrowl()
{
}

CFStringRef
AGrowl::CopyActionDescriptionString() const
{
	return CFString::CreateWithFormat( CFCS( CFSTR("hkGrowlTitle")->CopyLocalized() ), CFStringRef(mTitle) );
}

#if KMF_IS_ENGINE

float
AGrowl::DoAction()
{
	float time = 10;
	
	CFContainer<CFStringRef> mDisplayTitle = TokenText::CopyResult( mTitle, time );
	CFContainer<CFStringRef> mDisplayText = TokenText::CopyResult( mText, time );

	if ( [GrowlApplicationBridge isGrowlRunning] ) {
		[GrowlApplicationBridge 
		 notifyWithTitle:mDisplayTitle->GetNS()
		 description:mDisplayText->GetNS()
		 notificationName:@"Growl Action"
		 iconData:nil priority:0 isSticky:NO clickContext:nil];
	} else {
		[TextResultsBrieflyWindowController DisplayWithTitle:mDisplayTitle->GetNS() text:mDisplayText->GetNS()];
	}
	return 20;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AGrowl::ActionController() const
{
	return [AGrowlEditor class];
}

NSString*
AGrowl::GetActionExtraHTML() const
{
	return UserTextForHTML( mText->GetNS() );
}

#endif

#pragma mark streaming

void
AGrowl::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Title"), CFStringRef(mTitle) );
	dictionary->Set( CFSTR("Text"), CFStringRef(mText) );
}

void
AGrowl::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef newTitle;
	if ( dictionary->Get( CFSTR("Title"), newTitle ) ) {
		mTitle = newTitle->Copy();
	}

	CFStringRef newText;
	XAssert( dictionary->Get( CFSTR("Text"), newText ) );
	mText = newText->Copy();
}
