/*
 *  AHideApplications.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Dec 16 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "AHideApplications.h"
#import "PSSwitcherApplicationList.h"
#import "MMacroDomain.h"
#import "KMIconRegistry.h"
#if KMF_EDITOR_WINDOW
#import "AHideApplicationsEditor.h"
#endif

static const Boolean kDefaultHonorExcluded = true;

AHideApplications::AHideApplications( const EHideApplicationsActionType& inType )
	:	HBaseAction(),
		mHideActionType( inType ),
		mHonorExcluded( kDefaultHonorExcluded )
{
}

HActionType
AHideApplications::GetActionType() const
{
	HActionType	theType;

    switch ( mHideActionType ) {
		case EHideApplicationsActionType::kAllApplications:
			theType = HActionType::kHideAll;
			break;
		case EHideApplicationsActionType::kOtherApplications:
			theType = HActionType::kHideOthers;
			break;
		case EHideApplicationsActionType::kCurrentApplication:
			theType = HActionType::kHideCurrent;
			break;
    }
    
    return theType;
}

#if KMF_IS_ENGINE
float
AHideApplications::DoAction()
{
	float time = 0;
	
	switch ( mHideActionType ) {
		case EHideApplicationsActionType::kCurrentApplication:
			XApplicationIdentifier::FrontApplication().Hide();
			time = 2;
			break;
		case EHideApplicationsActionType::kAllApplications:
			XApplicationIdentifier::FinderDefaultApplication().SwitchAndHideOthers( true, mHonorExcluded );
			time = 5;
			break;
		case EHideApplicationsActionType::kOtherApplications:
			XApplicationIdentifier::FrontApplication().HideOthers( mHonorExcluded );
			time = 5;
			break;
	}
	return time;
}
#endif

#if KMF_EDITOR_WINDOW

Class
AHideApplications::ActionController() const
{
	if ( mHideActionType != EHideApplicationsActionType::kCurrentApplication ) {
		return [AHideApplicationsEditor class];
	}
	return HBaseAction::ActionController();
}

NSString*
AHideApplications::GetActionHTMLDescriptionString() const
{
	if ( mHonorExcluded || (mHideActionType == EHideApplicationsActionType::kCurrentApplication) ) return GetActionDescriptionString();
	return [NSString stringWithFormat:@"%@ Including Excluded Applications", GetActionDescriptionString()];
}

#endif

#pragma mark streaming

void
AHideApplications::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->SetBoolean( CFSTR("HonorExcluded"), mHonorExcluded );
}

void
AHideApplications::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	dictionary->GetBoolean( CFSTR("HonorExcluded"), mHonorExcluded, kDefaultHonorExcluded );
}
