//
//  AHiliteLocation.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Mon 7 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ArrowStrings.h"

typedef enum { // WARNING: tag valued in AHiliteLocation.xib match this enum
	kHiliteLocationStyleRectangle,
	kHiliteLocationStyleOval, 
	kHiliteLocationStyleTarget, 
	kHiliteLocationStyleDEFAULT = kHiliteLocationStyleTarget, 
} HiliteLocationStyleType;
#define kHiliteLocationStyleTypeNamesArray {CFSTR("Rectangle"), CFSTR("Oval"), CFSTR("Target"), NULL}

#if KMF_IS_ENGINE
@class HiliteController;
#endif

class AHiliteLocation
	:	public HBaseAction
{
private:

	HiliteLocationStyleType			mStyle;
    CFContainer<CFStringRef>		mLeft;
    CFContainer<CFStringRef>		mTop;
    CFContainer<CFStringRef>		mWidth;
    CFContainer<CFStringRef>		mHeight;
	Boolean							mAnimate;
    CFContainer<CFStringRef>		mDisplayTime;

#if KMF_IS_ENGINE
	HiliteController*				mHiliteController;
#endif

protected:

public:

    AHiliteLocation();
	AHiliteLocation( const AHiliteLocation& inAction );
	
    virtual ~AHiliteLocation();

	virtual AHiliteLocation*		Clone() const { return new AHiliteLocation( *this ); }
	virtual AHiliteLocation*		Create() const { return new AHiliteLocation(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kHiliteLocation; }
	
	HiliteLocationStyleType			GetStyle() const { return mStyle; }
	void							SetStyle( HiliteLocationStyleType inStyle ) { mStyle = inStyle; }
	
	const ArrowString::Style*		GetArrowStyles() const; // changes whenever Style changes

	void							SetLeft( CFStringRef inString ) { mLeft = inString->Copy(); }
	void							SetLeft( double inValue ) { mLeft = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyLeft() const { return mLeft->Copy(); }
	NSString*						GetLeft() const { return mLeft->GetNS(); }
	
	void							SetTop( CFStringRef inString ) { mTop = inString->Copy(); }
	void							SetTop( double inValue ) { mTop = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyTop() const { return mTop->Copy(); }
	NSString*						GetTop() const { return mTop->GetNS(); }
	
	void							SetWidth( CFStringRef inString ) { mWidth = inString->Copy(); }
	void							SetWidth( double inValue ) { mWidth = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyWidth() const { return mWidth->Copy(); }
	NSString*						GetWidth() const { return mWidth->GetNS(); }
	
	void							SetHeight( CFStringRef inString ) { mHeight = inString->Copy(); }
	void							SetHeight( double inValue ) { mHeight = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyHeight() const { return mHeight->Copy(); }
	NSString*						GetHeight() const { return mHeight->GetNS(); }
	
	Boolean							GetAnimate() const { return mAnimate; }
	void							SetAnimate( Boolean inAnimate ) { mAnimate = inAnimate; }
	
	void							SetDisplayTime( CFStringRef inString ) { mDisplayTime = inString->Copy(); }
	void							SetDisplayTime( double inValue ) { mDisplayTime = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyDisplayTime() const { return mDisplayTime->Copy(); }
	NSString*						GetDisplayTime() const { return mDisplayTime->GetNS(); }
	
	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return kVeryLongDefaultTimeOutValue; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

