//
//  AIfThenElse.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Mon 28 Mar 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "CConditionList.h"
#import "HActionList.h"

class AIfThenElse
	:	public HBaseAction
{

private:

#if KMF_IS_ENGINE
	NSUUID*							mTrackingID;
	Boolean							mExecuting;
#endif
	
	CConditionList					mConditions;
	HActionList						mThenActionList;
	HActionList						mElseActionList;

public:

    AIfThenElse();
	AIfThenElse( const AIfThenElse& inAction );
    virtual ~AIfThenElse();

	virtual AIfThenElse*			Clone() const { return new AIfThenElse( *this ); }
	virtual AIfThenElse*			Create() const { return new AIfThenElse(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kIfThenElse; }
	
	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return kVeryLongDefaultTimeOutValue; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	const CConditionList&			GetConditionList() const { return mConditions; }
	void							SetConditionList( const CConditionList& inConditions ) { mConditions = inConditions; }
	
	const HActionList&				GetThenActionList() const { return mThenActionList; }
	void							SetThenActionList( const HActionList& inActions ) { mThenActionList = inActions; }
	void							SetThenActionAtIndex( const CFIndex& inIndex, HBaseAction* inAction ) { mThenActionList.SetActionAtIndex( inIndex, inAction ); }
	
	const HActionList&				GetElseActionList() const { return mElseActionList; }
	void							SetElseActionList( const HActionList& inActions ) { mElseActionList = inActions; }
	void							SetElseActionAtIndex( const CFIndex& inIndex, HBaseAction* inAction ) { mElseActionList.SetActionAtIndex( inIndex, inAction ); }
	
	virtual void					DoForEverySubAction( ActionBlock inApplier ) { mThenActionList.DoForEveryActionAndSubAction( inApplier ); mElseActionList.DoForEveryActionAndSubAction( inApplier ); }

	virtual BOOL					MatchesTextSearch( NSString* search ) const;
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

