/*
 *  AInsertText.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Dec 16 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "StyledTextSource.h"

#if KMF_IS_ENGINE
#import "InsertText.h"
#endif

// WARNING: enum valued used in editor
enum class InsertTextActionType {
	kByTyping,
	kByPasting,
	kByPastingStyles,
	kDisplayWindow,
	LASTENUM( kDisplayBriefly ),
	kDEFAULT = kByPasting,
};
#define kInsertTextActionNamesArray {CFSTR("ByTyping"), CFSTR("ByPasting"), CFSTR("ByPastingStyles"), CFSTR("DisplayWindow"), CFSTR("DisplayBriefly"), NULL}

class AInsertText
	:	public HBaseAction
{
private:

	StyledTextSource				mStyledText;
	InsertTextActionType			mAction = InsertTextActionType::kDEFAULT;
	
#if KMF_IS_ENGINE
	std::unique_ptr<InsertText>		mInsertText; // not copied
	NSUInteger						mSimulateLeftArrows;
#endif

public:
	
    AInsertText() = default;
	AInsertText( const AInsertText& inAction );
	AInsertText( const InsertTextActionType& inActionType );
    virtual ~AInsertText() = default;

	virtual AInsertText*			Clone() const { return new AInsertText( *this ); }
	virtual AInsertText*			Create() const { return new AInsertText(); }

    virtual HActionType				GetActionType() const { return HActionType::kInsertText; }
	virtual	CFStringRef				CopyActionTitle() const;
	virtual	CFStringRef				CopyActionDescriptionString() const;

#if KMF_EDITOR_WINDOW
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual	XIcon					GetActionDescriptionXIcon() const;

	virtual HBaseAction*			Merge( HBaseAction* action );

	void							SetPlainText( NSString* inString ) { mStyledText.SetPlainText( inString ); }
	void							SetStyledText( NSAttributedString* inString ) { mStyledText.SetStyledText( inString ); }

	NSString*						GetSourceString() const { return mStyledText.GetString(); }
	NSAttributedString*				GetSourceAttributedString() const { return mStyledText.GetAttributedString(); }

	Boolean							GetUseStyledText() const { return mStyledText.GetUseStyledText(); }
	
	void							SetInsertTextAction( InsertTextActionType inInsertTextAction ) { mAction = inInsertTextAction; }
	InsertTextActionType			GetInsertTextAction() const { return mAction; }
	
	bool							InsertTextActionIsStyled() const { return mAction == InsertTextActionType::kByPastingStyles || mAction == InsertTextActionType::kDisplayWindow; }
	bool							InsertTextActionIsDisplay() const { return mAction == InsertTextActionType::kDisplayWindow || mAction == InsertTextActionType::kDisplayBriefly; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	static CFStringRef				CopySummaryText( CFStringRef text );
	static CFStringRef				CopySummaryText( NSString* text );
	static NSString*				GetSummaryText( NSString* text );
};

