/*
 *  AMacroGroupToggle.h
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Fri 6 Jun 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "MMacroSerializedObject.h"

enum class MacroGroupToggleMode { // WARNING: tag value in AMacroGroupToggle.xib match this enum
	kInternalAutomatic,
	kOnce,
	kActivate,
	kDeactivate,
	kToggle,
	kOnceWithPalette,
	kShowPalette,
	kHidePalette,
	kTogglePalette,
	kDEFAULT = kOnceWithPalette,
};
#define kMacroGroupToggleModeNamesArray {CFSTR("InternalAutomatic"), CFSTR("Once"), CFSTR("Activate"), CFSTR("Deactivate"), CFSTR("Toggle"), CFSTR("OnceWithPalette"), CFSTR("ShowPalette"), CFSTR("HidePalette"), CFSTR("TogglePalette"), NULL}

class AMacroGroupToggle
	:	public HBaseAction
{

private:

	NSString*						mMacroGroupUID = @"";
	MacroGroupToggleMode			mToggleMode = MacroGroupToggleMode::kDEFAULT;
	Boolean							mDisplayToggle = true;

public:

    AMacroGroupToggle() = default;
	AMacroGroupToggle( const AMacroGroupToggle& inAction ) = default;
	AMacroGroupToggle( const MacroGroupToggleMode& inMode ) : mToggleMode(inMode) { ; }
    virtual ~AMacroGroupToggle() = default;

	virtual AMacroGroupToggle*		Clone() const { return new AMacroGroupToggle( *this ); }
	virtual AMacroGroupToggle*		Create() const { return new AMacroGroupToggle(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kMacroGroupToggle; }

	virtual NSString*				GetNewActionTitle() const;
 	virtual	CFStringRef				CopyActionDescriptionString() const;
 	virtual	XIcon					GetActionDescriptionXIcon() const;

	NSString*						GetMacroGroupUID() const { return mMacroGroupUID; }
	void							SetMacroGroupUID( NSString* inMacroGroupUID ) { mMacroGroupUID = [inMacroGroupUID copy]; }

	MacroGroupToggleMode			GetToggleMode() const { return mToggleMode; }
	void							SetToggleMode( MacroGroupToggleMode inToggleMode ) { mToggleMode = inToggleMode; }

	Boolean							GetDisplayToggle() const { return mDisplayToggle; }
	void							SetDisplayToggle( Boolean inDisplayToggle ) { mDisplayToggle = inDisplayToggle; }

	static Boolean					UsesDisplayToggle( MacroGroupToggleMode mode ) { return mode == MacroGroupToggleMode::kOnce || mode == MacroGroupToggleMode::kActivate || mode == MacroGroupToggleMode::kDeactivate || mode == MacroGroupToggleMode::kToggle; }
	Boolean							UsesDisplayToggle() const { return UsesDisplayToggle( mToggleMode ); }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

