/*
 *  AMacroGroupToggle.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Fri 6 Jun 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "AMacroGroupToggle.h"
#import "MMacroGroup.h"
#import "MMacroDomain.h"
#import "MMacroSerializedObject.h"
#import "AMacroGroupToggleEditor.h"

static const CFStringRef kMacroGroupToggleModeNames[] = kMacroGroupToggleModeNamesArray;

NSString*
AMacroGroupToggle::GetNewActionTitle() const
{
	CFContainer<CFStringRef> format = CFString::CreateWithFormat( CFSTR("kMacroGroupToggle%@"), kMacroGroupToggleModeNames[int(mToggleMode)] );
	format = format->CopyLocalized();
	return format->GetNS();
}

CFStringRef
AMacroGroupToggle::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFString::CreateWithFormat( CFSTR("kMacroGroupToggle%@Format"), kMacroGroupToggleModeNames[int(mToggleMode)] );
	format = format->CopyLocalized();

	CFContainer<CFStringRef> name;
	MMacroGroup* group = MMacroSerializedObject::FindMacroGroupByUID( mMacroGroupUID );
	if ( group ) {
		name = group->CopyName();
	} else {
		name = CFSTR("NoToggleMacroGroup")->CopyLocalized();
	}

	return CFString::CreateWithFormat( format, CFStringRef(name) );
}

XIcon
AMacroGroupToggle::GetActionDescriptionXIcon() const
{
	XIcon result;
	bool palette = false;
	switch( mToggleMode ) {
		case MacroGroupToggleMode::kInternalAutomatic:
		case MacroGroupToggleMode::kOnce:
		case MacroGroupToggleMode::kActivate:
		case MacroGroupToggleMode::kDeactivate:
		case MacroGroupToggleMode::kToggle:
			palette = false;
			break;
		case MacroGroupToggleMode::kOnceWithPalette:
		case MacroGroupToggleMode::kShowPalette:
		case MacroGroupToggleMode::kHidePalette:
		case MacroGroupToggleMode::kTogglePalette:
			palette = true;
			break;
	}
	if ( palette ) {
		result = MMacroGroup::gPaletteBadgeFolderIcon();
	} else {
		result = MMacroGroup::gHotKeyBadgeFolderIcon();
	}
	return result;
}

#if KMF_IS_ENGINE

float
AMacroGroupToggle::DoAction()
{
	MMacroGroup* group = MMacroSerializedObject::FindMacroGroupByUID( mMacroGroupUID );
	if ( group ) {
		group->DoMacroGroupToggle( mToggleMode, mDisplayToggle );
	}
	return 10;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AMacroGroupToggle::ActionController() const
{
	return [AMacroGroupToggleEditor class];
}

#endif

#pragma mark streaming

void
AMacroGroupToggle::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	check( mToggleMode != MacroGroupToggleMode::kInternalAutomatic );
	
	dictionary->Set( CFSTR("MacroGroupUID"), CFBridgingToCFString(mMacroGroupUID) );
	dictionary->Set( CFSTR("ToggleMode"), kMacroGroupToggleModeNames[int(mToggleMode)] );
	dictionary->SetBoolean( CFSTR("DisplayToggle"), mDisplayToggle );
}

void
AMacroGroupToggle::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("MacroGroupUID"), value, CFSTR("") );
	check( value );
	mMacroGroupUID = value->GetNS();
	mToggleMode = dictionary->GetEnumClass( CFSTR("ToggleMode"), kMacroGroupToggleModeNames, MacroGroupToggleMode::kDEFAULT );
	dictionary->GetBoolean( CFSTR("DisplayToggle"), mDisplayToggle, true );

	check( mToggleMode != MacroGroupToggleMode::kInternalAutomatic );
	if ( mToggleMode == MacroGroupToggleMode::kInternalAutomatic ) {
		mToggleMode = MacroGroupToggleMode::kDEFAULT;
	}
}

