/*
 *  AManipulateWindow.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sat Mar 15 2003.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "NSStringExtensions.h"
#import "ApplicationTarget.h"

enum class ManipulateWindowTargetType { // WARNING: tag valued in AManipulateWindow.xib match this enum
	kFrontWindow,
	kNamedWindow,
	kWindowNameContaining,
	kWindowIndex,
	kDEFAULT = kFrontWindow, 
};
#define kManipulateWindowTargetTypeNamesArray {CFSTR("FrontWindow"), CFSTR("NamedWindow"), CFSTR("WindowNameContaining"), CFSTR("WindowIndex"), NULL}

enum class ManipulateWindowActionType { // WARNING: tag valued in AManipulateWindow.xib match this enum
	kResizeWindowByPercent,
	kResizeWindowBy,
	kResizeWindowTo,
	kMoveWindowBy,
	kMoveWindowTo,
	kMoveAndResize,
	kCenterWindow,
	kCenterWindowAt,
	kCloseWindow,
	kZoomWindow,
	kMinimizeWindow,
	kUnminimizeWindow,
	kToggleMinimizeWindow,
	kSelectWindow,
	kDEFAULT = kResizeWindowByPercent, 
};
// WARNING: Also used in Localized.string
#define kManipulateWindowActionTypeNamesArray {CFSTR("ResizeWindowByPercent"), CFSTR("ResizeWindowBy"), CFSTR("ResizeWindowTo"), CFSTR("MoveWindowBy"), CFSTR("MoveWindowTo"), CFSTR("MoveAndResize"), CFSTR("CenterWindow"), CFSTR("CenterWindowAt"), CFSTR("CloseWindow"), CFSTR("ZoomWindow"), CFSTR("ReallyMinimizeWindow"), CFSTR("UnminimizeWindow"), CFSTR("MinimizeWindow"), CFSTR("SelectWindow"), NULL}

#define kManipulateWindowDefaultNumber @"125"
#define kManipulateWindowDefaultSizeNumber @"300"
#define kManipulateWindowDefaultWindowIndex @"2"

#if KMF_IS_ENGINE
class XAXWindowElement;
#endif

class AManipulateWindow
	:	public HBaseAction
{
private:

	ManipulateWindowActionType		mAction = ManipulateWindowActionType::kDEFAULT;
	ManipulateWindowTargetType		mTargeting = ManipulateWindowTargetType::kDEFAULT;
	NSString*						mWindowName = @"";
	NSString*						mHorizontalNumber = kManipulateWindowDefaultNumber;
	NSString*						mVerticalNumber = kManipulateWindowDefaultNumber;
	NSString*						mWidthNumber = kManipulateWindowDefaultSizeNumber;
	NSString*						mHeightNumber = kManipulateWindowDefaultSizeNumber;
	NSString*						mWindowIndex = kManipulateWindowDefaultWindowIndex;
	ApplicationTarget				mTargetApplication;

#if KMF_IS_ENGINE
	void							ExecuteActionOnWindow( ProcessSerialNumber appPSN, const XAXWindowElement& inWindowElement );
#endif

protected:

public:

    AManipulateWindow() = default;
	AManipulateWindow( const AManipulateWindow& inAction ) = default;
    virtual ~AManipulateWindow() = default;

	virtual AManipulateWindow*		Clone() const { return new AManipulateWindow( *this ); }
	virtual AManipulateWindow*		Create() const { return new AManipulateWindow(); }
	
	virtual HBaseAction*			Merge( HBaseAction* action );

	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kManipulateWindow; }
	
	ManipulateWindowActionType		GetWindowAction() const { return mAction; }
	void							SetWindowAction( ManipulateWindowActionType inAction ) { mAction = inAction; }

	ManipulateWindowTargetType		GetWindowTargeting() const { return mTargeting; }
	void							SetWindowTargeting( ManipulateWindowTargetType inTargeting ) { mTargeting = inTargeting; }
	
	NSString*						GetWindowTargetName() const { return mWindowName; }
	void							SetWindowTargetName( NSString* inString ) { mWindowName = [inString copy]; }

	bool							NeedHorizontalVertical() const;
	bool							NeedWidthHeight() const;
	
	void							SetHorizontalNumber( NSString* inString ) { mHorizontalNumber = [inString copy]; }
	void							SetHorizontalNumber( double inValue ) { mHorizontalNumber = [NSString stringWithLongInt:lround(inValue)]; }
	NSString*						GetHorizontalNumber() const { return mHorizontalNumber; }

	void							SetVerticalNumber( NSString* inString ) { mVerticalNumber = [inString copy]; }
	void							SetVerticalNumber( double inValue ) { mVerticalNumber = [NSString stringWithLongInt:lround(inValue)]; }
	NSString*						GetVerticalNumber() const { return mVerticalNumber; }

	void							SetWidthNumber( NSString* inString ) { mWidthNumber = [inString copy]; }
	void							SetWidthNumber( double inValue ) { mWidthNumber = [NSString stringWithLongInt:lround(inValue)]; }
	NSString*						GetWidthNumber() const { return mWidthNumber; }

	void							SetHeightNumber( NSString* inString ) { mHeightNumber = [inString copy]; }
	void							SetHeightNumber( double inValue ) { mHeightNumber = [NSString stringWithLongInt:lround(inValue)]; }
	NSString*						GetHeightNumber() const { return mHeightNumber; }

	void							SetWindowIndex( NSString* inString ) { mWindowIndex = [inString copy]; }
	void							SetWindowIndex( double inValue ) { mWindowIndex = [NSString stringWithLongInt:lround(inValue)]; }
	NSString*						GetWindowIndex() const { return mWindowIndex; }

	ApplicationTargetApplicationType		GetApplicationTargetType() const { return mTargetApplication.GetApplicationTargetType(); }
    virtual const XApplicationIdentifier&	GetTargetApplication() const { return mTargetApplication.GetTargetApplication(); }

	void							SetApplicationTargetApplication( const XApplicationIdentifier& inApp ) { mTargetApplication.SetApplicationTargetApplication( inApp ); }
	void							SetApplicationTargetFrontApplication() { mTargetApplication.SetApplicationTargetFrontApplication(); }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual BOOL					CanBeInstigatedByClickAction() const { return YES; }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

