/*
 *  AMouseMoveAndClick.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Apr 28 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "ThreadFinished.h"
#import "MatchImage.h"

enum class MouseMoveAndClickRelativeType  { // WARNING: tag valued in AMouseMoveAndClick.xib match this enum
	kScreen,
	kWindow, 
	kMouse, 
	kImage, 
	kDEFAULT = kWindow, 
};
#define kMouseRelativeTypeNamesArray {CFSTR("Screen"), CFSTR("Window"), CFSTR("Mouse"), CFSTR("Image"), NULL}

enum class MouseMoveAndClickRelativeCornerType { // WARNING: tag valued in AMouseMoveAndClick.xib match this enum
	kTopLeft,
	kTopRight, 
	kBottomLeft, 
	kBottomRight, 
	kCenter, 
	kDEFAULT = kTopLeft, 
};
#define kMouseRelativeCornerTypeNamesArray {CFSTR("TopLeft"), CFSTR("TopRight"), CFSTR("BottomLeft"), CFSTR("BottomRight"), CFSTR("Center"), NULL}

enum class MouseMoveAndClickDragType { // WARNING: tag valued in AMouseMoveAndClick.xib match this enum
	kNone,
	kTo, 
	kRelative, 
	kDEFAULT = kNone, 
};
#define kMouseDragTypeNamesArray {CFSTR("None"), CFSTR("To"), CFSTR("Relative"), NULL}

class AMouseMoveAndClick final // because we override SaveToDictionary includeRecordedDetails, this must be "final"
	:	public HBaseAction
{
private:

	UInt32							mClickCount = 1;
	UInt32							mButton = kCGMouseButtonLeft;
	MouseMoveAndClickRelativeType			mMouseRelative = MouseMoveAndClickRelativeType::kDEFAULT;
	MouseMoveAndClickRelativeCornerType		mMouseRelativeCorner = MouseMoveAndClickRelativeCornerType::kDEFAULT;
    CFContainer<CFStringRef>		mHorizontalPos = CFSTR("0");
    CFContainer<CFStringRef>		mVerticalPos = CFSTR("0");
	UInt32							mModifiers = 0;
	MouseMoveAndClickDragType		mMouseDrag = MouseMoveAndClickDragType::kDEFAULT;
    CFContainer<CFStringRef>		mDragHorizontalPos = CFSTR("0");
    CFContainer<CFStringRef>		mDragVerticalPos = CFSTR("0");
	Boolean							mRestoreMouseLocation = false;
	NSImage*						mImage = nil;
	UInt32							mFuzz = MatchImage::kDefaultFuzz;
	Boolean							mDisplayMatches = false;

	NSDictionary*					mRecordedDetails = nil;

#if KMF_IS_ENGINE
	CGRect							mRelativeRect;
	Boolean							mRelativeRectFound;
	ThreadFinished::Token			mThreadToken = ThreadFinished::kNull;
	double							mHorizontalPosValue;
	double							mVerticalPosValue;
	double							mDragHorizontalPosValue;
	double							mDragVerticalPosValue;
	NSString*						mCancelMessage = nil;
	void							CancelThisMacro( NSString* message );
	void							MatchCompleted( bool match, CGPoint topleft );
#endif

	CFStringRef						CopyActionDescriptionAtString( bool lowercase ) const;
	CFStringRef						CopyActionDescriptionShortString() const;

protected:
	
public:
    AMouseMoveAndClick() = default;
	AMouseMoveAndClick( const AMouseMoveAndClick& inAction );

	virtual ~AMouseMoveAndClick();
	
	virtual AMouseMoveAndClick*		Clone() const { return new AMouseMoveAndClick( *this ); }
	virtual AMouseMoveAndClick*		Create() const { return new AMouseMoveAndClick(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kMouseMoveAndClick; }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;
	
	virtual HBaseAction*			Merge( HBaseAction* action );

	void							SetClickCount( UInt32 inClickCount ) { mClickCount = inClickCount; }
	UInt32							GetClickCount() const { return mClickCount; }
	
	void							SetButton( UInt32 inButton ) { mButton = inButton; }
	UInt32							GetButton() const { return mButton; }
	
	void							SetMouseRelative( const MouseMoveAndClickRelativeType& inRelative ) { mMouseRelative = inRelative; }
	MouseMoveAndClickRelativeType	GetMouseRelative() const { return mMouseRelative; }
	
	void							SetMouseRelativeCorner( const MouseMoveAndClickRelativeCornerType& inRelativeCorner ) { mMouseRelativeCorner = inRelativeCorner; }
	MouseMoveAndClickRelativeCornerType		GetMouseRelativeCorner() const { return mMouseRelativeCorner; }
	
	void							SetHorizontalPos( CFStringRef inString ) { mHorizontalPos = inString->Copy(); }
	void							SetHorizontalPos( double inValue ) { mHorizontalPos = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyHorizontalPos() const { return mHorizontalPos->Copy(); }
	NSString*						GetHorizontalPos() const { return mHorizontalPos->GetNS(); }

	void							SetVerticalPos( CFStringRef inString ) { mVerticalPos = inString->Copy(); }
	void							SetVerticalPos( double inValue ) { mVerticalPos = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyVerticalPos() const { return mVerticalPos->Copy(); }
	NSString*						GetVerticalPos() const { return mVerticalPos->GetNS(); }
	
	void							SetModifiers( UInt32 inModifiers ) { mModifiers = inModifiers; }
	UInt32							GetModifiers() const { return mModifiers; }
	
	void							SetMouseDrag( const MouseMoveAndClickDragType inMouseDrag ) { mMouseDrag = inMouseDrag; }
	MouseMoveAndClickDragType		GetMouseDrag() const { return mMouseDrag; }
	
	void							SetDragHorizontalPos( CFStringRef inString ) { mDragHorizontalPos = inString->Copy(); }
	void							SetDragHorizontalPos( double inValue ) { mDragHorizontalPos = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyDragHorizontalPos() const { return mDragHorizontalPos->Copy(); }
	NSString*						GetDragHorizontalPos() const { return mDragHorizontalPos->GetNS(); }
	
	void							SetDragVerticalPos( CFStringRef inString ) { mDragVerticalPos = inString->Copy(); }
	void							SetDragVerticalPos( double inValue ) { mDragVerticalPos = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyDragVerticalPos() const { return mDragVerticalPos->Copy(); }
	NSString*						GetDragVerticalPos() const { return mDragVerticalPos->GetNS(); }
	
	void							SetRestoreMouseLocation( Boolean inRestoreMouseLocation ) { mRestoreMouseLocation = inRestoreMouseLocation; }
	Boolean							GetRestoreMouseLocation() const { return mRestoreMouseLocation; }
	
	NSImage*						GetImage() const { return mImage; }
	void							SetImage( NSImage* inImage );
	
	void							SetFuzz( UInt32 inFuzz ) { mFuzz = inFuzz; }
	UInt32							GetFuzz() const { return mFuzz; }

	void							SetDisplayMatches( Boolean inDisplayMatches ) { mDisplayMatches = inDisplayMatches; }
	Boolean							GetDisplayMatches() const { return mDisplayMatches; }

	NSDictionary*					GetRecordedDetails() const { return mRecordedDetails; }
	void							SetRecordedDetails( NSDictionary* inRecordedDetails ) { mRecordedDetails = [inRecordedDetails copy]; }

#if KMF_IS_ENGINE
	virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual BOOL					IsClickAction() const { return YES; }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const { SaveToDictionary( dictionary, false ); }
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary, BOOL includeRecordedDetails ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	static bool						GetFrontWindowRect( CGRect &windowRect );

};

