/*
 *  AMouseMoveAndClick.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Apr 28 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "XErrorHandler.h"

#import "AMouseMoveAndClick.h"
#import "UCGRemoteControl.h"
#import "XContainer.h"
#import "XAXApplicationElement.h"
#import "XAXWindowElement.h"
#import "Calculate.h"
#import "NSImageExtensions.h"
#import "NSStringExtensions.h"
#import "KMVariables.h"

#if KMF_IS_ENGINE
#import "KMVariables.h"
#import "KMNotifications.h"
#import "ThreadFinished.h"
#import "HExecuteActionListManager.h"
#import "Growl/GrowlApplicationBridge.h"
#import "KMDefines.h"
#import "FindImageWindowController.h"
#endif

#if KMF_EDITOR_WINDOW
#import "AMouseMoveAndClickEditor.h"
#import "TemplateEngine.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef void* CGSConnectionRef;	// abstract representation of CG connection (one per app)
typedef void* CGSWindowRef;		// abstract representation of CG window
extern OSStatus 		CGSGetWindowLevel(CGSConnectionRef connection, CGSWindowRef window, int* level);
extern OSStatus 		CGSGetOnScreenWindowCount(const CGSConnectionRef cid, CGSConnectionRef targetCID, int* outCount); // targetCID = nil -- get all windows
extern OSStatus 		CGSGetOnScreenWindowList(const CGSConnectionRef cid, CGSConnectionRef targetCID, int count, CGSWindowRef* list, int* outCount);
extern OSStatus 		CGSGetScreenRectForWindow(const CGSConnectionRef cid, CGSWindowRef wid, CGRect *rect);
extern CGSConnectionRef	_CGSDefaultConnection();   // to get CGSConnectionRef for current app
extern CGSWindowRef 	GetNativeWindowFromWindowRef(WindowRef window); // to get a CGSWindowRef from Carbon WindowRef
extern WindowRef  		GetWindowRefFromNativeWindow(CGSWindowRef window); // ...and vise versa (may fail if there's no WindowRef for that CFSWindowRef)

#ifdef __cplusplus
}
#endif

AMouseMoveAndClick::AMouseMoveAndClick( const AMouseMoveAndClick& inAction )
	:	HBaseAction( inAction ),
		mClickCount( inAction.mClickCount ),
		mButton( inAction.mButton ),
		mMouseRelative( inAction.mMouseRelative ),
		mMouseRelativeCorner( inAction.mMouseRelativeCorner ),
		mHorizontalPos( inAction.mHorizontalPos ),
		mVerticalPos( inAction.mVerticalPos ),
		mModifiers( inAction.mModifiers ),
		mMouseDrag( inAction.mMouseDrag ),
		mDragHorizontalPos( inAction.mDragHorizontalPos ),
		mDragVerticalPos( inAction.mDragVerticalPos ),
		mRestoreMouseLocation( inAction.mRestoreMouseLocation ),
		mImage( [inAction.mImage copy] ),
		mFuzz( inAction.mFuzz ),
		mDisplayMatches( inAction.mDisplayMatches ),
		mRecordedDetails( [inAction.mRecordedDetails copy] )
{
#if KMF_IS_ENGINE
	mThreadToken = ThreadFinished::kNull;
	mCancelMessage = nil;
#endif
}

AMouseMoveAndClick::~AMouseMoveAndClick()
{
#if KMF_IS_ENGINE
	if ( mThreadToken != ThreadFinished::kNull ) {
		ThreadFinished::AbandonToken( mThreadToken );
	}
#endif
}

HBaseAction*
AMouseMoveAndClick::Merge( HBaseAction* action )
{
	check( action );
	AMouseMoveAndClick* newAction = NULL;
	AMouseMoveAndClick* actualAction = dynamic_cast<AMouseMoveAndClick*>( action );

	if ( !actualAction ) {
		// newAction = NULL;
	} else if ( mClickCount < 1 || mClickCount >= 3 || mClickCount != actualAction->mClickCount-1 ) {
		// newAction = NULL;
	} else if ( mButton != actualAction->mButton ) {
		// newAction = NULL;
	} else if ( (mMouseRelative != MouseMoveAndClickRelativeType::kScreen && mMouseRelative != MouseMoveAndClickRelativeType::kWindow) || mMouseRelative != actualAction->mMouseRelative ) {
		// newAction = NULL;
	} else if ( mMouseRelativeCorner != actualAction->mMouseRelativeCorner ) {
		// newAction = NULL;
	} else if ( !mHorizontalPos->IsEqual( actualAction->mHorizontalPos ) || !mVerticalPos->IsEqual( actualAction->mVerticalPos ) ) {
		// newAction = NULL;
	} else if ( mModifiers != actualAction->mModifiers ) {
		// newAction = NULL;
	} else if ( mMouseDrag != MouseMoveAndClickDragType::kNone || mMouseDrag != actualAction->mMouseDrag ) {
		// newAction = NULL;
	} else if ( mRestoreMouseLocation != actualAction->mRestoreMouseLocation ) {
		// newAction = NULL;
	} else {
		newAction = new AMouseMoveAndClick( *this );
		newAction->mClickCount = actualAction->mClickCount;
	}

	return newAction;
}

CFStringRef
AMouseMoveAndClick::CopyActionDescriptionShortString() const
{	
	CFContainer<CFStringRef> click = CFString::CreateWithFormat( CFSTR("kmMouseActionClick%dTitle"), mClickCount );
	click = click->CopyLocalized();
	if ( mClickCount > 0 ) {
		CFContainer<CFStringRef> buttonclick = CFString::CreateWithFormat( CFSTR("kmMouseActionClickButton%d"), mButton );
		buttonclick = buttonclick->CopyLocalized();
		click = CFString::CreateWithFormat( click, buttonclick.Get() );
	}
	if ( mClickCount > 0 && mMouseDrag != MouseMoveAndClickDragType::kNone ) {
		CFContainer<CFStringRef> andDrag = CFSTR("kmMouseActionAndDragTitle")->CopyLocalized();
		click = CFString::CreateWithFormat( CFSTR("%@ %@"), CFStringRef(click), CFStringRef(andDrag) );
	}
	if ( mClickCount > 0 ) {
		if ( mRestoreMouseLocation ) {
			CFContainer<CFStringRef> nomove = CFSTR("kmMouseActionNoMoveAndTitle")->CopyLocalized();
			click = CFString::CreateWithFormat( nomove, CFStringRef(click) );
		} else {
			CFContainer<CFStringRef> move = CFSTR("kmMouseActionMoveAndTitle")->CopyLocalized();
			click = CFString::CreateWithFormat( move, CFStringRef(click) );
		}
	}

	return click->Copy();
}

CFStringRef
AMouseMoveAndClick::CopyActionDescriptionAtString( bool lowercase ) const
{
	CFContainer<CFStringRef> corner;
	switch ( mMouseRelativeCorner ) {
		case MouseMoveAndClickRelativeCornerType::kTopLeft:
			corner = CFSTR("kmMouseActionTopLeftTitle")->CopyLocalized();
			break;
		case MouseMoveAndClickRelativeCornerType::kTopRight:
			corner = CFSTR("kmMouseActionTopRightTitle")->CopyLocalized();
			break;
		case MouseMoveAndClickRelativeCornerType::kBottomLeft:
			corner = CFSTR("kmMouseActionBottomLeftTitle")->CopyLocalized();
			break;
		case MouseMoveAndClickRelativeCornerType::kBottomRight:
			corner = CFSTR("kmMouseActionBottomRightTitle")->CopyLocalized();
			break;
		case MouseMoveAndClickRelativeCornerType::kCenter:
			corner = CFSTR("kmMouseActionCenterTitle")->CopyLocalized();
			break;
	}

	CFContainer<CFStringRef> relativeObject;
	switch ( mMouseRelative ) {
		case MouseMoveAndClickRelativeType::kScreen:
			relativeObject = CFSTR("kmMouseActionScreenTitle")->CopyLocalized();
			break;
		case MouseMoveAndClickRelativeType::kWindow:
			relativeObject = CFSTR("kmMouseActionWindowTitle")->CopyLocalized();
			break;
		case MouseMoveAndClickRelativeType::kMouse:
			relativeObject = CFSTR("kmMouseActionCurrentPositionTitle")->CopyLocalized();
			break;
		case MouseMoveAndClickRelativeType::kImage:
			relativeObject = CFSTR("kmMouseActionImageTitle")->CopyLocalized();
			break;
	}
	if ( mMouseRelative != MouseMoveAndClickRelativeType::kMouse ) {
		relativeObject = CFString::CreateWithFormat( relativeObject, CFStringRef(corner) );
	}

	if ( lowercase ) {
		CFContainer<CFMutableStringRef> mro = relativeObject->MutableCopy();
		mro->Lowercase();
		relativeObject = mro->Copy();
	}

	CFContainer<CFStringRef> format = CFSTR("kmMouseTitleAtFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, mHorizontalPos.Get(), mVerticalPos.Get(), relativeObject.Get() );
}

CFStringRef
AMouseMoveAndClick::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("kmMouseTitleFormat")->CopyLocalized();
	CFContainer<CFStringRef> click = CopyActionDescriptionShortString();
	CFContainer<CFStringRef> at = CopyActionDescriptionAtString( false );

	CFContainer<CFStringRef> atORto = (mClickCount > 0) ? CFSTR("kmMouseClickAt") : CFSTR("kmMouseJustMoveTo");
	atORto = atORto->CopyLocalized();

	return CFString::CreateWithFormat( format, click.Get(), atORto.Get(), at.Get() );
}

void
AMouseMoveAndClick::SetImage( NSImage* inImage )
{
	mImage = [inImage copy];
}

bool
AMouseMoveAndClick::GetFrontWindowRect( CGRect &windowRect )
{
	bool rectFound = false;

	if ( AXAPIEnabled() ) {
		XAXApplicationElement theFrontApp;
		if ( theFrontApp ) {
			XAXWindowElement windowElement( theFrontApp.GetFocussedOrMainWindow() );
			if ( !!windowElement ) {
				rectFound = windowElement.GetPosition( windowRect.origin ) && windowElement.GetSize( windowRect.size );
			}
		}
		XErrorHandlerMaybeLogFormat2( "MouseMoveAndClick", CFSTR("AXA returns %d window %@"), rectFound, NSStringFromRect( NSRectFromCGRect( windowRect ) ) );
	}
	
	if ( !rectFound ) {
		CGSConnectionRef myConnection;
		OSStatus theErr;
		int windowCount;
		
		myConnection = _CGSDefaultConnection();
		theErr = CGSGetOnScreenWindowCount( myConnection, NULL, &windowCount );
		if ( theErr == noErr ) {
			CGSWindowRef* pWindowList = new CGSWindowRef[windowCount+5];
			theErr = CGSGetOnScreenWindowList( myConnection, NULL, windowCount, pWindowList, &windowCount );
			if ( theErr == noErr ) {
                for ( int i = 0; i < windowCount; i++ ) {
                    int windowLevel;
                    theErr = CGSGetWindowLevel( myConnection, pWindowList[i], &windowLevel );
                    
                    if ( theErr == noErr && windowLevel == kCGNormalWindowLevel ) {
                        theErr = CGSGetScreenRectForWindow( myConnection, pWindowList[i], &windowRect );
                        rectFound = theErr == noErr;
                        break;
                    }
                }
            }
			
			delete [] pWindowList;
		}
		XErrorHandlerMaybeLogFormat2( "MouseMoveAndClick", CFSTR("CGSGetScreenRectForWindow returns %d window %@"), rectFound, NSStringFromRect( NSRectFromCGRect( windowRect ) ) );
	}

	return rectFound;
}

#if KMF_IS_ENGINE

void
AMouseMoveAndClick::CancelThisMacro( NSString* message )
{
	ThreadFinished::AbandonToken( mThreadToken );
	mThreadToken = ThreadFinished::kNull;
	HExecuteActionListManager::CancelThisExecutionGroup();
	[GrowlApplicationBridge 
		 notifyWithTitle:CFBridgingToNS(kmEngineName)
		 description:[NSString stringWithFormat:[@"MacroCancelledFormat" localized], message]
		 notificationName:@"Macro Cancelled"
		 iconData:nil priority:0 isSticky:NO clickContext:nil];		
	KMVariables::Set( kMouseClickResultVariable, message, KMNotifications::kLoadVariableFromEngineAction );
	XErrorHandlerMaybeLogMessage( "display", ([NSString stringWithFormat:[@"MacroCancelledFormat" localized], message]) );
}

void
AMouseMoveAndClick::MatchCompleted( bool match, CGPoint topleft )
{
	if ( match ) {
		mRelativeRectFound = true;
		mRelativeRect.origin = topleft;
		mRelativeRect.size = NSSizeToCGSize( mImage.size );
	} else {
		mCancelMessage = [[@"MouseClickNoImageFound" localized] copy];
	}
	ThreadFinished::AbandonToken( mThreadToken );
	mThreadToken = ThreadFinished::kNull;
}

float
AMouseMoveAndClick::DoAction()
{
	XErrorHandlerMaybeLogMessage( "MouseMoveAndClick", CFSTR("Mouse Move or Click Start" ) );
	
	mRelativeRectFound = false;

	NSDictionary* calculationContext = nil;
	
	KMVariables::Set( kMouseClickResultVariable, @"OK", KMNotifications::kLoadVariableFromEngineAction );
	switch( mMouseRelative ) {
		case MouseMoveAndClickRelativeType::kMouse:
			HIGetMousePosition( kHICoordSpaceScreenPixel, NULL, &mRelativeRect.origin ); // kHICoordSpace72DPIGlobal?
			mRelativeRect.size = CGSizeZero;
			mRelativeRectFound = true;
			break;
		case MouseMoveAndClickRelativeType::kWindow:
			mRelativeRectFound = GetFrontWindowRect( mRelativeRect );
			if ( mRelativeRectFound ) {
				calculationContext = @{ @"WINDOW" : NSStringFromRect( mRelativeRect ) };
			} else {
				CancelThisMacro( [@"MouseClickNoFrontWindow" localized] );
			}
			XErrorHandlerMaybeLogFormat2( "MouseMoveAndClick", CFSTR("Mouse Move or Click relative to %d window rectFound %@"), mRelativeRectFound, NSStringFromRect( NSRectFromCGRect( mRelativeRect ) ) );
			break;
		case MouseMoveAndClickRelativeType::kScreen:
			mRelativeRect = CGDisplayBounds( kCGDirectMainDisplay );
			mRelativeRectFound = true;
			XErrorHandlerMaybeLogFormat1( "MouseMoveAndClick", CFSTR("Mouse Move or Click relative to main screen rectFound %@"), NSStringFromRect( NSRectFromCGRect( mRelativeRect ) ) );
			break;
		case MouseMoveAndClickRelativeType::kImage:
			NSRect imageRect;
			imageRect.origin = NSZeroPoint;
			imageRect.size = mImage.size;
			calculationContext = @{ @"IMAGE" : NSStringFromRect(imageRect) };
			if ( mImage ) {
				mThreadToken = ThreadFinished::Get();
				ThreadFinished::Token token = mThreadToken; // private variable, not part of possibly deleted object
				bool displayMatches = mDisplayMatches;
				MatchImage::MatchDisplay( mImage, mFuzz, false, ^( bool match, bool match_without_duplicates, const MatchImage::Results results ) {
					if ( !ThreadFinished::IsFinished( token ) ) {
						if ( displayMatches ) {
							[FindImageWindowController addMatches:results matchWithoutDuplicates:match_without_duplicates requiresUniqueMatch:true];
						}
						CGPoint topleft = results.size() > 0 ? results[0].frame.origin : CGPointMake(0,0);
						MatchCompleted( match, topleft );
					}
				});
			} else {
				CancelThisMacro( [@"MouseClickNoImage" localized] );
			}
			break;
	}

	if ( !Calculate::CalculateValue( mHorizontalPos->GetNS(), mHorizontalPosValue, calculationContext ) ) {
		CancelThisMacro( [NSString stringWithFormat:[@"MouseClickCalculateFailed" localized], mHorizontalPos.Get()] );
	} else if ( !Calculate::CalculateValue( mVerticalPos->GetNS(), mVerticalPosValue, calculationContext ) ) {
		CancelThisMacro( [NSString stringWithFormat:[@"MouseClickCalculateFailed" localized], mVerticalPos.Get()] );
	} else if ( mMouseDrag != MouseMoveAndClickDragType::kNone && !Calculate::CalculateValue( mDragHorizontalPos->GetNS(), mDragHorizontalPosValue, calculationContext ) ) {
		CancelThisMacro( [NSString stringWithFormat:[@"MouseClickCalculateFailed" localized], mDragHorizontalPos.Get()] );
	} else if ( mMouseDrag != MouseMoveAndClickDragType::kNone && !Calculate::CalculateValue( mDragVerticalPos->GetNS(), mDragVerticalPosValue, calculationContext ) ) {
		CancelThisMacro( [NSString stringWithFormat:[@"MouseClickCalculateFailed" localized], mDragVerticalPos.Get()] );
	} else {
		mHorizontalPosValue = roundf( mHorizontalPosValue );
		mVerticalPosValue = roundf( mVerticalPosValue );
		mDragHorizontalPosValue = roundf( mDragHorizontalPosValue );
		mDragVerticalPosValue = roundf( mDragVerticalPosValue );
	}

	return 2;
}

Boolean
AMouseMoveAndClick::IsActionCompleted()
{
	if ( mThreadToken == ThreadFinished::kNull ) {
		if ( mRelativeRectFound ) {
			CGPoint mouseRelativePoint;
			
			mouseRelativePoint = mRelativeRect.origin;
			switch ( mMouseRelativeCorner ) {
				case MouseMoveAndClickRelativeCornerType::kTopLeft:
					// do nothing, already have it
					break;
				case MouseMoveAndClickRelativeCornerType::kTopRight:
					mouseRelativePoint.x += mRelativeRect.size.width;
					break;
				case MouseMoveAndClickRelativeCornerType::kBottomLeft:
					mouseRelativePoint.y += mRelativeRect.size.height;
					break;
				case MouseMoveAndClickRelativeCornerType::kBottomRight:
					mouseRelativePoint.x += mRelativeRect.size.width;
					mouseRelativePoint.y += mRelativeRect.size.height;
					break;
				case MouseMoveAndClickRelativeCornerType::kCenter:
					mouseRelativePoint.x += roundf( mRelativeRect.size.width / 2 );
					mouseRelativePoint.y += roundf( mRelativeRect.size.height / 2 );
					break;
			}
			
			XErrorHandlerMaybeLogFormat2( "MouseMoveAndClick", CFSTR("Mouse Move or Click relative to point %d,%d"), (int)mouseRelativePoint.x, (int)mouseRelativePoint.y );
			
			mouseRelativePoint.x = roundf( mouseRelativePoint.x );
			mouseRelativePoint.y = roundf( mouseRelativePoint.y );
			CGPoint mousePoint = mouseRelativePoint;
			mousePoint.x += mHorizontalPosValue;
			mousePoint.y += mVerticalPosValue;
			
			CGPoint dragPoint;
			switch ( mMouseDrag ) {
				case MouseMoveAndClickDragType::kNone:
					dragPoint = mousePoint;
					break;
				case MouseMoveAndClickDragType::kRelative:
					dragPoint = mousePoint;
					dragPoint.x += mDragHorizontalPosValue;
					dragPoint.y += mDragVerticalPosValue;
					break;
				case MouseMoveAndClickDragType::kTo:
					dragPoint = mouseRelativePoint;
					dragPoint.x += mDragHorizontalPosValue;
					dragPoint.y += mDragVerticalPosValue;
					break;
			}
			
			XErrorHandlerMaybeLogFormat2( "MouseMoveAndClick", CFSTR("Mouse Move or Click adjusted to point %d,%d"), (int)mousePoint.x, (int)mousePoint.y );
			XErrorHandlerMaybeLogFormat2( "MouseMoveAndClick", CFSTR("Mouse Move or Click drag to point %d,%d"), (int)dragPoint.x, (int)dragPoint.y );
			bool move = !mRestoreMouseLocation || mClickCount == 0;
			UCGRemoteControl::PostMouse( GetModifiers(), mousePoint, dragPoint, move, mClickCount, mButton );
			XErrorHandlerMaybeLogMessage( "MouseMoveAndClick", CFSTR("Mouse Move or Click Finished" ) );
		} else {
			check( mCancelMessage );
			if ( mCancelMessage ) {
				CancelThisMacro( mCancelMessage );
			}
		}
	}
	return mThreadToken == ThreadFinished::kNull;
}

void
AMouseMoveAndClick::ActionAborted()
{
	if ( mThreadToken != ThreadFinished::kNull ) {
		ThreadFinished::AbandonToken( mThreadToken );
		mThreadToken = ThreadFinished::kNull;
	}
}

#endif

#if KMF_EDITOR_WINDOW

Class
AMouseMoveAndClick::ActionController() const
{
	return [AMouseMoveAndClickEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
{% if modifiers %}With {{ modifiers | htmlencode }} at{% else %}At{% /if %} {{ at | htmlencode }}.<br />
{% if image %}
<img src={{ image | inlineimage: -1 }} alt="Search Image" width="{{ image | imagewidth }}" height="{{ image | imageheight }}" /> Fuzz: {{ fuzz }}%<br />
{% /if %}
{% if drag %}And drag {{ drag }} ({{ draghorizontal | htmlencode }},{{ dragvertical | htmlencode }}).<br />{% /if %}
Store action result in variable ‘{{ result | htmlencode }}’.
)RAW";

NSString*
AMouseMoveAndClick::GetActionHTMLDescriptionString() const
{
	CFContainer<CFStringRef> shortstr = CopyActionDescriptionShortString();
	return shortstr->GetNS();
}

NSString*
AMouseMoveAndClick::GetActionExtraHTML() const
{
	CFContainer<CFStringRef> at = CopyActionDescriptionAtString( true );
	NSMutableArray* modifiersa = [NSMutableArray new];
	if ( mModifiers & controlKey ) [modifiersa addObject:@"Control"];
	if ( mModifiers & optionKey ) [modifiersa addObject:@"Option"];
	if ( mModifiers & shiftKey ) [modifiersa addObject:@"Shift"];
	if ( mModifiers & cmdKey ) [modifiersa addObject:@"Command"];
	NSString* modifiers = [modifiersa componentsJoinedByString:@","];
	NSString* drag;
	switch ( mMouseDrag ) {
		case MouseMoveAndClickDragType::kNone:
			drag = @"";
			break;
		case MouseMoveAndClickDragType::kRelative:
			drag = @"relative";
			break;
		case MouseMoveAndClickDragType::kTo:
			drag = @"to";
			break;
	}
	NSImage* image = (id)[NSNull null];
	if ( mImage && mMouseRelative == MouseMoveAndClickRelativeType::kImage ) {
		image = [mImage imageForHTMLDisplay];
	}
	NSDictionary *variables = @{
									@"at" : at->GetNS(),
									@"modifiers" : modifiers,
									@"draghorizontal" : mDragHorizontalPos->GetNS(),
									@"dragvertical" : mDragVerticalPos->GetNS(),
									@"drag" : drag,
									@"image" : image,
									@"fuzz" : @(mFuzz),
									@"result" : kMouseClickResultVariable,
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

enum class MouseMoveAndClickActionType {
	kMove,
	kClick, 
	kMoveAndClick,
	kDEFAULT = kMoveAndClick,
};
#define kMouseActionTypeNamesArray {CFSTR("Move"), CFSTR("Click"), CFSTR("MoveAndClick"), NULL}

static const CFStringRef kMouseActionTypeNames[] = kMouseActionTypeNamesArray;
static const CFStringRef kMouseRelativeTypeNames[] = kMouseRelativeTypeNamesArray;
static const CFStringRef kMouseRelativeCornerTypeNames[] = kMouseRelativeCornerTypeNamesArray;
static const CFStringRef kMouseDragTypeNames[] = kMouseDragTypeNamesArray;

void
AMouseMoveAndClick::SaveToDictionary( CFMutableDictionaryRef dictionary, BOOL includeRecordedDetails ) const
{
	// Old Style Action
	if ( mClickCount == 0 ) {
		dictionary->Set( CFSTR("Action"), kMouseActionTypeNames[int(MouseMoveAndClickActionType::kMove)] );
	} else if ( mRestoreMouseLocation ) {
		dictionary->Set( CFSTR("Action"), kMouseActionTypeNames[int(MouseMoveAndClickActionType::kClick)] );
	} else {
		dictionary->Set( CFSTR("Action"), kMouseActionTypeNames[int(MouseMoveAndClickActionType::kMoveAndClick)] );
	}
	
	dictionary->SetNumber( CFSTR("ClickCount"), mClickCount );
	dictionary->SetNumber( CFSTR("Button"), mButton );
	
	dictionary->Set( CFSTR("Relative"), kMouseRelativeTypeNames[int(mMouseRelative)] );
	dictionary->Set( CFSTR("RelativeCorner"), kMouseRelativeCornerTypeNames[int(mMouseRelativeCorner)] );

	dictionary->Set( CFSTR("HorizontalPositionExpression"), CFStringRef(mHorizontalPos) );
	dictionary->Set( CFSTR("VerticalPositionExpression"), CFStringRef(mVerticalPos) );

	dictionary->SetNumber( CFSTR("Modifiers"), mModifiers );

	dictionary->Set( CFSTR("MouseDrag"), kMouseDragTypeNames[int(mMouseDrag)] );
	dictionary->Set( CFSTR("DragHorizontalPosition"), CFStringRef(mDragHorizontalPos) );
	dictionary->Set( CFSTR("DragVerticalPosition"), CFStringRef(mDragVerticalPos) );

	dictionary->SetBoolean( CFSTR("RestoreMouseLocation"), mRestoreMouseLocation );

	if ( includeRecordedDetails && mRecordedDetails ) {
		dictionary->Set( CFSTR("RecordedDetails"),  CFBridgingToCF( mRecordedDetails ) );
	}

	[mImage saveToDictionary:dictionary];
	dictionary->SetNumber( CFSTR("Fuzz"), mFuzz );
	dictionary->SetBoolean( CFSTR("DisplayMatches"), mDisplayMatches );
}

void
AMouseMoveAndClick::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	MouseMoveAndClickActionType oldAction = dictionary->GetEnumClass( CFSTR("Action"), kMouseActionTypeNames, MouseMoveAndClickActionType::kDEFAULT );
	
	SInt32 intvalue;
	if ( !dictionary->Get( CFSTR("ClickCount"), mClickCount ) ) {
		mClickCount = (oldAction == MouseMoveAndClickActionType::kMove) ? 0 : 1;
	}
	if ( !dictionary->Get( CFSTR("Button"), mButton ) ) mButton = kCGMouseButtonLeft;
	
	mMouseRelative = dictionary->GetEnumClass( CFSTR("Relative"), kMouseRelativeTypeNames, MouseMoveAndClickRelativeType::kDEFAULT );
	mMouseRelativeCorner = dictionary->GetEnumClass( CFSTR("RelativeCorner"), kMouseRelativeCornerTypeNames, MouseMoveAndClickRelativeCornerType::kDEFAULT );
	if ( !dictionary->Get( CFSTR("Modifiers"), mModifiers ) ) mModifiers = 0;

	CFStringRef value;
	dictionary->Get( CFSTR("HorizontalPositionExpression"), value, CFStringRef(NULL) );
	mHorizontalPos = value ? value->Copy() : NULL;
	
	if ( !mHorizontalPos && dictionary->Get( CFSTR("HorizontalPosition"), intvalue ) ) {
		mHorizontalPos = CFString::CreateWithFormat( CFSTR("%d"), intvalue );
	}
	
	if ( !mHorizontalPos ) {
		mHorizontalPos = CFSTR("0");
	}

	dictionary->Get( CFSTR("VerticalPositionExpression"), value, CFStringRef(NULL) );
	mVerticalPos = value ? value->Copy() : NULL;
	
	if ( !mVerticalPos && dictionary->Get( CFSTR("VerticalPosition"), intvalue ) ) {
		mVerticalPos = CFString::CreateWithFormat( CFSTR("%d"), intvalue );
	}
	
	if ( !mVerticalPos ) {
		mVerticalPos = CFSTR("0");
	}
	
	mMouseDrag = dictionary->GetEnumClass( CFSTR("MouseDrag"), kMouseDragTypeNames, MouseMoveAndClickDragType::kDEFAULT );

	dictionary->Get( CFSTR("DragHorizontalPosition"), value, CFSTR("0") );
	check( value );
	mDragHorizontalPos = value->Copy();
	dictionary->Get( CFSTR("DragVerticalPosition"), value, CFSTR("0") );
	check( value );
	mDragVerticalPos = value->Copy();
	
	dictionary->GetBoolean( CFSTR("RestoreMouseLocation"), mRestoreMouseLocation, oldAction == MouseMoveAndClickActionType::kClick );

	CFDictionaryRef d;
	dictionary->Get( CFSTR("RecordedDetails"), d, (CFDictionaryRef)NULL );
	if ( d ) {
		mRecordedDetails = [CFBridgingToNS( d ) copy];
	} else {
		mRecordedDetails = nil;
	}

	mImage = [[NSImage imageFromDictionary:dictionary] copy];
	if ( !dictionary->Get( CFSTR("Fuzz"), mFuzz ) ) mFuzz = MatchImage::kDefaultFuzz;

	dictionary->GetBoolean( CFSTR("DisplayMatches"), mDisplayMatches, false );
}
