//
//  ANewFolder.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 11 Apr 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "ANewFolder.h"
#import "FileReference.h"
#import "NSStringExtensions.h"
#import "NSObjectExtensions.h"
#import "XErrorHandler.h"
#import "NSStringRegularExpressions.h"

#if KMF_EDITOR_WINDOW
#import "ANewFolderEditor.h"
#endif

CFStringRef
ANewFolder::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("kmNewFolderTitleLongFormat")->CopyLocalized();
	CFContainer<CFStringRef> result = CFString::CreateWithFormat( format, mDestination );
	return result->Copy();
}

#if KMF_IS_ENGINE

float
ANewFolder::DoAction()
{
	float time = 10;

	NSString* destination = FileReference::PathToActualPath( mDestination );
	if ( ![destination isMatchedByRegex:@"^/.+"] ) {
		XErrorHandlerMaybeLogMessage( "display", CFSTR("No file destination specified for New Folder action") );
		return 0;
	}
	NSError* error = nil;
	BOOL result = [[NSFileManager defaultManager] createDirectoryAtPath:destination withIntermediateDirectories:NO attributes:nil error:&error];
	if ( !result ) {
		XErrorHandlerMaybeLogFormat2( "display", CFSTR("Create folder %@ failed with error %@"), destination, error );
	}
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ANewFolder::ActionController() const
{
	return [ANewFolderEditor class];
}

#endif

#pragma mark streaming

void
ANewFolder::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	FileReference::SavePathToDictionary( dictionary, mDestination, CFSTR("Destination") );
}

void
ANewFolder::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFContainer<CFStringRef> dest;
	FileReference::LoadPathFromDictionary( dictionary, dest, NULL, CFSTR("Destination") );
	mDestination = dest->GetNS();
}
