//
//  ANewImage.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 4 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "ANewImage.h"
#import "NSColorExtensions.h"
#import "Calculate.h"
#import "XErrorHandler.h"

#if KMF_EDITOR_WINDOW
#import "ANewImageEditor.h"
#import "NSStringExtensions.h"
#import "TemplateEngine.h"
#import "NSImageExtensions.h"
#endif
#if KMF_IS_ENGINE
#import "XClipboardDataSet.h"
#import "XClipboardHistory.h"
#endif

const CFStringRef kDefaultWidth = CFSTR("100");
const CFStringRef kDefaultHeight = CFSTR("100");

static const int kDefaultAlpha = 100;
static const RGBColor kDefaultColor = { 0, 152, 255 };

ANewImage::ANewImage()
	:	HBaseAction(),
		mWidthNumber( kDefaultWidth ),
		mHeightNumber( kDefaultHeight )
{
	mColor = [NSColor colorWithCalibratedRed:kDefaultColor.red/255.0 green:kDefaultColor.green/255.0 blue:kDefaultColor.blue/255.0 alpha:kDefaultAlpha/100.0];
}

ANewImage::ANewImage( const ANewImage& inAction )
	:	HBaseAction( inAction ),
		mTarget( inAction.mTarget ),
		mWidthNumber( inAction.mWidthNumber ),
		mHeightNumber( inAction.mHeightNumber ),
		mColor( [inAction.mColor copy] )
{
}
	
ANewImage::~ANewImage()
{
}

void
ANewImage::SetColor( NSColor* inColor )
{
	mColor = [[inColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] copy];
}

#if KMF_IS_ENGINE

float
ANewImage::DoAction()
{
	double width,height;
	if ( !Calculate::CalculateValue( mWidthNumber->GetNS(), width ) ) {
		XErrorHandlerMaybeLogFormat1("always", CFSTR("New Image %@ failed"), mWidthNumber->GetNS() );
	} else 	if ( !Calculate::CalculateValue( mHeightNumber->GetNS(), height ) ) {
		XErrorHandlerMaybeLogFormat1("always", CFSTR("New Image %@ failed"), mHeightNumber->GetNS() );
	} else if ( width < 0.6 || height < 0.6 || width > 10000 || height > 10000 ) {
		XErrorHandlerMaybeLogFormat2("always", CFSTR("Invalid New Image size %f,%f"), (float)width, (float)height );
	} else {
		NSImage* image = [[NSImage alloc] initWithSize:NSMakeSize( round(width), round(height) )];
		if ( image ) {
			[image lockFocus];
			[mColor set];
			[NSBezierPath fillRect:NSMakeRect(0,0,image.size.width,image.size.height)];
			[image unlockFocus];
			mTarget.WriteClipboardDataSet( XClipboardDataSet( image ) );
		}
	}
	
	return 10;
}


#endif

#if KMF_EDITOR_WINDOW

Class
ANewImage::ActionController() const
{
	return [ANewImageEditor class];
}

NSString*
ANewImage::GetActionHTMLDescriptionString() const
{
	NSString* format = [@"kmNewImageHTMLDescriptionTitleFormat" localized];
	return [NSString stringWithFormat:format, mTarget.GetDisplayName()];
}

static NSString* kActionDisplayHTML = @R"RAW(
<span class="IconLabel">Size ({{ width | htmlencode }},{{ height | htmlencode }}) and color</span> </span><img src={{ color | inlineimage: 16 }} alt="Swatch" width="16" height="16" />
)RAW";

NSString*
ANewImage::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"width" : mWidthNumber->GetNS(),
									@"height" : mHeightNumber->GetNS(),
									@"color" : [NSImage imageWithSwatch:mColor],
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
ANewImage::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	mTarget.SaveToDictionary( dictionary, CFSTR("Target") );
	dictionary->Set( CFSTR("Width"), CFStringRef(mWidthNumber) );
	dictionary->Set( CFSTR("Height"), CFStringRef(mHeightNumber) );
	[mColor saveWithAlphaToDictionary:dictionary];
}

void
ANewImage::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mTarget.InitializeFromDictionary( dictionary, CFSTR("Target") );
	CFStringRef value;
	dictionary->Get( CFSTR("Width"), value, kDefaultWidth );
	check( value );
	mWidthNumber = value->Copy();
	dictionary->Get( CFSTR("Height"), value, kDefaultHeight );
	check( value );
	mHeightNumber = value->Copy();
	mColor = [[NSColor colorFromDictionary:dictionary defaultColor:kDefaultColor defaultAlpha:kDefaultAlpha] copy];
}
