//
//  ANotification.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 27 Nov 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "ANotification.h"
#import "NSStringExtensions.h"
#import "NSObjectExtensions.h"
#if KMF_IS_ENGINE
#import "TokenText.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ANotificationEditor.h"
#import "TemplateEngine.h"
#endif

ANotification::ANotification()
	:	HBaseAction()
{
	mTitle = [@"kNotificationDefaultTitle" localized];
	mSubtitle = [@"kNotificationDefaultSubtitle" localized];
	mText = [@"kNotificationDefaultText" localized];
	mSoundName = @"";
}

CFStringRef
ANotification::CopyActionDescriptionString() const
{
	return CFString::CreateWithFormat( CFCS( CFSTR("hkNotificationTitle")->CopyLocalized() ), mTitle );
}

#if KMF_IS_ENGINE

float
ANotification::DoAction()
{
	float t1 = 0;
	float t2 = 0;
	float t3 = 0;

	NSString* dTitle = TokenText::GetResult( mTitle, &t1 );
	NSString* dSubtitle = TokenText::GetResult( mSubtitle, &t2 );
	NSString* dText = TokenText::GetResult( mText, &t3 );

	NSUserNotification* notification = [NSUserNotification new];
	notification.title = dTitle;
	notification.subtitle = dSubtitle;
	notification.informativeText = dText;
	if ( !IsEmpty( mSoundName ) ) {
		notification.soundName = mSoundName;
	}
	[[NSUserNotificationCenter defaultUserNotificationCenter] deliverNotification:notification];
	return 20 + t1 + t2 + t3;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ANotification::ActionController() const
{
	return [ANotificationEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
{% if subtitle %}Subtitle: {{ subtitle | htmlencode }}.<br />{% /if %}
{% if soundName %}Play Sound: {{ soundName | htmlencode }}.<br />{% /if %}
{{ text }}
)RAW";

NSString*
ANotification::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"subtitle" : mSubtitle,
									@"soundName" : mSoundName,
									@"text" : UserTextForHTML( mText ),
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
ANotification::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Title"), CFBridgingToCFString(mTitle) );
	dictionary->Set( CFSTR("Subtitle"), CFBridgingToCFString(mSubtitle) );
	dictionary->Set( CFSTR("Text"), CFBridgingToCFString(mText) );
	dictionary->Set( CFSTR("SoundName"), CFBridgingToCFString(mSoundName) );
}

void
ANotification::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef newString;
	dictionary->Get( CFSTR("Title"), newString, CFBridgingToCFString([@"kNotificationDefaultTitle" localized]) );
	check( newString );
	mTitle = newString->GetNS();

	dictionary->Get( CFSTR("Subtitle"), newString, CFBridgingToCFString([@"kNotificationDefaultSubtitle" localized]) );
	check( newString );
	mSubtitle = newString->GetNS();

	dictionary->Get( CFSTR("Text"), newString, CFBridgingToCFString([@"kNotificationDefaultText" localized]) );
	check( newString );
	mText = newString->GetNS();

	dictionary->Get( CFSTR("SoundName"), newString, CFSTR("") );
	check( newString );
	mSoundName = newString->GetNS();
}
