/*
 *  AOpen1File.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sat Apr 27 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "XApplicationIdentifier.h"

class AOpen1File
	:	public HBaseAction
{

private:
	
	CFContainer<CFStringRef>		mPath = CFSTR("");
	Boolean							mIsDefaultApplication = true;
	XApplicationIdentifier			mApplication;
	
#if KMF_IS_ENGINE	
	EventTime						mActionTime;
#endif
	
public:

    AOpen1File() = default;
	AOpen1File( const AOpen1File& inAction ) = default;
    virtual ~AOpen1File() = default;

	virtual AOpen1File*				Clone() const { return new AOpen1File( *this ); }
	virtual AOpen1File*				Create() const { return new AOpen1File(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual	XIcon					GetActionDescriptionXIcon() const;

    virtual HActionType				GetActionType() const { return HActionType::kOpen1File; }
	
	CFStringRef						CopyPath() const { return mPath->Copy(); }
	NSString*						GetPath() const { return mPath->GetNS(); }
	void							SetPath( CFStringRef path ) { mPath = path->Copy(); }

    virtual Boolean					IsDefaultApplication() const { return mIsDefaultApplication; }
	virtual void					SetIsDefaultApplication( const Boolean& inIsDefaultApplication ) { mIsDefaultApplication = inIsDefaultApplication; }
	
	virtual const XApplicationIdentifier&	GetApplication() const { return mApplication; }
	virtual void							SetApplication( const XApplicationIdentifier& inApp ) { mApplication = inApp; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	Boolean							IsActionCompleted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

