/*
 *  AOpen1File.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sat Apr 27 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "AOpen1File.h"
#import "KMIconRegistry.h"
#import "KMPreferences.h"
#import "FileReference.h"
#import "NSObjectExtensions.h"
#import "NSStringExtensions.h"
#if KMF_EDITOR_WINDOW
#import "AOpen1FileEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
AOpen1File::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> theFormatStr = CFSTR("kmOpen1FileTitleFormat")->CopyLocalized();
	return CFString::CreateWithFormat( theFormatStr, mPath.Get() );
}

XIcon
AOpen1File::GetActionDescriptionXIcon() const
{
	XIcon result;

	NSString* path = [FileReference::PathToActualPath( mPath->GetNS() ) existsPath];
	if ( path ) {
		result = XIcon( path, KMIconRegistry::kGenericDocumentIcon );
	} else {
		result = GetActionXIcon();
	}
	check( result.Defined() );
	return result;
}

#if KMF_IS_ENGINE

static const float kOpenFileDelayTime = 0.25;
static float gOpenFileDelay;

float
AOpen1File::DoAction()
{
	mActionTime = GetCurrentEventTime();
	Get( KMPreferences::kOpenFileDelay, &gOpenFileDelay, kOpenFileDelayTime );
	
	NSString* path = [FileReference::PathToActualPath( mPath->GetNS() ) existsPath];
	NSString* app = mApplication.GetPath();
	
	if ( !IsEmpty( path ) ) {
		if ( IsEmpty( app ) ) {
			[[NSWorkspace sharedWorkspace] openFile:path];
		} else {
			[[NSWorkspace sharedWorkspace] openFile:path withApplication:app];
		}
	}
	
	return 10;
}

Boolean
AOpen1File::IsActionCompleted()
{
	return GetCurrentEventTime() > mActionTime + gOpenFileDelay;
}
#endif

#if KMF_EDITOR_WINDOW

Class
AOpen1File::ActionController() const
{
	return [AOpen1FileEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
<span class="IconTitle">With</span> {{ application }}
)RAW";

NSString*
AOpen1File::GetActionExtraHTML() const
{
	if ( mIsDefaultApplication ) return @"";
	NSDictionary *variables = @{
							@"application" : mApplication.GetDisplayHTMLContent(),
						};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
AOpen1File::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	FileReference::SavePathToDictionary( dictionary, mPath->GetNS(), CFSTR("Path") );
	dictionary->SetBoolean( CFSTR("IsDefaultApplication"), mIsDefaultApplication );
	SafeCFPropertyListRef app = mApplication.CopyToPropertyList();
	dictionary->Set( CFSTR("Application"), app );
	CFRelease( app );
}

void
AOpen1File::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	FileReference::LoadPathFromDictionary( dictionary, mPath, CFSTR("NewFile"), CFSTR("Path") );
	dictionary->GetBoolean( CFSTR("IsDefaultApplication"), mIsDefaultApplication, true );
	SafeCFPropertyListRef app = (SafeCFPropertyListRef)dictionary->Get( CFSTR("Application") );
	if ( app ) {
		mApplication.LoadFromPropertyList( app );
	}
	if ( !mApplication.HasValue() ) {
		mIsDefaultApplication = true;
	}
}
