//
//  AOpenFinderSelection.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 10 May 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "AOpenFinderSelection.h"
#import "KMIconRegistry.h"
#import "NSStringExtensions.h"
#if KMF_IS_ENGINE
#import "XRunningApplicationsList.h"
#import "NSAppleScriptExtensions.h"
#import "CFinderSelectionCollection.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AOpenFinderSelectionEditor.h"
#endif

CFStringRef
AOpenFinderSelection::CopyActionDescriptionString() const
{
	NSString* theNameStr;
	if ( GetApplicationTargetType() == ApplicationTargetApplicationType::kFront ) {
		theNameStr = [@"kOpenFinderSelectionTargetCurrentApplication" localized];
	} else {
		theNameStr = mTargetApplication.GetDisplayName();
	}
	CFContainer<CFStringRef> theFormatStr = CFSTR("kmOpenFinderSelectionTitleFormat")->CopyLocalized();
	return CFString::CreateWithFormat( theFormatStr, theNameStr );
}

XIcon
AOpenFinderSelection::GetActionDescriptionXIcon() const
{
	return mTargetApplication.GetXIcon();
}

#if KMF_IS_ENGINE

float
AOpenFinderSelection::DoAction()
{
	XApplicationIdentifier app = mTargetApplication.GetCurrentlyTargetedApplication();
	NSString* bundleid = app.GetBundleIdentifier();
	CFinderSelectionCollection::FixFinderSelectionBug();
	if ( bundleid && [bundleid isEqualToString:@"com.apple.finder"] ) {
		(void)[NSAppleScript executeWithSource:@"tell application \"Finder\" to open (the selection as alias list)"];
	} else {
		NSString* apppath = app.GetPath();
		if ( apppath ) {
			CFContainer<CFURLRef> url = CFURL::Create( CFBridgingToCFString(apppath), kCFURLPOSIXPathStyle, true );
			if ( url.Defined() ) {
				CFContainer<CFStringRef> path = url->CopyFileSystemPath( kCFURLHFSPathStyle );
				if ( path.Defined() ) {
					(void)[NSAppleScript executeWithSource:@"tell application \"Finder\" to open (the selection as alias list) using application file %@" param:path->GetNS()];
				}
			}
		}
	}
	
	
	return 10;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AOpenFinderSelection::ActionController() const
{
	return [AOpenFinderSelectionEditor class];
}

#endif

#pragma mark streaming

void
AOpenFinderSelection::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	mTargetApplication.SaveToDictionary( dictionary, CFSTR("TargetingType"), CFSTR("TargetApplication") );
}

void
AOpenFinderSelection::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{

	Boolean isCurrent;
	if ( dictionary->GetBoolean( CFSTR("IsCurrentApplication"), isCurrent ) ) {
		mTargetApplication.SetApplicationTargetFrontApplication();
		if ( !isCurrent ) {
			SafeCFPropertyListRef app = (SafeCFPropertyListRef)dictionary->Get( CFSTR("Application") );
			if ( app ) {
				XApplicationIdentifier application;
				application.LoadFromPropertyList( app );
				if ( application.HasValue() ) {
					mTargetApplication.SetApplicationTargetApplication( app );
				}
			}
		}
	} else {
		mTargetApplication.InitializeFromDictionary( dictionary, CFSTR("TargetingType"), CFSTR("TargetApplication") );
	}
}
