/*
 *  AOpenPrefPane.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Mon Mar 18 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "AOpenPrefPane.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "XIcon.h"
#import "NSStringExtensions.h"
#import "XErrorHandler.h"
#if KMF_IS_EDITOR
#import "NSObjectExtensions.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AOpenPrefPaneEditor.h"
#endif

AOpenPrefPane::AOpenPrefPane()
	:	HBaseAction()
{
}

AOpenPrefPane::AOpenPrefPane( const AOpenPrefPane& inAction )
	:	HBaseAction( inAction ),
		mAlias( inAction.mAlias )
{
}

AOpenPrefPane::~AOpenPrefPane()
{
}

#if KMF_EDITOR_WINDOW
static NSInteger NameOrder( id obj1, id obj2, void *context )
{
	return [[obj1 lastPathComponent] compare:[obj2 lastPathComponent]];
}

void
AOpenPrefPane::UpdateToDefaultValues()
{
	if ( !mAlias.Valid() ) {
		NSMutableArray* prefPanes = [AOpenPrefPaneEditor prefPanePaths];
		if ( !IsEmpty( prefPanes ) ) {
			[prefPanes sortUsingFunction:NameOrder context:NULL];
			NSString* p = [prefPanes objectAtIndex:0];
			if ( [p existsPath] ) {
				mAlias = p;
			}
		}
	}
}
#endif

XIcon
AOpenPrefPane::GetActionDescriptionXIcon() const
{
	XIcon result;
	NSString* path = mAlias.GetPath();
	if ( path ) {
		result = XIcon( path, KMIconRegistry::kNone );
	}
	if ( !result ) {
		result = GetActionXIcon();
	}
	return result;
}

CFStringRef
AOpenPrefPane::CopyActionDescriptionString() const
{
	NSString* name = [mAlias.GetPath() bundleName];
	if ( !name ) {
		name = @"Unknown";
	}

	CFContainer<CFStringRef> theFormatStr = CFSTR("kmOpenPrefPaneTitleFormat")->CopyLocalized();
	
	return CFString::CreateWithFormat( theFormatStr, name );
}

#if KMF_IS_ENGINE

float
AOpenPrefPane::DoAction()
{
	NSString* prefpath = GetFilePath();
	NSString* sysprefpath = [[[NSWorkspace sharedWorkspace] URLForApplicationWithBundleIdentifier:@"com.apple.systempreferences"] path];

	if ( sysprefpath && prefpath ) {
		[[NSWorkspace sharedWorkspace] openFile:prefpath withApplication:sysprefpath];
	} else {
		XErrorHandlerMaybeLogFormat2( "always", CFSTR("AOpenPrefPane::DoAction failed (pref is %@, syspref is %@)"), prefpath, sysprefpath );
	}
	return 10;
}

Boolean
AOpenPrefPane::IsActionCompleted()
{
	for ( NSRunningApplication* app in [NSRunningApplication runningApplicationsWithBundleIdentifier:@"com.apple.systempreferences"] ) {
		if ( app.active ) {
			return true;
		}
	}
	return false;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AOpenPrefPane::ActionController() const
{
	return [AOpenPrefPaneEditor class];
}

#endif
#pragma mark streaming

void
AOpenPrefPane::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	mAlias.SaveToDictionary( dictionary, CFSTR("NewFile") );
}

void
AOpenPrefPane::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mAlias.LoadFromDictionary( dictionary, CFSTR("NewFile") );
}
