/*
 *  AOpenURL.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sat Dec 15 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "XApplicationIdentifier.h"
#import "NSStringExtensions.h"

class AOpenURL
	:	public HBaseAction
{
private:

	NSString*						mURL = [@"kmDefaultOpenURLValue" localized];
	Boolean							mIsDefaultApplication = true;
	XApplicationIdentifier			mApplication;
	
#if KMF_IS_ENGINE	
	EventTime						mActionTime;
#endif
	
public:
    AOpenURL() = default;
 	AOpenURL( const AOpenURL& inAction ) = default;
	virtual ~AOpenURL() = default;

	virtual AOpenURL*				Clone() const { return new AOpenURL( *this ); }
	virtual AOpenURL*				Create() const { return new AOpenURL(); }
	
	NSString*						GetURL() const { return mURL; }
	void							SetURL( NSString* inURL ) { check(inURL); mURL = [inURL copy]; }

	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual	XIcon					GetActionDescriptionXIcon() const;

    virtual HActionType				GetActionType() const { return HActionType::kOpenURL; }
	
	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return 20*kEventDurationSecond; }
	
    virtual Boolean					IsDefaultApplication() const { return mIsDefaultApplication; }
	virtual void					SetIsDefaultApplication( const Boolean& inIsDefaultApplication ) { mIsDefaultApplication = inIsDefaultApplication; }
	
	virtual const XApplicationIdentifier&	GetApplication() const { return mApplication; }
	virtual void							SetApplication( const XApplicationIdentifier& inApp ) { mApplication = inApp; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

