/*
 *  AOpenURL.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sat Dec 15 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "AOpenURL.h"
#import "XCoreFoundation.h"
#import "KMIconRegistry.h"
#import "XErrorHandler.h"
#import "KMPreferences.h"
#if KMF_IS_ENGINE
#import "TokenText.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AOpenURLEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
AOpenURL::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> theFormatStr = CFSTR("kmOpenURLTitleFormat")->CopyLocalized();		
	CFStringRef theString = CFString::CreateWithFormat( theFormatStr, mURL );
	
	return theString;
}

XIcon
AOpenURL::GetActionDescriptionXIcon() const
{
	return GetActionXIcon();
}


#if KMF_IS_ENGINE

static const float kOpenURLDelayTime = 0.25;
static float gOpenURLDelay;

float
AOpenURL::DoAction()
{
	float time = 0;
	
	mActionTime = GetCurrentEventTime();
	Get( KMPreferences::kOpenURLDelay, &gOpenURLDelay, kOpenURLDelayTime );

	NSString* urls = TokenText::GetResult( mURL, &time );
	NSURL* url = [NSURL URLWithString:urls];
	if ( url ) {
		NSString* bundleid = mIsDefaultApplication ? nil : mApplication.GetBundleIdentifier();
		if ( bundleid ) {
			[[NSWorkspace sharedWorkspace] openURLs:@[url] withAppBundleIdentifier:bundleid options:NSWorkspaceLaunchDefault additionalEventParamDescriptor:nil launchIdentifiers:NULL];
		} else {
			[[NSWorkspace sharedWorkspace] openURL:url];
		}
	}
	
	return 10 + time + 0.25 * mURL.length;
}

Boolean
AOpenURL::IsActionCompleted()
{
	return GetCurrentEventTime() > mActionTime + gOpenURLDelay;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AOpenURL::ActionController() const
{
	return [AOpenURLEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
<span class="IconTitle">With</span> {{ application }}
)RAW";

NSString*
AOpenURL::GetActionExtraHTML() const
{
	if ( mIsDefaultApplication ) return @"";
	NSDictionary *variables = @{
							@"application" : mApplication.GetDisplayHTMLContent(),
						};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
AOpenURL::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("URL"), CFBridgingToCFString(mURL) );
	dictionary->SetBoolean( CFSTR("IsDefaultApplication"), mIsDefaultApplication );
	if ( !mIsDefaultApplication ) {
		CFContainer<SafeCFPropertyListRef> app = mApplication.CopyToPropertyList();
		dictionary->Set( CFSTR("Application"), app );
	}
}

void
AOpenURL::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef newURL;
	dictionary->Get( CFSTR("URL"), newURL, CFStringRef(NULL) );
	if ( newURL ) {
		mURL = newURL->GetNS();
	} else {
		mURL = [@"kmDefaultOpenURLValue" localized];
	}

	dictionary->GetBoolean( CFSTR("IsDefaultApplication"), mIsDefaultApplication, true );
	mApplication.Reset();
	if ( !mIsDefaultApplication ) {
		SafeCFPropertyListRef app = (SafeCFPropertyListRef)dictionary->Get( CFSTR("Application") );
		if ( app ) {
			mApplication.LoadFromPropertyList( app );
		}
	}
	if ( !mApplication.HasValue() ) {
		mIsDefaultApplication = true;
	}
}
