/*
 *  APause.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun May 12 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "APause.h"
#import "Calculate.h"

#if KMF_IS_ENGINE
#import "XErrorHandler.h"
#endif

#if KMF_EDITOR_WINDOW
#import "APauseEditor.h"
#endif

const CFStringRef kDefaultPauseTime = CFSTR("1.5");

APause::APause()
	:	HBaseAction(),
		mPauseTime( kDefaultPauseTime )
{
}

APause::APause( const APause& inAction )
	:	HBaseAction( inAction ),
		mPauseTime( inAction.mPauseTime )
{
}

APause::~APause()
{
}

CFStringRef
APause::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> titleFormat = CFSTR("hkPauseTitleFormat")->CopyLocalized();
	NSString* time = [NSString stringWithFormat:@"%@", mPauseTime->GetNS()];
	return CFString::CreateWithFormat( titleFormat, time );
}

#if KMF_IS_ENGINE

float
APause::DoAction()
{
	double pausetime;
	if ( !Calculate::CalculateValue( mPauseTime->GetNS(), pausetime ) ) {
		XErrorHandlerMaybeLogFormat1("always", CFSTR("Pause calculation %@ failed"), mPauseTime->GetNS() );
		pausetime = 0;
	}
	mActionFinishTime = GetCurrentEventTime() + pausetime;
	return 0;
}

Boolean
APause::IsActionCompleted()
{
	return GetCurrentEventTime() >= mActionFinishTime;
}

#endif

#if KMF_EDITOR_WINDOW

Class
APause::ActionController() const
{
	return [APauseEditor class];
}

#endif

#pragma mark streaming

void
APause::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Time"), CFStringRef(mPauseTime) );
}

void
APause::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Time"), value, CFStringRef(NULL) );
	mPauseTime = value ? value->Copy() : NULL;

	double pausetime;
	if ( !mPauseTime && dictionary->Get( CFSTR("Seconds"), pausetime ) ) {
		mPauseTime = Calculate::CreateDoubleValue( pausetime );
	}
	
	if ( !mPauseTime ) {
		mPauseTime = kDefaultPauseTime;
	}
}

