//
//  APauseUntil.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 4 Feb 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "APauseUntil.h"
#if KMF_EDITOR_WINDOW
#import "APauseUntilEditor.h"
#import "TemplateEngine.h"
#else
#import "KMPreferences.h"
#endif

APauseUntil::APauseUntil()
	:	HBaseAction()
{
}

APauseUntil::APauseUntil( const APauseUntil& inAction )
	:	HBaseAction( inAction ),
		mConditions( inAction.mConditions )
{
}

APauseUntil::~APauseUntil()
{
}

CFStringRef
APauseUntil::CopyActionDescriptionString() const
{
	return CFSTR("hkPauseUntilTitle")->CopyLocalized();
}

#if KMF_IS_ENGINE

const EventTimerInterval kPauseCheckTime = 0.02;

float
APauseUntil::DoAction()
{
	mActionNextCheckTime = GetCurrentEventTime();
	return 0;
}

Boolean
APauseUntil::IsActionCompleted()
{
	if ( mConditions.GetCount() == 0 ) return true;

	if ( GetCurrentEventTime() >= mActionNextCheckTime ) {
		Boolean result = mConditions.ConditionResult() == ConditionTestResult::kTrue;
		mActionNextCheckTime = GetCurrentEventTime() + kPauseCheckTime;
		return result;
	} else {
		return false;
	}
}

void
APauseUntil::ActionAborted()
{
	mConditions.ConditionTestAbort();
}

#endif

#if KMF_EDITOR_WINDOW

Class
APauseUntil::ActionController() const
{
	return [APauseUntilEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
<ul class="IconList">
	{% for condition in conditions %}
		<li><img src={{ condition.icon | inlineimage: 16 }} alt="Condition icon" width="16" height="16" /><span class="IconTitle">{{ condition.description | htmlencode }}</span>{% if condition.extraicon %}<img src={{ condition.extraicon | inlineimage: 16 }} alt="Swatch" width="16" height="16" />{% /if %}</li>
	{% /for %}
</ul>
)RAW";

NSString*
APauseUntil::GetActionHTMLDescriptionString() const
{
	if ( mConditions.GetCount() == 0 ) return @"Pause Until With No Conditions Will Do Nothing";
	return [NSString stringWithFormat:@"Pause Until %@",mConditions.GetConditionDisplayText()];
}

NSString*
APauseUntil::GetActionExtraHTML() const
{
	if ( mConditions.GetCount() == 0 ) return @"";

	NSDictionary *variables = @{
									@"conditions" : mConditions.GetConditionDisplayContent(),
								};
	NSString *result = [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
	return result;
}

#endif

BOOL
APauseUntil::MatchesTextSearch( NSString* searchString ) const
{
	if ( HBaseAction::MatchesTextSearch( searchString ) ) return YES;
	if ( mConditions.MatchesTextSearch( searchString ) ) return YES;
	return NO;
}

#pragma mark streaming

void
APauseUntil::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	SafeCFPropertyListRef conditions = (SafeCFPropertyListRef)mConditions.CopyToPropertyList();
	dictionary->Set( CFSTR("Conditions"), conditions );
	conditions->Release();
}

void
APauseUntil::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	SafeCFPropertyListRef conditions;
	XAssert( dictionary->Get( CFSTR("Conditions"), conditions ) );
	mConditions.LoadFromPropertyList( conditions );
}

