//
//  APlaySound.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 19 May 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"

extern CFStringRef kPlaySoundDefaultDeviceUID;
extern CFStringRef kPlaySoundSoundEffectsDeviceUID;
extern CFStringRef kPlaySoundBeepPath;

class APlaySound
:	public HBaseAction
{
private:
	
	CFContainer<CFStringRef>		mPath;
	CFContainer<CFStringRef>		mOutputDeviceUID;
	UInt32							mVolume;
	
#if KMF_IS_ENGINE
	NSSound*						mSound;
#endif
	
public:
	
    APlaySound();
	APlaySound( const APlaySound& inAction );
    virtual ~APlaySound();
	
	virtual APlaySound*				Clone() const { return new APlaySound( *this ); }
	virtual APlaySound*				Create() const { return new APlaySound(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;
 	
	virtual HActionType				GetActionType() const { return HActionType::kPlaySound; }
	
	bool							IsBeep() const;
	
	void							SetPath( CFStringRef inString ) { mPath = inString->Copy(); }
	CFStringRef						CopyPath() const { return mPath->Copy(); }
	NSString*						GetPath() const { return mPath->GetNS(); }	
	
	void							SetOutputDeviceUID( CFStringRef inString ) { mOutputDeviceUID = inString->Copy(); }
	CFStringRef						CopyOutputDeviceUID() const { return mOutputDeviceUID->Copy(); }
	NSString*						GetOutputDeviceUID() const { return mOutputDeviceUID->GetNS(); }
	
	void							SetVolume( UInt32 inVolume ) { mVolume = inVolume; }
	UInt32							GetVolume() const { return mVolume; }
	
	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return kVeryLongDefaultTimeOutValue; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

    virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

