//
//  APlugIn.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 27 Mar 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "AExecuteRootScript.h"
#import "MTextResultsWindow.h"

enum class InformationParameterTypes {
	kInvalid,
	kString,
	kTokenString,
	kCalculation,
	kText,
	kTokenText,
	kCheckbox,
	kPopupMenu,
	kHidden,
};
#define kInformationParameterTypesNamesArray {CFSTR("INVALID"), CFSTR("String"), CFSTR("TokenString"), CFSTR("Calculation"), CFSTR("Text"), CFSTR("TokenText"), CFSTR("Checkbox"), CFSTR("PopupMenu"), CFSTR("Hidden"), NULL}

extern NSString* kReloadThirdPartyActionsNotification;

class APlugIn
	:	public AExecuteRootScript
{

private:

	NSString*						mFolderName = @"";
	NSDictionary*					mParameters = [NSDictionary new];

	MTextResultsWindowKind			mDisplayKind = MTextResultsWindowKind::kNone;
	VariableContainer				mVariable = @"Variable";
	ClipboardTarget					mClipboardTarget;
	Boolean							mTrimResults = true;

	void							UpdateToDefaults();

	NSString*						WeakGetInformationKeyedString( NSString* key ) const;
	NSString*						GetInformationKeyedString( NSString* key ) const;
	NSNumber*						GetInformationKeyedNumber( NSString* key ) const;
	NSString*						GetInformationKeyedFile( NSString* key ) const;

	NSString*						GetInformationName() const;
	NSString*						GetInformationScript() const;
	NSString*						GetInformationIcon() const;
	NSString*						GetInformationTitle() const;
//	MTextResultsWindowKind			GetInformationResults() const;
//	Boolean							GetInformationTrimResults() const;

    MTextResultsWindowKind			GetInformationDefaultDisplayKind() const;

	NSArray*						GetInformationParameters() const;
	NSDictionary*					GetInformationParameterEntry( NSUInteger index ) const;

	virtual NSString*				GetScriptTypeName() const { return GetInformationName(); }

#if KMF_IS_ENGINE
	virtual NSString*				GetLongScriptDisplayName() const;

	virtual NSString*				GetShellCommand();

	Boolean							mStripTrailingLineFeed;
	virtual Boolean					StripTrailingLineFeed() { return mStripTrailingLineFeed; }

	virtual Boolean					TrimResults() { return mTrimResults; }
	virtual MTextResultsWindowKind	DisplayKind() { return mDisplayKind; }
	virtual NSString*				SaveToVariable() { return mVariable->GetNS(); }
	virtual void					WriteClipboardDataSet( const XClipboardDataSet& dataset ) { mClipboardTarget.WriteClipboardDataSet( dataset ); }

	virtual NSDictionary*			GetExecutionParameters();
#endif

public:

    APlugIn() = default;
	APlugIn( NSString* folderName );
	APlugIn( const APlugIn& inAction ) = default;
    virtual ~APlugIn() = default;

	virtual APlugIn*				Clone() const { return new APlugIn( *this ); }
	virtual APlugIn*				Create() const { return new APlugIn(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kPlugIn; }
    virtual XIcon					GetActionXIcon() const;

	virtual NSString*				GetNewActionTitle() const;

	virtual	CFStringRef				CopyActionDescriptionString() const;

	virtual NSString*				GetToolTip() const;

	virtual Boolean					HasTimeOut() const;
	virtual EventTime				DefaultTimeOutPeriod() const;

#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraResults() const;
	virtual	NSString*				GetActionExtraHTML() const;

	virtual NSString*				GetActionKey() const { return mFolderName; }

	static NSArray*					ListPlugIns();

	static Boolean					UnzipLikelyContains( NSString* output );
	static Boolean					ImportUnzipped( NSString* parentpath );
#endif

	NSUInteger						GetInformationParameterCount() const;
	NSString*						GetInformationParameterEntryLabel( NSUInteger index ) const;
	InformationParameterTypes		GetInformationParameterEntryType( NSUInteger index ) const;
	NSString*						GetInformationParameterEntryDefault( NSUInteger index ) const;
	NSString*						GetInformationParameterEntryMenu( NSUInteger index ) const;

	NSString*						GetParameterValue( NSString* label ) const;
	void							SetParameterValue( NSString* label, NSString* value );

	virtual NSString*				GetVariable() const { check( mVariable ); return mVariable->GetNS(); }
	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }

	NSString*						GetGenericNamedClipboardUID() const { return mClipboardTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mClipboardTarget.SetGenericNamedClipboardUID( inUID ); }

	virtual Boolean					GetTrimResults() const { return mTrimResults; }
	virtual void					SetTrimResults( Boolean inTrimResults ) { mTrimResults = inTrimResults; }
	
    virtual MTextResultsWindowKind	GetDisplayKind() const { return mDisplayKind; }
	virtual void					SetDisplayKind( MTextResultsWindowKind inDisplayKind ) { mDisplayKind = inDisplayKind; }

#if KMF_EDITOR_WINDOW
    virtual Boolean					IsAllowedDisplayKind( MTextResultsWindowKind kind ) const;
	virtual int						CountAllowedDisplayKind() const;
	virtual Boolean					ShouldDisplayKind() const;
	virtual Boolean					NeedDisplayKind() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	static void						ClearCache();
	
};

