/*
 *  APressButton.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Thu Apr 24 2003.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"

class XAXUIElement;

const Boolean kDefaultPressButtonStopOnFailure = true;

class APressButton
	:	public HBaseAction
{
private:

	CFContainer<CFStringRef>		mButtonName;
	Boolean							mStopOnFailure;
	
public:
    APressButton();
	APressButton( const APressButton& inAction );
    virtual ~APressButton();

	virtual APressButton*			Clone() const { return new APressButton( *this ); }
	virtual APressButton*			Create() const { return new APressButton(); }
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues();
#endif
	
    virtual HActionType				GetActionType() const { return HActionType::kPressButton; }
	virtual	CFStringRef				CopyActionDescriptionString() const;

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	void							SetButtonName( CFStringRef inString ) { mButtonName = inString->Copy(); }
	CFStringRef						CopyButtonName() const { return mButtonName->Copy(); }
	NSString*						GetButtonName() const { return mButtonName->GetNS(); }
	
	Boolean							GetStopOnFailure() const { return mStopOnFailure; }
	void							SetStopOnFailure( Boolean inStopOnFailure ) { mStopOnFailure = inStopOnFailure; }
};

