//
//  AProgramSwitcher.h
//  Keyboard Maestro
//
//  Created by Michael Kamprath.
//  Copyright (c) 2004 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "XApplicationList.h"
#import "PSSwitcherApplicationList.h"

enum class ProgramSwitcherWindowType {
	kVerticalList,
	kHorizontalIcons,
	kIconGrid,
	kCoverFlow,
	kDEFAULT = kIconGrid,
};
#define kProgramSwitcherWindowTypeNamesArray {CFSTR("VerticalList"), CFSTR("HorizontalIcons"), CFSTR("IconGrid"), CFSTR("CoverFlow"), NULL}

class AProgramSwitcher : public HBaseAction
{
private:

#if KMF_IS_ENGINE
	NSUUID*							mUUID = [NSUUID UUID];
#endif

	XApplicationList				mIncludedApps;
	PSApplicationSortOrderType		mAppSortOrder = kPSApplicationSortOrderDEFAULT;
	ProgramSwitcherWindowType		mTheme = ProgramSwitcherWindowType::kDEFAULT;
	NSColor*						mColor;
	SInt16							mIconSize;
	
	Boolean							mSwitcherUnderMouse = false;
	Boolean							mHideOthers = false;
	Boolean							mMouseRollover = true;

public:
	AProgramSwitcher();
	AProgramSwitcher( const AProgramSwitcher& inAction ) = default;
	virtual ~AProgramSwitcher() = default;

	virtual AProgramSwitcher*		Clone() const { return new AProgramSwitcher( *this ); }
	virtual AProgramSwitcher*		Create() const { return new AProgramSwitcher(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kProgramSwitcher; }

#if KMF_IS_ENGINE
    NSString*						GetUID() const { return [mUUID UUIDString]; }
	virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	SInt16							GetIconSize() const { return mIconSize; }
	void							SetIconSize( SInt16 inSize );

	PSApplicationSortOrderType		GetAppSortOrder() const { return mAppSortOrder; }
	void							SetAppSortOrder( PSApplicationSortOrderType inAppSortOrder ) { mAppSortOrder = inAppSortOrder; }

	ProgramSwitcherWindowType		GetSwitcherTheme() const { return mTheme; }
	void							SetSwitcherTheme( ProgramSwitcherWindowType inTheme ) { mTheme = inTheme; }

	NSColor*						GetSwitcherColor() const { return mColor; }
	void							SetSwitcherColor( NSColor* inColor );
	
	Boolean							GetSwitcherUnderMouse() const { return mSwitcherUnderMouse; }
	void							SetSwitcherUnderMouse( Boolean inSwitcherUnderMouse ) { mSwitcherUnderMouse = inSwitcherUnderMouse; }

	Boolean							GetHideOthers() const { return mHideOthers; }
	void							SetHideOthers( Boolean inHideOthers ) { mHideOthers = inHideOthers; }
	
	Boolean							GetMouseRollover() const { return mMouseRollover; }
	void							SetMouseRollover( Boolean inMouseRollover ) { mMouseRollover = inMouseRollover; }
	
	const XApplicationList&			GetIncludedApps() const { return mIncludedApps; }
	void							SetIncludedApps( const XApplicationList& inIncludedApps ) { mIncludedApps = inIncludedApps; }
	
};

