//
//  APromptForUserInput.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 6 May 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"

#if KMF_IS_ENGINE
@class KMPromptForUserInputController;
#endif

class APromptForUserInput : public HBaseAction
{
private:
	CFContainer<CFStringRef>		mTitle;
	CFContainer<CFStringRef>		mPrompt;
	NSArray*						mVariables;
	VariableContainer				mResultButtonVariable;
	NSArray*						mButtons;

#if KMF_IS_ENGINE
	KMPromptForUserInputController*	mPromptController;
#endif

public:
	APromptForUserInput();
	APromptForUserInput( const APromptForUserInput& inAction );
	virtual ~APromptForUserInput();

	virtual APromptForUserInput*	Clone() const { return new APromptForUserInput( *this ); }
	virtual APromptForUserInput*	Create() const { return new APromptForUserInput(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kPromptForUserInput; }

	virtual CFStringRef				CopyActionDescriptionString() const;

	CFStringRef						CopyTitle() const { return mTitle->Copy(); }
	NSString*						GetTitle() const { return mTitle->GetNS(); }
	void							SetTitle( CFStringRef name ) { mTitle = name->Copy(); }
	
	CFStringRef						CopyPrompt() const { return mPrompt->Copy(); }
	NSString*						GetPrompt() const { return mPrompt->GetNS(); }
	void							SetPrompt( CFStringRef name ) { mPrompt = name->Copy(); }
	
//	CFStringRef						CopyResultButtonVariable() const { return mResultButtonVariable->Copy(); }
//	NSString*						GetResultButtonVariable() const { return mResultButtonVariable->GetNS(); }
//	void							SetResultButtonVariable( CFStringRef inVariable ) { check( inVariable ); mResultButtonVariable = inVariable->Copy(); }

	NSArray*						GetVariables() const { return mVariables; }
	void							SetVariables( NSArray* variables );
	
	NSArray*						GetButtons() const { return mButtons; }
	void							SetButtons( NSArray* buttons ) { mButtons = [buttons copy]; }
	
	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return kVeryLongDefaultTimeOutValue; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
};

