/*
 *  AQuickMacro.h
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Mon 28 Jan 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"

class AQuickMacro
	:	public HBaseAction
{
private:

	CFContainer<CFStringRef>		mQuickMacroUID;
	CGKeyCode						mKeyCode;
	UInt32							mModifiers;
	Boolean							mEnableRepeat;
	Boolean							mAddToMacroPalette;
	Boolean							mAddToStatusMenu;

public:
	
    AQuickMacro();
	AQuickMacro( const AQuickMacro& inAction );
    virtual ~AQuickMacro();

	virtual AQuickMacro*			Clone() const { return new AQuickMacro( *this ); }
	virtual AQuickMacro*			Create() const { return new AQuickMacro(); }
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateForPasteOrAdd();
#endif
	
	void							SetKeystroke( const CGKeyCode& inKeyCode, const UInt32& inModifiers, Boolean inRepeat );
	CGKeyCode						GetKeyCode() const				{ return mKeyCode; }
	UInt32							GetModifiers() const			{ return mModifiers; }
	Boolean							GetEnableRepeat() const			{ return mEnableRepeat; }

	void							SetAddToMacroPalette( Boolean inAddToMacroPalette ) { mAddToMacroPalette = inAddToMacroPalette; }
	Boolean							GetAddToMacroPalette() const	{ return mAddToMacroPalette; }

	void							SetAddToStatusMenu( Boolean inAddToStatusMenu ) { mAddToStatusMenu = inAddToStatusMenu; }
	Boolean							GetAddToStatusMenu() const	{ return mAddToStatusMenu; }

	CFStringRef						CopyQuickMacroUID() const { return mQuickMacroUID->Copy(); }
	NSString*						GetQuickMacroUID() const { return mQuickMacroUID->GetNS(); }

    virtual HActionType				GetActionType() const { return HActionType::kQuickMacro; }
	virtual	CFStringRef				CopyActionDescriptionString() const;

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
	
	static void						ResetQuickMacroUIDSet(); // ugly hack!

};

