//
//  AQuickTimePlayerControl.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 18 Sep 2009.
//  Copyright (c) 2009 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"

enum class QuickTimePlayerControlCommand {
	kPlay,
	kPlayPause,
	kPause,
	kStepForward,
	kStepBackward,
	kIncreaseVolume,
	kDecreaseVolume,
	kSetVolume,
	kDEFAULT = kPlayPause, 
} ;
#define kQuickTimePlayerControlCommandNamesArray {CFSTR("Play"), CFSTR("PlayPause"), CFSTR("Pause"), \
	CFSTR("StepForward"), CFSTR("StepBackward"), \
	CFSTR("IncreaseVolume"), CFSTR("DecreaseVolume"), CFSTR("SetVolume"), \
	NULL}

class AQuickTimePlayerControl
	:	public HBaseAction
{
private:

	QuickTimePlayerControlCommand	mCommand;
	CFContainer<CFStringRef>		mArgumentString;

protected:

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif

public:
    AQuickTimePlayerControl();
 	AQuickTimePlayerControl( const AQuickTimePlayerControl& inAction );
 	AQuickTimePlayerControl( const QuickTimePlayerControlCommand& inCommand );
	virtual ~AQuickTimePlayerControl();

	virtual AQuickTimePlayerControl*			Clone() const { return new AQuickTimePlayerControl( *this ); }
	virtual AQuickTimePlayerControl*			Create() const { return new AQuickTimePlayerControl(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kQuickTimePlayerControl; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
    virtual XIcon					GetActionXIcon() const;

	virtual NSString*				GetNewActionTitle() const;

	void							SetCommand( QuickTimePlayerControlCommand inCommand );
	QuickTimePlayerControlCommand	GetCommand() const;

	void							SetArgumentString( CFStringRef inString );
	void							SetArgumentInteger( NSInteger inValue );
	CFStringRef						CopyArgumentString() const;
	NSString*						GetArgumentString() const { return CFBridgingToNS( mArgumentString ); }
	NSInteger						GetArgumentInteger() const { return mArgumentString->GetIntValue(); }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

