/*
 *  AQuitApplications.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Dec 16 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "AQuitApplications.h"
#import "PSSwitcherApplicationList.h"
#import "KMDefines.h"
#import "MMacroDomain.h"
#import "KMExcludedApplications.h"
#if KMF_EDITOR_WINDOW
#import "AQuitApplicationsEditor.h"
#endif

static const Boolean kDefaultHonorExcluded = true;

AQuitApplications::AQuitApplications( const EQuitApplicationsActionType& inType )
	:	HBaseAction(),
		mQuitActionType( inType ),
		mHonorExcluded( kDefaultHonorExcluded )
{
}
	
HActionType
AQuitApplications::GetActionType() const
{
	HActionType	theType;

    switch ( mQuitActionType ) {
		case EQuitApplicationsActionType::kAllApplications:
			theType = HActionType::kQuitAll;
			break;
		case EQuitApplicationsActionType::kOtherApplications:
			theType = HActionType::kQuitOthers;
			break;
    }
    
    return theType;
}

#if KMF_IS_ENGINE
float
AQuitApplications::DoAction()
{
	float time = 0;
	PSSwitcherApplicationList *theAppList = NULL;
    theAppList = new PSSwitcherApplicationList( kPSApplicationSortOrderDEFAULT, NULL, GetHonorExcluded() ? &KMExcludedApplications::Get() : NULL );
    if ( theAppList ) {
		CFIndex	i;
		CFIndex	theCurrentApp;
		
		if ( mQuitActionType == EQuitApplicationsActionType::kAllApplications ) {
			theCurrentApp = 0x7FFFFFFF;
		} else {
			theCurrentApp = theAppList->GetCurrentAppIndex();
		}
			
		for (i = 0; i < theAppList->GetCount(); i++) {
			if ( (theCurrentApp != i) && theAppList->IsAppRunning(i) ) {
				theAppList->SetQuitApp( i, true );
				time += 3;
			}
		}
		
		theAppList->SendMessagesToApps();
		
		delete theAppList;
    }
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AQuitApplications::ActionController() const
{
	return [AQuitApplicationsEditor class];
}

NSString*
AQuitApplications::GetActionHTMLDescriptionString() const
{
	if ( mHonorExcluded ) return GetActionDescriptionString();
	return [NSString stringWithFormat:@"%@ Including Excluded Applications", GetActionDescriptionString()];
}

#endif

#pragma mark streaming

void
AQuitApplications::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->SetBoolean( CFSTR("HonorExcluded"), mHonorExcluded );
}

void
AQuitApplications::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	dictionary->GetBoolean( CFSTR("HonorExcluded"), mHonorExcluded, kDefaultHonorExcluded );
}
