/*
 *  AQuitSpecificApplication.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Fri May 31 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "ApplicationTarget.h"

// WARNING: enum valued used in menu selection in action editor
enum class QuitSpecificActionType {
	kQuit,
	kQuitRelaunch,
	kForceQuit,
	kForceQuitRelaunch,
	kDEFAULT = kQuit, 
};
#define kQuitSpecificActionTypeNamesArray {CFSTR("Quit"), CFSTR("QuitRelaunch"), CFSTR("ForceQuit"), CFSTR("ForceQuitRelaunch"), NULL}

class AQuitSpecificApplication
	:	public HBaseAction
{
private:

	QuitSpecificActionType			mAction = QuitSpecificActionType::kDEFAULT;
	ApplicationTarget				mTargetApplication;
	
public:
	
    AQuitSpecificApplication() = default;
	AQuitSpecificApplication( const AQuitSpecificApplication& inAction ) = default;
    virtual ~AQuitSpecificApplication() = default;
	
	virtual AQuitSpecificApplication*		Clone() const { return new AQuitSpecificApplication( *this ); }
	virtual AQuitSpecificApplication*		Create() const { return new AQuitSpecificApplication(); }
	
	virtual	CFStringRef				CopyActionTitle() const;
	virtual	CFStringRef				CopyActionDescriptionString() const;
 	virtual	XIcon					GetActionDescriptionXIcon() const;
 	virtual	NSString*				GetNewActionTitle() const;

	virtual HActionType				GetActionType() const { return HActionType::kQuitSpecificApp; }

 	QuitSpecificActionType			GetQuitSpecificAction() const { return mAction; }
	void							SetQuitSpecificAction( QuitSpecificActionType inAction ) { mAction = inAction; }
	
	ApplicationTargetApplicationType		GetApplicationTargetType() const { return mTargetApplication.GetApplicationTargetType(); }
    virtual const XApplicationIdentifier&	GetTargetApplication() const { return mTargetApplication.GetTargetApplication(); }

	void							SetApplicationTargetApplication( const XApplicationIdentifier& inApp ) { mTargetApplication.SetApplicationTargetApplication( inApp ); }
	void							SetApplicationTargetFrontApplication() { mTargetApplication.SetApplicationTargetFrontApplication(); }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif

	virtual BOOL					CanBeInstigatedByClickAction() const { return YES; }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

