/*
 *  AQuitSpecificApplication.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Fri May 31 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "AQuitSpecificApplication.h"
#import "KMDefines.h"
#import "XProcessSerialNumberList.h"
#import "MMacroDomain.h"
#import "XException.h"
#import "NSStringExtensions.h"
#import "KMIconRegistry.h"
#if KMF_EDITOR_WINDOW
#import "AQuitSpecificApplicationEditor.h"
#endif

CFStringRef
AQuitSpecificApplication::CopyActionTitle() const
{
	switch ( GetApplicationTargetType() ) {
		case ApplicationTargetApplicationType::kFront: {
			return CFSTR("hkQuitCurrentApp")->CopyLocalized();
		}
		case ApplicationTargetApplicationType::kSpecific: {
			NSString* theNameStr = mTargetApplication.GetDisplayName();
			CFContainer<CFStringRef> theFormatStr = CFSTR("kmQuitSpecificAppTitleFormat")->CopyLocalized();
			return CFString::CreateWithFormat( theFormatStr, theNameStr );
		}
	}
}

NSString*
AQuitSpecificApplication::GetNewActionTitle() const
{
	return [@"hkQuitSpecificApp" localized];
}

CFStringRef
AQuitSpecificApplication::CopyActionDescriptionString() const
{
	CFStringRef key;
	switch ( mAction ) {
		case QuitSpecificActionType::kQuit:
			key = CFSTR("kQuitSpecificActionType_QuitFormat");
			break;
		case QuitSpecificActionType::kQuitRelaunch:
			key = CFSTR("kQuitSpecificActionType_QuitRelaunchFormat");
			break;
		case QuitSpecificActionType::kForceQuit:
			key = CFSTR("kQuitSpecificActionType_ForceQuitFormat");
			break;
		case QuitSpecificActionType::kForceQuitRelaunch:
			key = CFSTR("kQuitSpecificActionType_ForceQuitRelaunchFormat");
			break;
	}
	CFContainer<CFStringRef> theFormatStr = key->CopyLocalized();
	NSString* theNameStr;
	if ( GetApplicationTargetType() == ApplicationTargetApplicationType::kSpecific ) {
		theNameStr = mTargetApplication.GetDisplayName();
	} else {
		theNameStr = [@"kQuitSpecificTargetFrontApplication" localized];
	}
	return CFString::CreateWithFormat( theFormatStr, theNameStr );
}

XIcon
AQuitSpecificApplication::GetActionDescriptionXIcon() const
{
	switch ( mAction ) {
		case QuitSpecificActionType::kQuit:
			return XIcon( mTargetApplication.GetXIcon(), XIcon(KMIconRegistry::kQuitBadgeIcon) );
		case QuitSpecificActionType::kQuitRelaunch:
			return XIcon( mTargetApplication.GetXIcon(), XIcon(KMIconRegistry::kQuitBadgeIcon), XIcon(KMIconRegistry::kLaunchAppIcon) );
		case QuitSpecificActionType::kForceQuit:
			return XIcon( mTargetApplication.GetXIcon(), XIcon(KMIconRegistry::kForceQuitBadgeIcon) );
		case QuitSpecificActionType::kForceQuitRelaunch:
			return XIcon( mTargetApplication.GetXIcon(), XIcon(KMIconRegistry::kForceQuitBadgeIcon), XIcon(KMIconRegistry::kLaunchAppIcon) );
	}
}

#if KMF_IS_ENGINE

float
AQuitSpecificApplication::DoAction()
{
	XApplicationIdentifier app = mTargetApplication.GetCurrentlyTargetedApplication();
	switch ( mAction ) {
		case QuitSpecificActionType::kQuit:
			app.Quit( false );
			break;
		case QuitSpecificActionType::kQuitRelaunch:
			app.Quit( true );
			break;
		case QuitSpecificActionType::kForceQuit:
			app.ForceQuit( false );
			break;
		case QuitSpecificActionType::kForceQuitRelaunch:
			app.ForceQuit( true );
			break;
	}
	return 6;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AQuitSpecificApplication::ActionController() const
{
	return [AQuitSpecificApplicationEditor class];
}

#endif

#pragma mark streaming

static const CFStringRef kQuitSpecificActionTypeNames[] = kQuitSpecificActionTypeNamesArray;

void
AQuitSpecificApplication::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Action"), kQuitSpecificActionTypeNames[int(mAction)] );
	mTargetApplication.SaveToDictionary( dictionary, CFSTR("Target"), CFSTR("Application") );
}

void
AQuitSpecificApplication::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mTargetApplication.InitializeFromDictionary( dictionary, CFSTR("Target"), CFSTR("Application") );
	mAction = dictionary->GetEnumClass( CFSTR("Action"), kQuitSpecificActionTypeNames, QuitSpecificActionType::kQuit );
}
