//
//  AReadFile.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 5 Apr 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"
#import "ClipboardReference.h"

enum class ReadFileDestinationType { // WARNING: tag valued in AReadFile.xib match this enum
	kClipboard,
	kNamedClipboard,
	kVariable,
	kDEFAULT = kClipboard, 
};
#define kReadFileDestinationTypeNamesArray {CFSTR("Clipboard"), CFSTR("NamedClipboard"), CFSTR("Variable"), NULL}

class AReadFile
	:	public HBaseAction
{

private:

	ReadFileDestinationType			mDestination = ReadFileDestinationType::kDEFAULT;
	ClipboardReference				mClipboardReference;
	VariableContainer				mVariable = CFSTR("Variable");
	CFContainer<CFStringRef>		mSource = CFSTR("");
	
public:

    AReadFile() = default;
	AReadFile( const AReadFile& inAction ) = default;
    virtual ~AReadFile() = default;

	virtual AReadFile*				Clone() const { return new AReadFile( *this ); }
	virtual AReadFile*				Create() const { return new AReadFile(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kReadFile; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mClipboardReference.UpdateClipboardReference(); }
#endif

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	ReadFileDestinationType			GetDestination() const { return mDestination; }
	void							SetDestination( ReadFileDestinationType inDestination ) { mDestination = inDestination; }
	
	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	virtual NSString*				GetVariable() const { return mVariable->GetNS(); }
	
	NSString*						GetClipboardUID() const { return mClipboardReference.GetClipboardUID(); }
	void							SetClipboardUID( NSString* inUID ) { mClipboardReference.SetClipboardUID( inUID ); }

	CFStringRef						CopySource() const { return mSource->Copy(); }
	NSString*						GetSource() const { return mSource->GetNS(); }
	void							SetSource( CFStringRef source ) { mSource = source->Copy(); }
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
	
};

