//
//  ARepeat.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Mon 28 Mar 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "HActionList.h"

class ARepeat
	:	public HBaseAction
{

private:

#if KMF_IS_ENGINE
	NSUUID*							mTrackingID;
	SInt32							mCountRemaining;
#endif
	
	HActionList						mActionList;
    CFContainer<CFStringRef>		mCount;

public:

    ARepeat();
	ARepeat( const ARepeat& inAction );
    virtual ~ARepeat();

	virtual ARepeat*				Clone() const { return new ARepeat( *this ); }
	virtual ARepeat*				Create() const { return new ARepeat(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kRepeat; }
	virtual Boolean					IsLoop() const { return true; }

	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return kVeryLongDefaultTimeOutValue; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	const HActionList&				GetActionList() const { return mActionList; }
	void							SetActionList( const HActionList& inActions ) { mActionList = inActions; }
	void							SetActionAtIndex( const CFIndex& inIndex, HBaseAction* inAction ) { mActionList.SetActionAtIndex( inIndex, inAction ); }
	
	virtual void					DoForEverySubAction( ActionBlock inApplier ) { mActionList.DoForEveryActionAndSubAction( inApplier ); }

	void							SetCount( CFStringRef inString ) { mCount = inString->Copy(); }
	CFStringRef						CopyCount() const { return mCount->Copy(); }
	NSString*						GetCount() const { return mCount->GetNS(); }
		
	virtual BOOL					MatchesTextSearch( NSString* search ) const;
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

