//
//  AResizeImage.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 9 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ClipboardTarget.h"
#import "ArrowStrings.h"

enum class ResizeImageActionType { // WARNING: tag valued in AResizeImage.xib match this enum
	kScaleTo,
	kScaleToFit,
	kResizeTo,
	kAddMargin,
	kCrop,
	kDEFAULT = kResizeTo,
};
#define kResizeImageActionTypeNamesArray {CFSTR("ScaleTo"), CFSTR("ScaleToFit"), CFSTR("ResizeTo"), CFSTR("AddMargin"), CFSTR("Crop"), NULL}

constexpr NSString* kAResizeImageLeft = @"100";
constexpr NSString* kAResizeImageTop = @"100";
constexpr NSString* kAResizeImageWidth = @"100";
constexpr NSString* kAResizeImageHeight = @"100";

class AResizeImage
	:	public HBaseAction
{
private:

	ResizeImageActionType			mResizeAction = ResizeImageActionType::kDEFAULT;
	ClipboardTarget					mTarget;
    NSString*						mLeft = kAResizeImageLeft;
    NSString*						mTop = kAResizeImageTop;
    NSString*						mWidth = kAResizeImageWidth; // or Right
    NSString*						mHeight = kAResizeImageHeight; // or Bottom

protected:

public:

    AResizeImage() = default;
    AResizeImage( ResizeImageActionType inAction ) : mResizeAction(inAction) { ; }
	AResizeImage( const AResizeImage& inAction ) = default;
    virtual ~AResizeImage() = default;

	virtual AResizeImage*			Clone() const { return new AResizeImage( *this ); }
	virtual AResizeImage*			Create() const { return new AResizeImage(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kResizeImage; }
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); }
#endif
	
	NSString*						GetNewActionTitle() const;
	virtual	CFStringRef				CopyActionDescriptionString() const;
	
	ResizeImageActionType			GetResizeAction() const { return mResizeAction; }
	void							SetResizeAction( ResizeImageActionType inResizeAction ) { mResizeAction = inResizeAction; }
	
	const ArrowString::Style*		GetArrowStyles() const; // changes whenever ResizeAction changes

	NSString*						GetGenericNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }
	
	void							SetLeft( NSString* inString ) { mLeft = [inString copy]; }
	void							SetLeft( double inValue ) { mLeft = [NSString stringWithFormat:@"%f", inValue]; }
	NSString*						GetLeft() const { return mLeft; }
	
	void							SetTop( NSString* inString ) { mTop = [inString copy]; }
	void							SetTop( double inValue ) { mTop = [NSString stringWithFormat:@"%f", inValue]; }
	NSString*						GetTop() const { return mTop; }
	
	void							SetWidth( NSString* inString ) { mWidth = [inString copy]; }
	void							SetWidth( double inValue ) { mWidth = [NSString stringWithFormat:@"%f", inValue]; }
	NSString*						GetWidth() const { return mWidth; }

	void							SetHeight( NSString* inString ) { mHeight = [inString copy]; }
	void							SetHeight( double inValue ) { mHeight = [NSString stringWithFormat:@"%f", inValue]; }
	NSString*						GetHeight() const { return mHeight; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

