//
//  ASafariControl.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 18 Sep 2009.
//  Copyright (c) 2009 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "NSStringExtensions.h"
#import "WebBrowserInterface.h"

enum class SafariCommand {
	kNewWindow,
	kNewTab,
	kNextTab,
	kPreviousTab,
	kSelectTab,
	kSetTitle,
	kWaitForComplete,
	kSetDocumentURL,
	kClickLink,
	kSubmitForm,
	kResetForm,
	kFocusField,
	kSelectField,
	kDEFAULT = kNewTab,
} ;
#define kSafariCommandNamesArray {CFSTR("NewWindow"), CFSTR("NewTab"), CFSTR("NextTab"), CFSTR("PreviousTab"), CFSTR("SelectTab"), CFSTR("SetTitle"), CFSTR("WaitForComplete"), CFSTR("SetDocumentURL"), CFSTR("ClickLink"), CFSTR("SubmitForm"), CFSTR("ResetForm"), CFSTR("FocusField"), CFSTR("SelectField"), NULL}

class ASafariControl
	:	public HBaseAction
{
private:

	SafariCommand					mCommand = SafariCommand::kDEFAULT;
	NSString*						mArgumentString = @"";

#if KMF_IS_ENGINE
	EventTime						mActionLastIncompleteTime;
	EventTime						mActionLastIncompleteCheckTime;
	EventTime						mActionRequiredTime;
#endif

	WebBrowserInterface*			mWebBrowserInterface = WebBrowserInterface::Safari();

protected:

#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif

public:
    ASafariControl() = default;
 	ASafariControl( const ASafariControl& inAction ) = default;
 	ASafariControl( WebBrowserInterface* interface, const SafariCommand& inCommand );
	virtual ~ASafariControl() = default;

	virtual ASafariControl*			Clone() const { return new ASafariControl( *this ); }
	virtual ASafariControl*			Create() const { return new ASafariControl(); }

	virtual WebBrowserInterface*	GetWebBrowserInterface() const { return mWebBrowserInterface; }
	virtual NSString*				GetWebBrowserName() const { return mWebBrowserInterface->GetName(); }

    virtual HActionType				GetActionType() const { return HActionType::kSafariControl; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual NSString*				GetNewActionTitle() const;
 	virtual	XIcon					GetActionDescriptionXIcon() const { return mWebBrowserInterface->GetXIcon(); }

	void							SetCommand( SafariCommand inCommand ) { mCommand = inCommand; }
	SafariCommand					GetCommand() const { return mCommand; }

	void							SetArgumentString( NSString* inString ) { mArgumentString = [inString copy]; }
	void							SetArgumentInteger( NSInteger inValue ) { mArgumentString = [NSString stringWithLongInt:inValue]; }
	NSString*						GetArgumentString() const { return mArgumentString; }
	NSInteger						GetArgumentInteger() const { return [mArgumentString integerValue]; }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

