//
//  AScreenCapture.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 19 Apr 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ClipboardTarget.h"

typedef enum { // WARNING: tag valued in AScreenCapture.xib match this enum
	kScreenCaptureSourceScreenAll,
	kScreenCaptureSourceScreenMain,
	kScreenCaptureSourceScreenIndex,
	kScreenCaptureSourceWindowFront,
	kScreenCaptureSourceWindowName,
	kScreenCaptureSourceWindowNameContaining,
	kScreenCaptureSourceWindowNameMatching,
	kScreenCaptureSourceWindowIndex,
	kScreenCaptureSourceDEFAULT = kScreenCaptureSourceScreenAll,
} ScreenCaptureSourceType;
#define kScreenCaptureSourceTypeNamesArray {CFSTR("ScreenAll"), CFSTR("ScreenMain"), CFSTR("ScreenIndex"), CFSTR("WindowFront"), CFSTR("WindowName"), CFSTR("WindowNameContaining"), CFSTR("WindowNameMatching"), CFSTR("WindowIndex"), NULL}

class XAXWindowElement;

class AScreenCapture
	:	public HBaseAction
{
private:

	ScreenCaptureSourceType			mSource;
	Boolean							mIncludeShadows;
    CFContainer<CFStringRef>		mIndex;
	CFContainer<CFStringRef>		mWindowName;
	ClipboardTarget					mDestination;

#if KMF_IS_ENGINE
	CGImageRef						CaptureWindow( const XAXWindowElement& inWindowElement );
#endif

protected:

public:

    AScreenCapture();
	AScreenCapture( const AScreenCapture& inAction );
	
    virtual ~AScreenCapture();

	virtual AScreenCapture*			Clone() const { return new AScreenCapture( *this ); }
	virtual AScreenCapture*			Create() const { return new AScreenCapture(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kScreenCapture; }
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mDestination.UpdateToDefaultValues(); }
#endif
	
	ScreenCaptureSourceType			GetSource() const { return mSource; }
	void							SetSource( ScreenCaptureSourceType inSource ) { mSource = inSource; }
	
	Boolean							SourceIsScreen() const;

	Boolean							GetIncludeShadows() const { return mIncludeShadows; }
	void							SetIncludeShadows( Boolean inIncludeShadows ) { mIncludeShadows = inIncludeShadows; }

	Boolean							MustIncludeShadows() const { return SourceIsScreen(); }

	CFStringRef						CopyWindowName() const { return mWindowName->Copy(); }
	NSString*						GetWindowName() const { return mWindowName->GetNS(); }
	void							SetWindowName( CFStringRef name ) { mWindowName = name->Copy(); }
	
	void							SetIndex( CFStringRef inString ) { mIndex = inString->Copy(); }
	void							SetIndex( double inValue ) { mIndex = CFString::CreateWithNumber(lround(inValue)); }
	CFStringRef						CopyIndex() const { return mIndex->Copy(); }
	NSString*						GetIndex() const { return mIndex->GetNS(); }

	Boolean							GetUseNamedClipboard() const { return mDestination.GetUseNamedClipboard(); }
	void							SetUseNamedClipboard( Boolean inUseNamedClipboard ) { mDestination.SetUseNamedClipboard( inUseNamedClipboard ); }
	
	NSString*						GetNamedClipboardUID() const { return mDestination.GetClipboardUID(); }
	void							SetNamedClipboardUID( NSString* inUID ) { mDestination.SetClipboardUID( inUID ); }
	
	NSString*						GetGenericNamedClipboardUID() const { return mDestination.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mDestination.SetGenericNamedClipboardUID( inUID ); }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

