/*
 *  AScrollWheelEvent.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sat Feb 08 2003.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"

enum class ScrollWheelDirectionType {
	kUp,
	kDown,
	kLeft,
	kRight,
	kObsoleteHorizontal,
	kObsoleteVertical,
	kDEFAULT = kUp, 
};
#define kScrollWheelDirectionNamesArray {CFSTR("Up"), CFSTR("Down"), CFSTR("Left"), CFSTR("Right"), CFSTR("Horizontal"), CFSTR("Vertical"), NULL}

class AScrollWheelEvent
	:	public HBaseAction
{
private:

    CFContainer<CFStringRef>		mScrollAmount;
	ScrollWheelDirectionType		mScrollDirection;

public:

    AScrollWheelEvent();
	AScrollWheelEvent( const AScrollWheelEvent& inAction );
    virtual ~AScrollWheelEvent();
	
	virtual AScrollWheelEvent*		Clone() const { return new AScrollWheelEvent( *this ); }
	virtual AScrollWheelEvent*		Create() const { return new AScrollWheelEvent(); }
	
	virtual HBaseAction*			Merge( HBaseAction* action );

	virtual	CFStringRef				CopyActionDescriptionString() const;
    virtual HActionType				GetActionType() const { return HActionType::kScrollWheelEvent; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	ScrollWheelDirectionType		GetScrollDirection() const { return mScrollDirection; }
	void							SetScrollDirection( const ScrollWheelDirectionType& inScrollDirection ) { mScrollDirection = inScrollDirection; }

	void							SetScrollAmount( CFStringRef inString ) { mScrollAmount = inString->Copy(); }
	CFStringRef						CopyScrollAmount() const { return mScrollAmount->Copy(); }
	NSString*						GetScrollAmount() const { return mScrollAmount->GetNS(); }
	
};

