//
//  ASearchClipboard.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Mon 4 Feb 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ClipboardTarget.h"

// WARNING: enum valued used in menu selection in action editor
enum class SearchClipboardActionType {
	kCaseSensitiveRegEx,
	kIgnoreCaseRegEx,
	kDEFAULT = kIgnoreCaseRegEx, 
};
#define kSearchClipboardActionTypeNamesArray {CFSTR("CaseSensitiveRegEx"), CFSTR("IgnoreCaseRegEx"), NULL}

class ASearchClipboard
	:	public HBaseAction
{

private:

	SearchClipboardActionType		mAction = SearchClipboardActionType::kDEFAULT;
	ClipboardTarget					mTarget;
	NSString*						mSearch = @"";
	NSArray*						mCaptures = @[@""];

	NSArray*						PrivateGetCurrentResults( float* time, bool testRegex );

public:

    ASearchClipboard() = default;
	ASearchClipboard( const ASearchClipboard& inAction ) = default;
    virtual ~ASearchClipboard() = default;

	virtual ASearchClipboard*		Clone() const { return new ASearchClipboard( *this ); }
	virtual ASearchClipboard*		Create() const { return new ASearchClipboard(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kSearchClipboard; }

	NSArray*						GetCurrentResults();
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	virtual	CFStringRef				CopyActionDescriptionString() const;

	void							SetAction( SearchClipboardActionType inValue ) { mAction = inValue; }
	SearchClipboardActionType		GetAction() const { return mAction; }

	NSString*						GetGenericNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }

	void							SetSearch( NSString* inString ) { mSearch = [inString copy]; SetRequiredCaptures(); }
	NSString*						GetSearch() const { return mSearch; }
	
	NSUInteger						RequiredCaptureCount() const; // returns 0 if an invalid pattern is present
	void							SetRequiredCaptures();

	void							SetCaptures( NSArray* inCaptures ) { mCaptures = [inCaptures copy]; check( RequiredCaptureCount() == 0 || mCaptures.count == RequiredCaptureCount() ); SetRequiredCaptures(); }
	NSArray*						GetCaptures() const { return mCaptures; }

};

