//
//  ASearchReplaceClipboard.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Mon 4 Apr 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ClipboardTarget.h"

// WARNING: enum valued used in menu selection in action editor
enum class SearchReplaceClipboardActionType {
	kCaseSensitiveString,
	kIgnoreCaseString,
	kCaseSensitiveRegEx,
	kIgnoreCaseRegEx,
	kDEFAULT = kIgnoreCaseString, 
};
#define kSearchReplaceClipboardActionTypeNamesArray {CFSTR("CaseSensitiveString"), CFSTR("IgnoreCaseString"), CFSTR("CaseSensitiveRegEx"), CFSTR("IgnoreCaseRegEx"), NULL}

class ASearchReplaceClipboard
	:	public HBaseAction
{

private:

	ClipboardTarget						mTarget;
	SearchReplaceClipboardActionType	mAction = SearchReplaceClipboardActionType::kDEFAULT;
	NSString*							mSearch = @"";
	NSString*							mReplace = @"";

public:

    ASearchReplaceClipboard() = default;
	ASearchReplaceClipboard( const ASearchReplaceClipboard& inAction ) = default;
    virtual ~ASearchReplaceClipboard() = default;

	virtual ASearchReplaceClipboard*	Clone() const { return new ASearchReplaceClipboard( *this ); }
	virtual ASearchReplaceClipboard*	Create() const { return new ASearchReplaceClipboard(); }
	
	virtual HActionType					GetActionType() const { return HActionType::kSearchReplaceClipboard; }

#if KMF_IS_ENGINE
    virtual float						DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual void						UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); }

	virtual Class						ActionController() const;
	
	virtual	NSString*					GetActionExtraHTML() const;
#endif

	virtual void						SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void						InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	virtual	CFStringRef					CopyActionDescriptionString() const;

	NSString*							GetGenericNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void								SetGenericNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }

	void								SetAction( SearchReplaceClipboardActionType inValue ) { mAction = inValue; }
	SearchReplaceClipboardActionType	GetAction() const { return mAction; }
	Boolean								IsRegExAction() const { return mAction == SearchReplaceClipboardActionType::kCaseSensitiveRegEx || mAction == SearchReplaceClipboardActionType::kIgnoreCaseRegEx; }

	void								SetSearch( NSString* inString ) { mSearch = inString; }
	NSString*							GetSearch() const { return mSearch; }
	
	void								SetReplace( NSString* inString ) { mReplace = inString; }
	NSString*							GetReplace() const { return mReplace; }
	
};

