//
//  ASearchReplaceVariable.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 19 Apr 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"

// WARNING: enum valued used in menu selection in action editor
enum class SearchReplaceVariableActionType {
	kCaseSensitiveString,
	kIgnoreCaseString,
	kCaseSensitiveRegEx,
	kIgnoreCaseRegEx,
	kDEFAULT = kIgnoreCaseString, 
};
#define kSearchReplaceVariableActionTypeNamesArray {CFSTR("CaseSensitiveString"), CFSTR("IgnoreCaseString"), CFSTR("CaseSensitiveRegEx"), CFSTR("IgnoreCaseRegEx"), NULL}

class ASearchReplaceVariable
	:	public HBaseAction
{

private:

	SearchReplaceVariableActionType	mAction;
	VariableContainer				mVariable;
	CFContainer<CFStringRef>		mSearch;
	CFContainer<CFStringRef>		mReplace;

public:

    ASearchReplaceVariable();
	ASearchReplaceVariable( const ASearchReplaceVariable& inAction );
    virtual ~ASearchReplaceVariable();

	virtual ASearchReplaceVariable*	Clone() const { return new ASearchReplaceVariable( *this ); }
	virtual ASearchReplaceVariable*	Create() const { return new ASearchReplaceVariable(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kSearchReplaceVariable; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	virtual	CFStringRef				CopyActionDescriptionString() const;

	void							SetAction( SearchReplaceVariableActionType inValue ) { mAction = inValue; }
	SearchReplaceVariableActionType	GetAction() const { return mAction; }
	Boolean							IsRegExAction() const { return mAction == SearchReplaceVariableActionType::kCaseSensitiveRegEx || mAction == SearchReplaceVariableActionType::kIgnoreCaseRegEx; }

	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	NSString*						GetVariable() const { return mVariable->GetNS(); }
	
	void							SetSearch( CFStringRef inString ) { mSearch = inString->Copy(); }
	CFStringRef						CopySearch() const { return mSearch->Copy(); }
	NSString*						GetSearch() const { return mSearch->GetNS(); }
	
	void							SetReplace( CFStringRef inString ) { mReplace = inString->Copy(); }
	CFStringRef						CopyReplace() const { return mReplace->Copy(); }
	NSString*						GetReplace() const { return mReplace->GetNS(); }
	
	
};

