//
//  ASearchVariable.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 1 Feb 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"

// WARNING: enum valued used in menu selection in action editor
enum class SearchVariableActionType {
	kCaseSensitiveRegEx,
	kIgnoreCaseRegEx,
	kDEFAULT = kIgnoreCaseRegEx, 
};
#define kSearchVariableActionTypeNamesArray {CFSTR("CaseSensitiveRegEx"), CFSTR("IgnoreCaseRegEx"), NULL}

class ASearchVariable
	:	public HBaseAction
{

private:

	SearchVariableActionType		mAction;
	VariableContainer				mVariable;
	NSString*						mSearch;
	NSArray*						mCaptures;

	NSArray*						PrivateGetCurrentResults( float* time, bool testRegex );

public:

    ASearchVariable();
	ASearchVariable( const ASearchVariable& inAction ) = default;
    virtual ~ASearchVariable() = default;

	virtual ASearchVariable*		Clone() const { return new ASearchVariable( *this ); }
	virtual ASearchVariable*		Create() const { return new ASearchVariable(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kSearchVariable; }

	NSArray*						GetCurrentResults();
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	virtual	CFStringRef				CopyActionDescriptionString() const;

	void							SetAction( SearchVariableActionType inValue ) { mAction = inValue; }
	SearchVariableActionType		GetAction() const { return mAction; }

	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	NSString*						GetVariable() const { return mVariable->GetNS(); }
	
	void							SetSearch( NSString* inString ) { mSearch = [inString copy]; SetRequiredCaptures(); }
	NSString*						GetSearch() const { return mSearch; }
	
	NSUInteger						RequiredCaptureCount() const; // returns 0 if an invalid pattern is present
	void							SetRequiredCaptures();

	void							SetCaptures( NSArray* inCaptures ) { mCaptures = [inCaptures copy]; check( RequiredCaptureCount() == 0 || mCaptures.count == RequiredCaptureCount() ); SetRequiredCaptures(); }
	NSArray*						GetCaptures() const { return mCaptures; }

};

