/*
 *  ASekectMenuItem.h
 *  Keyboard Maestro
 *
 *  Created Unknown
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "ApplicationTarget.h"

const Boolean kDefaultSelectMenuItemStopOnFailure = true;

class ASelectMenuItem : public HBaseAction
{
private:
	
	NSArray*						mMenuItemPath = @[@"",@""];
	ApplicationTarget				mTargetApplication;
	Boolean							mStopOnFailure = kDefaultSelectMenuItemStopOnFailure;

public:
	ASelectMenuItem() = default;
	ASelectMenuItem( const ASelectMenuItem& inAction ) = default;
	virtual ~ASelectMenuItem() = default;

	virtual ASelectMenuItem*		Clone() const { return new ASelectMenuItem( *this ); }
	virtual ASelectMenuItem*		Create() const { return new ASelectMenuItem(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kSelectMenuItem; }

	virtual CFStringRef				CopyActionDescriptionString() const;

#if KMF_IS_ENGINE
	float							DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	ApplicationTargetApplicationType		GetApplicationTargetType() const { return mTargetApplication.GetApplicationTargetType(); }
    virtual const XApplicationIdentifier&	GetTargetApplication() const { return mTargetApplication.GetTargetApplication(); }

	void							SetApplicationTargetApplication( const XApplicationIdentifier& inApp ) { mTargetApplication.SetApplicationTargetApplication( inApp ); }
	void							SetApplicationTargetFrontApplication() { mTargetApplication.SetApplicationTargetFrontApplication(); }

	Boolean							GetMenuKeyCommand( UInt32 &modifiers, UniChar &ch ) const;
	
	NSArray*						GetMenuItemPath() const { return mMenuItemPath; }
	void							SetMenuItemPath( NSArray* inPath ) { mMenuItemPath = [inPath copy]; }

	Boolean							GetStopOnFailure() const { return mStopOnFailure; }
	void							SetStopOnFailure( Boolean inStopOnFailure ) { mStopOnFailure = inStopOnFailure; }

	virtual BOOL					CanBeInstigatedByClickAction() const { return YES; }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
};

