//
//  ASendMIDIMessage.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Mon 15 Mar 2010.
//  Copyright (c) 2010 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"

enum class MIDIMessageType {
	kNoteOn,
	kNoteOff,
	kControlChange,
	kDEFAULT = kNoteOn,
};
#define kMIDIMessageTypeNamesArray {CFSTR("NoteOn"), CFSTR("NoteOff"), CFSTR("ControlChange"), NULL}

class ASendMIDIMessage
	:	public HBaseAction
{
private:

	MIDIMessageType					mMIDIMessageType;
	UInt32							mChannel;
	UInt32							mNote;
	UInt32							mValue;

public:

    ASendMIDIMessage();
	ASendMIDIMessage( const ASendMIDIMessage& inAction );
	ASendMIDIMessage( const MIDIMessageType& inActionType );

	virtual ASendMIDIMessage*		Clone() const { return new ASendMIDIMessage( *this ); }
	virtual ASendMIDIMessage*		Create() const { return new ASendMIDIMessage(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

	virtual NSString*				GetNewActionTitle() const;

    virtual HActionType				GetActionType() const { return HActionType::kSendMIDIMessage; }
	
	void							SetMIDIMessageType( MIDIMessageType inValue ) { mMIDIMessageType = inValue; }
	MIDIMessageType					GetMIDIMessageType() const { return mMIDIMessageType; }
	
	void							SetChannel( UInt32 inValue ) { mChannel = inValue; }
	UInt32							GetChannel() const { return mChannel; }
	
	void							SetNote( UInt32 inValue ) { mNote = inValue; }
	UInt32							GetNote() const { return mNote; }
	
	void							SetValue( UInt32 inValue ) { mValue = inValue; }
	UInt32							GetValue() const { return mValue; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
};

