/*
 *  ASetClipboardToPastClipboard.h
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Fri 2 May 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "ClipboardTarget.h"

const CFStringRef kDefaultPastClipboard = CFSTR("1");

class ASetClipboardToPastClipboard
	:	public HBaseAction
{

private:

	ClipboardTarget					mTarget;
    CFContainer<CFStringRef>		mPastClipboard = kDefaultPastClipboard;

public:

    ASetClipboardToPastClipboard() = default;
	ASetClipboardToPastClipboard( const ASetClipboardToPastClipboard& inAction ) = default;
    virtual ~ASetClipboardToPastClipboard() = default;

	virtual ASetClipboardToPastClipboard*		Clone() const { return new ASetClipboardToPastClipboard( *this ); }
	virtual ASetClipboardToPastClipboard*		Create() const { return new ASetClipboardToPastClipboard(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kSetClipboardToPastClipboard; }
	virtual	CFStringRef				CopyActionDescriptionString() const;

	void							SetPastClipboard( CFStringRef inString ) { mPastClipboard = inString->Copy(); }
	CFStringRef						CopyPastClipboard() const { return mPastClipboard->Copy(); }
	NSString*						GetPastClipboard() const { return mPastClipboard->GetNS(); }
	
	NSString*						GetGenericNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); }
	virtual Class					ActionController() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

