/*
 *  ASetClipboardToPastClipboard.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Fri 2 May 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "ASetClipboardToPastClipboard.h"
#import "CSClipboardDomain.h"
#import "XClipboardHistory.h"
#import "Calculate.h"
#if KMF_EDITOR_WINDOW
#import "ASetClipboardToPastClipboardEditor.h"
#endif
#if KMF_IS_ENGINE
#import "XErrorHandler.h"
#endif

CFStringRef
ASetClipboardToPastClipboard::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> titleFormat = CFSTR("aSetClipboardToPastClipboardFormat")->CopyLocalized();
	return CFString::CreateWithFormat( titleFormat, mTarget.GetDisplayName(), mPastClipboard->GetNS() );
}

#if KMF_IS_ENGINE

float
ASetClipboardToPastClipboard::DoAction()
{
	float time = 0;
	
	double doubleindex;
	if ( !Calculate::CalculateValue( mPastClipboard->GetNS(), doubleindex ) ) {
		XErrorHandlerMaybeLogFormat1("always", CFSTR("Delete Past Clipboard %@ failed"), mPastClipboard->GetNS() );
		doubleindex = -1;
	}
	
	SInt32 index = lround(doubleindex);

	SInt32 count = XClipboardHistory::Count();
	if ( 0 <= index && index < count ) {
		const XClipboardDataSet* pClip = XClipboardHistory::Get( index );
		if ( pClip != NULL ) {
			mTarget.WriteClipboardDataSet( *pClip );
			time = 20;
		}
	}
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetClipboardToPastClipboard::ActionController() const
{
	return [ASetClipboardToPastClipboardEditor class];
}

#endif

#pragma mark streaming

void
ASetClipboardToPastClipboard::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("PastExpression"), CFStringRef(mPastClipboard) );
	mTarget.SaveToDictionary( dictionary, CFSTR("Target") );
}

void
ASetClipboardToPastClipboard::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("PastExpression"), value, CFStringRef(NULL) );
	mPastClipboard = value ? value->Copy() : NULL;
	
	SInt32 index;
	if ( !mPastClipboard && dictionary->Get( CFSTR("Past"), index ) ) {
		mPastClipboard = Calculate::CreateDoubleValue( index );
	}
	
	if ( !mPastClipboard ) {
		mPastClipboard = kDefaultPastClipboard;
	}
	mTarget.InitializeFromDictionary( dictionary, CFSTR("Target") );
}
