/*
 *  ASetClipboardToText.h
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Fri 2 May 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "ClipboardTarget.h"
#import "StyledTextSource.h"

class ASetClipboardToText
	:	public HBaseAction
{
private:

	StyledTextSource				mStyledText;
	ClipboardTarget					mTarget;
	Boolean							mDisplayMode = false;
	
	virtual	CFStringRef				CopyActionDescriptionShortString() const;

public:
	
    ASetClipboardToText() = default;
	ASetClipboardToText( const ASetClipboardToText& inAction ) = default;
    virtual ~ASetClipboardToText() = default;

	virtual ASetClipboardToText*	Clone() const { return new ASetClipboardToText( *this ); }
	virtual ASetClipboardToText*	Create() const { return new ASetClipboardToText(); }

    virtual HActionType				GetActionType() const { return HActionType::kSetClipboardToText; }
	virtual	CFStringRef				CopyActionDescriptionString() const;

	void							SetPlainText( NSString* inString ) { mStyledText.SetPlainText( inString ); }
	void							SetStyledText( NSAttributedString* inString ) { mStyledText.SetStyledText( inString ); }

	NSString*						GetSourceString() const { return mStyledText.GetString(); }
	NSAttributedString*				GetSourceAttributedString() const { return mStyledText.GetAttributedString(); }

	Boolean							GetUseStyledText() const { return mStyledText.GetUseStyledText(); }

	void							SetDisplayMode( Boolean inDisplayMode ) { mDisplayMode = inDisplayMode; }
	Boolean							DisplayMode() const { return mDisplayMode; }

	NSString*						GetGenericNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); }
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

