/*
 *  ASetClipboardToText.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Fri 2 May 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "ASetClipboardToText.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "AInsertText.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "XClipboardDataSet.h"
#import "MTextResultsWindow.h"
#import "XClipboardHistory.h"
#import "TokenText.h"
#import "NSAttributedStringExtensions.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ASetClipboardToTextEditor.h"
#import "NSAttributedStringExtensions.h"
#endif

CFStringRef
ASetClipboardToText::CopyActionDescriptionShortString() const
{
	CFContainer<CFStringRef> format = CFSTR("aSetClipboardToText_DescriptionFormat")->CopyLocalized();
	NSString* textkey = mStyledText.GetUseStyledText() ? @"aSetClipboardToText_StyledText" : @"aSetClipboardToText_Text";
	return CFString::CreateWithFormat( format, mTarget.GetDisplayName(), [textkey localized] );
}

CFStringRef
ASetClipboardToText::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> summary = AInsertText::CopySummaryText( [mStyledText.GetAttributedString() string] );
	CFContainer<CFStringRef> shortstr = CopyActionDescriptionShortString();

	CFContainer<CFStringRef> format = CFSTR("aSetClipboardToText_DescriptionExtendedFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, shortstr.Get(), summary.Get() );
}

#if KMF_IS_ENGINE

float
ASetClipboardToText::DoAction()
{
	// Now replace any tokens with data
	NSMutableAttributedString* t = mStyledText.CopyMutableAttributedString();
	bool wasPlainText = t.wasPlainText;
	float time = TokenText::Process( t );
	
	if ( mDisplayMode ) {
		MTextResultsWindow::Display( MTextResultsWindowKind::kWindow, @"aSetClipboardToText_DisplayTitle", @"aSetClipboardToText_DisplayDescription", t );
	} else {
		mTarget.WriteClipboardDataSet( wasPlainText, t );
	}
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetClipboardToText::ActionController() const
{
	return [ASetClipboardToTextEditor class];
}

NSString*
ASetClipboardToText::GetActionHTMLDescriptionString() const
{
	CFContainer<CFStringRef> shortstr = CopyActionDescriptionShortString();
	return shortstr->GetNS();
}

NSString*
ASetClipboardToText::GetActionExtraHTML() const
{
	return mStyledText.GetDisplayHTMLContent();
}
#endif

#pragma mark streaming

void
ASetClipboardToText::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	mStyledText.SaveToDictionary( dictionary );
	dictionary->SetBoolean( CFSTR("JustDisplay"), mDisplayMode );
	mTarget.SaveToDictionary( dictionary, CFSTR("Target") );
}

void
ASetClipboardToText::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mStyledText.InitializeFromDictionary( dictionary );
	dictionary->GetBoolean( CFSTR("JustDisplay"), mDisplayMode, false );
	mTarget.InitializeFromDictionary( dictionary, CFSTR("Target") );
}
