//
//  ASetFileAttribute.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 11 Apr 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"
#import "AGetFileAttribute.h"
#import "VariableTransformationAction.h"

class ASetFileAttribute
	:	public HBaseAction,
		public VariableTransformationAction
{

private:

	FileAttributeType				mAttribute;
	CFContainer<CFStringRef>		mPath;
	CFContainer<CFStringRef>		mText;

public:

    ASetFileAttribute();
	ASetFileAttribute( const ASetFileAttribute& inAction ) = default;
    virtual ~ASetFileAttribute() = default;

	virtual ASetFileAttribute*		Clone() const { return new ASetFileAttribute( *this ); }
	virtual ASetFileAttribute*		Create() const { return new ASetFileAttribute(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kSetFileAttribute; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	
	virtual NSString*				GetVariable() const { return nil; }
	virtual NSString*				GetVariableValue() const;

	//	virtual NSString*				GetVariable() const { return nil; };
	virtual CFStringRef				CopyTransformationResult( NSUInteger* errorPos = NULL, float* time = NULL ) const;

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif
	
	FileAttributeType				GetAttribute() const { return mAttribute; }
	void							SetAttribute( FileAttributeType inAttribute ) { mAttribute = inAttribute; }
	
	CFStringRef						CopyPath() const { return mPath->Copy(); }
	NSString*						GetPath() const { return mPath->GetNS(); }
	void							SetPath( CFStringRef inPath ) { mPath = inPath->Copy(); }
	
	CFStringRef						CopyText() const { return mText->Copy(); }
	NSString*						GetText() const { return mText->GetNS(); }
	void							SetText( CFStringRef inText ) { mText = inText->Copy(); }
		
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
	
};

