//
//  ASetFindPasteboard.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 15 Apr 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "ASetFindPasteboard.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "AInsertText.h"
#import "TokenText.h"
#import "NSStringExtensions.h"
#import "XRunningApplicationsList.h"

#if KMF_IS_ENGINE
#import "XClipboardDataSet.h"
#import "MTextResultsWindow.h"
#import "XClipboardHistory.h"
#import "KMNotifications.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ASetFindPasteboardEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetFindPasteboard::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> summary = AInsertText::CopySummaryText( mText );
	CFContainer<CFStringRef> format = CFSTR("aSetFindPasteboard_DescriptionFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, CFStringRef(summary) );
}

#if KMF_IS_ENGINE

float
ASetFindPasteboard::DoAction()
{
	float time;
	NSString* result = TokenText::GetResult( mText, &time );

	NSPasteboard* pasteboard = [NSPasteboard pasteboardWithName:NSFindPboard];
	[pasteboard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
	[pasteboard setString:result forType:NSStringPboardType];

	ProcessSerialNumber frontProcess;
	if ( GetFrontProcess( &frontProcess ) == noErr ) {
		ProcessSerialNumber	tempProcess;
		if ( XRunningApplicationsList::FindApplicationWithBundleID( kFindApplicationDontValidate, @"com.apple.dock", tempProcess ) ) {
			verify_noerr( SetFrontProcessWithOptions( &tempProcess, kSetFrontProcessFrontWindowOnly ) );
			verify_noerr( SetFrontProcessWithOptions( &frontProcess, kSetFrontProcessFrontWindowOnly ) );
		}
	}

	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetFindPasteboard::ActionController() const
{
	return [ASetFindPasteboardEditor class];
}

#endif

#pragma mark streaming

void
ASetFindPasteboard::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Text"), CFBridgingToCFString(mText) );
}

void
ASetFindPasteboard::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Text"), value, CFSTR("") );
	check( value );
	mText = value->GetNS();
}
