//
//  ASetKeychainPasswordToText.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 6 Feb 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "ASetKeychainPasswordToText.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "AInsertText.h"
#import "TokenText.h"
#import "XErrorHandler.h"

#if KMF_IS_ENGINE
#import "KMNotifications.h"
#import "NSStringRegularExpressions.h"
#import "NSStringExtensions.h"
#import "AExecuteBaseScript.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ASetKeychainPasswordToTextEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetKeychainPasswordToText::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("aSetKeychainPasswordToText_DescriptionFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, mPasswordName, mText );
}

#if KMF_IS_ENGINE

NSString*
ASetKeychainPasswordToText::GetShellCommand()
{
	float ptime = 0;
	float ttime = 0;

	NSString* passwordname = TokenText::GetResult( mPasswordName, &ptime );
	check( passwordname ); // TokenText cannot fail
	NSString* passwordaccount = TokenText::GetResult( mPasswordAccount, &ptime );
	check( passwordaccount ); // TokenText cannot fail

	NSString* text = TokenText::GetResult( mText, &ttime );
	check( text ); // TokenText cannot fail

	return [NSString stringWithFormat:@"security add-generic-password -U -a %@ -s %@ -w %@ -T \"\"", [passwordaccount shellQuotedPath], [passwordname shellQuotedPath], [text shellQuotedPath]];
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetKeychainPasswordToText::ActionController() const
{
	return [ASetKeychainPasswordToTextEditor class];
}

#endif

#pragma mark streaming

void
ASetKeychainPasswordToText::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("PasswordName"), CFBridgingToCFString(mPasswordName) );
	dictionary->Set( CFSTR("Text"), CFBridgingToCFString(mText) );
	dictionary->Set( CFSTR("PasswordAccount"), CFBridgingToCFString(mPasswordAccount) );
}

void
ASetKeychainPasswordToText::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("PasswordName"), value, CFSTR("Password Name") );
	check( value );
	mPasswordName = value->GetNS();
	dictionary->Get( CFSTR("PasswordAccount"), value, CFSTR("%UserLoginID%") );
	check( value );
	mPasswordAccount = value->GetNS();
	dictionary->Get( CFSTR("Text"), value, CFSTR("") );
	check( value );
	mText = value->GetNS();
}
