//
//  ASetMacroEnable.h"
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 1 Mar 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "ASetMacroEnable.h"
#import "MMacroDomain.h"
#import "MMacro.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "KMPreferences.h"
#import "XNotificationCenter.h"
#import "KMNotifications.h"
#endif

#if KMF_EDITOR_WINDOW
#import "ASetMacroEnableEditor.h"
#endif

ASetMacroEnable::ASetMacroEnable()
	:	HBaseAction(),
		mAction( kSetMacroEnableDEFAULT ),
		mMacroUID( CFSTR("")->Copy() )
{
}

ASetMacroEnable::ASetMacroEnable( const ASetMacroEnable& inAction )
	:	HBaseAction( inAction ),
		mAction( inAction.mAction ),
		mMacroUID( inAction.mMacroUID )
{
}
	
ASetMacroEnable::~ASetMacroEnable()
{
}

CFStringRef
ASetMacroEnable::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> result;
	CFContainer<CFStringRef> format;
	switch ( mAction ) {
		case kSetMacroEnableEnable:
			format = CFSTR("kSetMacroEnableEnableTitleFormat")->CopyLocalized();
			break;
		case kSetMacroEnableDisable: 
			format = CFSTR("kSetMacroEnableDisableTitleFormat")->CopyLocalized();
			break;
		case kSetMacroEnableToggle:
			format = CFSTR("kSetMacroEnableToggleTitleFormat")->CopyLocalized();
			break;
	}
	
	MMacroSerializedObject* obj = MMacroSerializedObject::FindObjectByUID( mMacroUID );
	NSString* name;
	if ( obj ) {
		name = obj->GetName();
	} else {
		name = [@"kSetMacroEnableNoMacro" localized];
	}
	NSString* type;
	if ( obj && obj->GetAsMacroGroup() ) {
		type = [@"kSetMacroEnableIsMacroGroup" localized];
	} else {
		type = [@"kSetMacroEnableIsMacro" localized];
	}
	
	return CFString::CreateWithFormat( format, type, name );
}


#if KMF_IS_ENGINE

float
ASetMacroEnable::DoAction()
{
	MMacroSerializedObject* obj = MMacroSerializedObject::FindObjectByUID( mMacroUID.Get() );
	
	if ( obj ) {
		Boolean enable;
		switch ( mAction ) {
			case kSetMacroEnableEnable:
				enable = true;
				break;
			case kSetMacroEnableDisable: 
				enable = false;
				break;
			case kSetMacroEnableToggle:
				enable = !obj->IsActive();
				break;
		}
		if ( enable != obj->IsActive() ) {
			KMPreferences::SetMacroEnableOverride( mMacroUID, enable );
			obj->SetIsActive( enable );
			MMacroDomain::GetObject()->AdjustMacrosForFrontApp();
			if ( obj->GetAsMacroGroup() ) {
				MMacroDomain::GetObject()->AdjustMacroGroupForEnableToggle( obj->GetAsMacroGroup() );
			} else {
				MMacroDomain::GetObject()->AdjustMacroForEnableToggle( obj->GetAsMacro() );
			}
			XNotificationCenter::PostNotification( KMNotifications::kSetMacroEnable );
		}
	}
	
	return 20;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetMacroEnable::ActionController() const
{
	return [ASetMacroEnableEditor class];
}

#endif

#pragma mark streaming

static const CFStringRef kSetMacroEnableTargetTypeNames[] = kSetMacroEnableTypeNamesArray;

void
ASetMacroEnable::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Action"), kSetMacroEnableTargetTypeNames[mAction] );
	dictionary->Set( CFSTR("MacroUID"), mMacroUID.Get() );
}

void
ASetMacroEnable::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mAction = (SetMacroEnableType) dictionary->GetEnumerated( CFSTR("Action"), kSetMacroEnableTargetTypeNames, kSetMacroEnableDEFAULT );

	CFStringRef macroUID;
	XAssert( dictionary->Get( CFSTR("MacroUID"), macroUID ) );
	mMacroUID = macroUID->Copy();
}

