//
//  ASetSafariCheckbox.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Sun 23 Jun 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "ASetSafariCheckbox.h"
#import "KMDefines.h"
#import "NSStringExtensions.h"
#import "TokenText.h"
#import "Calculate.h"

#if KMF_IS_ENGINE
#endif
#if KMF_EDITOR_WINDOW
#import "ASetSafariCheckboxEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetSafariCheckbox::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format;
	if ( [mValue isEqualToString:@"0"] ) {
		format = CFSTR("aSetSafariCheckbox_DescriptionUncheckedFormat");
	} else if ( [mValue isEqualToString:@"1"] ) {
		format = CFSTR("aSetSafariCheckbox_DescriptionCheckedFormat");
	} else {
		format = CFSTR("aSetSafariCheckbox_DescriptionValueFormat");
	}
	format = format->CopyLocalized();
	return CFString::CreateWithFormat( format, GetWebBrowserName(), mField, mValue );
}

NSString*
ASetSafariCheckbox::GetNewActionTitle() const
{
	return [NSString stringWithFormat:[@"aSetSafariCheckbox_NewActionFormat" localized], GetWebBrowserName()];
}

CFStringRef
ASetSafariCheckbox::CopyTransformationResult( NSUInteger* errorPos, float* outTime ) const
{
	NSString* result = Calculate::Calculate( mValue, errorPos );
	if ( result ) {
		double v;
		if ( !Calculate::GetValue( result, v ) ) {
			result = nil;
		} else if ( EffectivelyTrueCalcuation( v ) ) {
			result = @"1";
		} else {
			result = @"0";
		}
	}
	if ( outTime ) *outTime = 5;
	return (CFStringRef)CFBridgingRetain( result );
}

NSString*
ASetSafariCheckbox::GetVariableValue() const
{
	float time;
	NSString* field = TokenText::GetResult( mField, &time );
	bool value;
	NSString* result;
	if ( mWebBrowserInterface->CheckboxValue( field, value ) ) {
		result = value ? @"1" : @"0";
	} else {
		result = nil;
	}
	return result;
}

#if KMF_IS_ENGINE

float
ASetSafariCheckbox::DoAction()
{
	NSUInteger errorPos;
	float time1 = 0;
	float time2 = 0;
	float time = 0;
	NSString* result = GetTransformationResult( &errorPos, &time1 );

	if ( result ) {
		NSString* field = TokenText::GetResult( mField, &time2 );
		mWebBrowserInterface->SetCheckboxValue( field, ![result isEqualToString:@"0"] );
		time = time1 + time2 + 5;
	}

	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetSafariCheckbox::ActionController() const
{
	return [ASetSafariCheckboxEditor class];
}

#endif

#pragma mark streaming

void
ASetSafariCheckbox::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Field"), CFBridgingToCFString(mField) );
	dictionary->Set( CFSTR("Value"), CFBridgingToCFString(mValue) );
	mWebBrowserInterface->SaveToDictionary( dictionary );
}

void
ASetSafariCheckbox::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Value"), value, CFSTR("1") );
	check( value );
	mValue = value->GetNS();
	dictionary->Get( CFSTR("Field"), value, CFSTR(kWebBrowserInterfaceDefaultField) );
	check( value );
	mField = value->GetNS();
	mWebBrowserInterface = WebBrowserInterface::InitializeFromDictionary( dictionary );
}
