//
//  ASetSafariFieldToText.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 13 Nov 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "VariableTransformationAction.h"
#import "WebBrowserInterface.h"

class ASetSafariFieldToText
	:	public HBaseAction,
		public VariableTransformationAction
{
private:

	NSString*						mField = @kWebBrowserInterfaceDefaultField;
    NSString*						mText = @"";
	WebBrowserInterface*			mWebBrowserInterface = WebBrowserInterface::Safari();

public:
	
    ASetSafariFieldToText() = default;
    ASetSafariFieldToText( WebBrowserInterface* interface ) : mWebBrowserInterface( interface ) { ; }
	ASetSafariFieldToText( const ASetSafariFieldToText& inAction ) = default;
    virtual ~ASetSafariFieldToText() = default;

	virtual ASetSafariFieldToText*	Clone() const { return new ASetSafariFieldToText( *this ); }
	virtual ASetSafariFieldToText*	Create() const { return new ASetSafariFieldToText(); }
	
	virtual WebBrowserInterface*	GetWebBrowserInterface() const { return mWebBrowserInterface; }
	virtual NSString*				GetWebBrowserName() const { return mWebBrowserInterface->GetName(); }

    virtual HActionType				GetActionType() const { return HActionType::kSetSafariFieldToText; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual NSString*				GetNewActionTitle() const;
 	virtual	XIcon					GetActionDescriptionXIcon() const { return mWebBrowserInterface->GetXIcon(); }

	void							SetField( NSString* inString ) { check(inString); mField = [inString copy]; }
	NSString*						GetField() const { return mField; }

	void							SetText( NSString* inString ) { check(inString); mText = [inString copy]; }
	NSString*						GetText() const { return mText; }

	virtual NSString*				GetVariable() const { return nil; }
	virtual NSString*				GetVariableValue() const;

	virtual CFStringRef				CopyTransformationResult( NSUInteger* errorPos = NULL, float* time = NULL ) const;

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

