//
//  ASetSafariFieldToText.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 13 Nov 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "ASetSafariFieldToText.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "AInsertText.h"
#import "TokenText.h"
#import "ASetVariableToSafariField.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "XClipboardDataSet.h"
#import "MTextResultsWindow.h"
#import "XClipboardHistory.h"
#import "KMNotifications.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ASetSafariFieldToTextEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetSafariFieldToText::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("aSetSafariFieldToText_DescriptionFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, GetWebBrowserName(), mField );
}

NSString*
ASetSafariFieldToText::GetNewActionTitle() const
{
	return [NSString stringWithFormat:[@"aSetSafariFieldToText_NewActionFormat" localized], GetWebBrowserName()];
}

CFStringRef
ASetSafariFieldToText::CopyTransformationResult( NSUInteger* errorPos, float* outTime ) const
{
	float time;
	CFContainer<CFStringRef> result = TokenText::CopyResult( CFBridgingToCFString(mText), time );
	if ( errorPos ) *errorPos = 0;
	if ( outTime ) *outTime = time;
	return result->Copy();
}

NSString*
ASetSafariFieldToText::GetVariableValue() const
{
	ASetVariableToSafariField* action = new ASetVariableToSafariField( mWebBrowserInterface );
	action->SetField( mField );
	CFContainer<CFStringRef> result = action->CopyTransformationResult( NULL, NULL );
	return result ? result->GetNS() : nil;
}

#if KMF_IS_ENGINE

float
ASetSafariFieldToText::DoAction()
{
	NSUInteger errorPos;
	float time1, time2;
	NSString* result = GetTransformationResult( &errorPos, &time1 );

	NSString* field = TokenText::GetResult( mField, &time2 );
	mWebBrowserInterface->SetFieldValue( field, result );

	return time1 + time2 + 10;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetSafariFieldToText::ActionController() const
{
	return [ASetSafariFieldToTextEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
{{ text }}
)RAW";

NSString*
ASetSafariFieldToText::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"text" : UserTextForHTML( mText ),
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
ASetSafariFieldToText::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Text"), CFBridgingToCFString(mText) );
	dictionary->Set( CFSTR("Field"), CFBridgingToCFString(mField) );
	mWebBrowserInterface->SaveToDictionary( dictionary );
}

void
ASetSafariFieldToText::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Text"), value, CFSTR("") );
	check( value );
	mText = value->GetNS();
	dictionary->Get( CFSTR("Field"), value, CFSTR(kWebBrowserInterfaceDefaultField) );
	check( value );
	mField = value->GetNS();
	mWebBrowserInterface = WebBrowserInterface::InitializeFromDictionary( dictionary );
}
