//
//  ASetSafariRadio.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Sun 23 Jun 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "VariableTransformationAction.h"
#import "WebBrowserInterface.h"

class ASetSafariRadio
	:	public HBaseAction,
		public VariableTransformationAction
{
private:

	NSString*						mField = @kWebBrowserInterfaceDefaultField;
    NSString*						mValue = @"";
	WebBrowserInterface*			mWebBrowserInterface = WebBrowserInterface::Safari();

public:
	
    ASetSafariRadio() = default;
    ASetSafariRadio( WebBrowserInterface* interface ) : mWebBrowserInterface( interface ) { ; }
	ASetSafariRadio( const ASetSafariRadio& inAction ) = default;
    virtual ~ASetSafariRadio() = default;

	virtual ASetSafariRadio*	Clone() const { return new ASetSafariRadio( *this ); }
	virtual ASetSafariRadio*	Create() const { return new ASetSafariRadio(); }
	
	virtual WebBrowserInterface*	GetWebBrowserInterface() const { return mWebBrowserInterface; }
	virtual NSString*				GetWebBrowserName() const { return mWebBrowserInterface->GetName(); }

    virtual HActionType				GetActionType() const { return HActionType::kSetSafariRadio; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual NSString*				GetNewActionTitle() const;
 	virtual	XIcon					GetActionDescriptionXIcon() const { return mWebBrowserInterface->GetXIcon(); }

	void							SetField( NSString* inString ) { check(inString); mField = [inString copy]; }
	NSString*						GetField() const { return mField; }

	void							SetValue( NSString* inString ) { check(inString); mValue = [inString copy]; }
	NSString*						GetValue() const { return mValue; }

	virtual NSString*				GetVariable() const { return nil; }
	virtual NSString*				GetVariableValue() const;

	virtual CFStringRef				CopyTransformationResult( NSUInteger* errorPos = NULL, float* time = NULL ) const;

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

