//
//  ASetSafariRadio.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Sun 23 Jun 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "ASetSafariRadio.h"
#import "KMDefines.h"
#import "NSStringExtensions.h"
#import "TokenText.h"
#import "Calculate.h"

#if KMF_IS_ENGINE
#endif
#if KMF_EDITOR_WINDOW
#import "ASetSafariRadioEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetSafariRadio::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("aSetSafariRadio_DescriptionFormat");
	format = format->CopyLocalized();
	return CFString::CreateWithFormat( format, GetWebBrowserName(), mField, mValue );
}

NSString*
ASetSafariRadio::GetNewActionTitle() const
{
	return [NSString stringWithFormat:[@"aSetSafariRadio_NewActionFormat" localized], GetWebBrowserName()];
}

CFStringRef
ASetSafariRadio::CopyTransformationResult( NSUInteger* errorPos, float* outTime ) const
{
	float time;
	CFContainer<CFStringRef> result = TokenText::CopyResult( CFBridgingToCFString(mText), time );
	if ( errorPos ) *errorPos = 0;
	if ( outTime ) *outTime = time;
	return result->Copy();
}

NSString*
ASetSafariRadio::GetVariableValue() const
{
	float time;
	NSString* field = TokenText::GetResult( mField, &time );
	return mWebBrowserInterface->RadioValue( field );
}

#if KMF_IS_ENGINE

float
ASetSafariRadio::DoAction()
{
	NSUInteger errorPos;
	float time1 = 0;
	float time2 = 0;
	float time = 0;
	NSString* result = GetTransformationResult( &errorPos, &time1 );

	if ( result ) {
		NSString* field = TokenText::GetResult( mField, &time2 );
		mWebBrowserInterface->SetRadioValue( field, result );
		time = time1 + time2 + 5;
	}

	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetSafariRadio::ActionController() const
{
	return [ASetSafariRadioEditor class];
}

#endif

#pragma mark streaming

void
ASetSafariRadio::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Field"), CFBridgingToCFString(mField) );
	dictionary->Set( CFSTR("Value"), CFBridgingToCFString(mValue) );
	mWebBrowserInterface->SaveToDictionary( dictionary );
}

void
ASetSafariRadio::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Value"), value, CFSTR("1") );
	check( value );
	mValue = value->GetNS();
	dictionary->Get( CFSTR("Field"), value, CFSTR(kWebBrowserInterfaceDefaultField) );
	check( value );
	mField = value->GetNS();
	mWebBrowserInterface = WebBrowserInterface::InitializeFromDictionary( dictionary );
}
