//
//  ASetVariableToCalculation.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 20 Apr 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "ASetVariableToCalculation.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "AInsertText.h"
#import "KMVariables.h"
#import "Calculate.h"

#if KMF_IS_ENGINE
#import "XErrorHandler.h"
#import "KMNotifications.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ASetVariableToCalculationEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetVariableToCalculation::CopyActionDescriptionString() const
{
//	CFContainer<CFStringRef> summary = AInsertText::CopySummaryText( mText );
	CFContainer<CFStringRef> format = CFSTR("aSetVariableToCalculation_DescriptionFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, CFStringRef(mVariable) );
}

CFStringRef
ASetVariableToCalculation::CopyTransformationResult( NSUInteger* errorPos, float* outTime ) const
{
	NSString* result = Calculate::Calculate( mText, errorPos );
	double v;
	if ( result && mUseFormat && Calculate::CalculateValue( result, v ) ) {
		NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
		[numberFormatter setFormat:mFormat];
		result = [numberFormatter stringFromNumber:[NSNumber numberWithDouble:v]];
	}
	if ( outTime ) *outTime = 10;
	return (CFStringRef)CFBridgingRetain( result );
}

#if KMF_IS_ENGINE

float
ASetVariableToCalculation::DoAction()
{
	NSUInteger errorPos;
	float time;
	
	NSString* result = GetTransformationResult( &errorPos, &time );
	if ( result ) {
		KMVariables::Set( mVariable->GetNS(), result, KMNotifications::kLoadVariableFromEngineAction );
	} else {
		XErrorHandlerMaybeLogFormat2("always", CFSTR("Calculation %@ failed with error at %d"), mText, errorPos );
	}
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetVariableToCalculation::ActionController() const
{
	return [ASetVariableToCalculationEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
{{ calculation }}
{% if useformat %}With format: {{ format | htmlencode }}{% /if %}
)RAW";

NSString*
ASetVariableToCalculation::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"calculation" : UserTextForHTML( mText ),
									@"useformat" : @(mUseFormat),
									@"format" : mFormat,
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
ASetVariableToCalculation::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Variable"), CFStringRef(mVariable) );
	dictionary->Set( CFSTR("Text"), CFBridgingToCFString(mText) );
	dictionary->SetBoolean( CFSTR("UseFormat"), mUseFormat );
	if ( mUseFormat ) {
		dictionary->Set( CFSTR("Format"), CFBridgingToCFString(mFormat) );
	}
}

void
ASetVariableToCalculation::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Variable"), value, CFSTR("Variable") );
	check( value );
	mVariable = value->Copy();
	dictionary->Get( CFSTR("Text"), value, CFSTR("1") );
	check( value );
	mText = value->GetNS();
	dictionary->GetBoolean( CFSTR("UseFormat"), mUseFormat, false );
	dictionary->Get( CFSTR("Format"), value, CFSTR("0.00") );
	check( value );
	mFormat = value->GetNS();
}
