//
//  ASetVariableToKeychainPassword.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 6 Feb 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"
#import "AExecuteRootScript.h"

class ASetVariableToKeychainPassword
	:	public AExecuteRootScript
{
private:

	VariableContainer				mVariable = @"Variable";
    NSString*						mPasswordName = @"Password Name";
    NSString*						mPasswordAccount = @"%UserLoginID%";

	virtual NSString*				GetScriptTypeName() const { return @"Set Variable to Keychain Password"; }

#if KMF_IS_ENGINE

	virtual NSString*				GetLongScriptDisplayName() const { return @"Set Variable to Keychain Password"; }
	virtual NSString*				GetShellCommand();

	virtual Boolean					StripTrailingLineFeed() { return false; }
	virtual Boolean					TrimResults() { return false; }

	virtual MTextResultsWindowKind	DisplayKind() { return MTextResultsWindowKind::kNone; }

	virtual void					DisplayResults( NSString* results );
	
#endif

public:
	
    ASetVariableToKeychainPassword() = default;
	ASetVariableToKeychainPassword( const ASetVariableToKeychainPassword& inAction ) = default;
    virtual ~ASetVariableToKeychainPassword() = default;

	virtual ASetVariableToKeychainPassword*	Clone() const { return new ASetVariableToKeychainPassword( *this ); }
	virtual ASetVariableToKeychainPassword*	Create() const { return new ASetVariableToKeychainPassword(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kSetVariableToKeychainPassword; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	
	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	virtual NSString*				GetVariable() const { return mVariable->GetNS(); }
	
	void							SetPasswordName( NSString* inString ) { check(inString); mPasswordName = [inString copy]; }
	NSString*						GetPasswordName() const { return mPasswordName; }

	void							SetPasswordAccount( NSString* inString ) { check(inString); mPasswordAccount = [inString copy]; }
	NSString*						GetPasswordAccount() const { return mPasswordAccount; }

#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

