//
//  ASetVariableToKeychainPassword.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 6 Feb 2013.
//  Copyright (c) 2013 Stairways Software. All rights reserved.
//

#import "ASetVariableToKeychainPassword.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "AInsertText.h"
#import "TokenText.h"
#import "XErrorHandler.h"

#if KMF_IS_ENGINE
#import "KMNotifications.h"
#import "NSStringRegularExpressions.h"
#import "NSStringExtensions.h"
#import "AExecuteBaseScript.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ASetVariableToKeychainPasswordEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetVariableToKeychainPassword::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> format = CFSTR("aSetVariableToKeychainPassword_DescriptionFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, CFStringRef(mVariable), mPasswordName );
}

#if KMF_IS_ENGINE

static NSString* kPasswordCaptureTextRegex = @R"RAW(^password: "(.*)")RAW"; // WARNING: " is unquoted by security!
static NSString* kPasswordCaptureBytesRegex = @R"RAW(^password: 0x([A-Fa-f0-9]+))RAW"; // WARNING: " is unquoted by security!

NSString*
ASetVariableToKeychainPassword::GetShellCommand()
{
	float ptime = 0;

	NSString* passwordname = TokenText::GetResult( mPasswordName, &ptime );
	check( passwordname ); // TokenText cannot fail
	NSString* passwordaccount = TokenText::GetResult( mPasswordAccount, &ptime );
	check( passwordaccount ); // TokenText cannot fail

	return [NSString stringWithFormat:@"security find-generic-password -a %@ -s %@ -g", [passwordaccount shellQuotedPath], [passwordname shellQuotedPath]];
}

void
ASetVariableToKeychainPassword::DisplayResults( NSString* results )
{
	if ( !IsEmpty( results ) ) {
		NSArray* components = [results captureComponentsMatchedByRegex:kPasswordCaptureTextRegex options:NSRegularExpressionCaseInsensitive];
		if ( components.count == 2 ) {
			results = components[1];
		} else {
			components = [results captureComponentsMatchedByRegex:kPasswordCaptureBytesRegex options:NSRegularExpressionCaseInsensitive];
			if ( components.count == 2 ) {
				NSString* bytes = components[1];
				if ( bytes.length % 2 == 0 ) {
					NSInteger len = bytes.length / 2;
					std::unique_ptr<uint8_t[]> b(new uint8_t[len]);
					for ( NSInteger i = 0; i < len; i++ ) {
						UniChar ch1 = [bytes characterAtIndex:i*2];
						UniChar ch2 = [bytes characterAtIndex:i*2+1];
						ch1 -= '0';
						if ( ch1 > 9 ) ch1 -= 'A'-'0'-10;
						ch2 -= '0';
						if ( ch2 > 9 ) ch2 -= 'A'-'0'-10;
						UniChar c = ch1*16+ch2;
						if ( c == 0 ) {
							len = i;
							break;
						}
						b[i] = c;
					}
					results = [[NSString alloc] initWithBytes:&b[0] length:len encoding:NSUTF8StringEncoding];
					if ( !results ) {
						check( false );
						results = [[NSString alloc] initWithBytes:&b[0] length:len encoding:NSMacOSRomanStringEncoding];
					}
				}
			}
		}
	}
	if ( IsEmpty( results ) ) {
		results = @"";
	}
	KMVariables::Set( mVariable->GetNS(), results, KMNotifications::kLoadVariableFromEngineAction );
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetVariableToKeychainPassword::ActionController() const
{
	return [ASetVariableToKeychainPasswordEditor class];
}

#endif

#pragma mark streaming

void
ASetVariableToKeychainPassword::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Variable"), CFStringRef(mVariable) );
	dictionary->Set( CFSTR("PasswordName"), CFBridgingToCFString(mPasswordName) );
	dictionary->Set( CFSTR("PasswordAccount"), CFBridgingToCFString(mPasswordAccount) );
}

void
ASetVariableToKeychainPassword::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Variable"), value, CFSTR("Variable") );
	check( value );
	mVariable = value->Copy();
	dictionary->Get( CFSTR("PasswordName"), value, CFSTR("Password Name") );
	check( value );
	mPasswordName = value->GetNS();
	dictionary->Get( CFSTR("PasswordAccount"), value, CFSTR("%UserLoginID%") );
	check( value );
	mPasswordAccount = value->GetNS();
	if ( IsEmpty( mPasswordAccount ) ) {
		mPasswordAccount = @"%UserLoginID%";
	}
}
