//
//  ASetVariableToSafariField.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 13 Nov 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "VariableTransformationAction.h"
#import "WebBrowserInterface.h"
#import "KMVariables.h"

class ASetVariableToSafariField
	:	public HBaseAction,
		public VariableTransformationAction
{
private:

	VariableContainer				mVariable = @"Variable";
    NSString*						mField = @kWebBrowserInterfaceDefaultField;

	WebBrowserInterface*			mWebBrowserInterface = WebBrowserInterface::Safari();
	
public:
	
    ASetVariableToSafariField() = default;
    ASetVariableToSafariField( WebBrowserInterface* interface ) : mWebBrowserInterface( interface ) { ; }
	ASetVariableToSafariField( const ASetVariableToSafariField& inAction ) = default;
    virtual ~ASetVariableToSafariField() = default;

	virtual ASetVariableToSafariField*	Clone() const { return new ASetVariableToSafariField( *this ); }
	virtual ASetVariableToSafariField*	Create() const { return new ASetVariableToSafariField(); }
	
	virtual WebBrowserInterface*	GetWebBrowserInterface() const { return mWebBrowserInterface; }
	virtual NSString*				GetWebBrowserName() const { return mWebBrowserInterface->GetName(); }

    virtual HActionType				GetActionType() const { return HActionType::kSetVariableToSafariField; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	virtual NSString*				GetNewActionTitle() const;
 	virtual	XIcon					GetActionDescriptionXIcon() const { return mWebBrowserInterface->GetXIcon(); }

	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	virtual NSString*				GetVariable() const { return mVariable->GetNS(); }

	void							SetField( NSString* inString ) { check(inString); mField = [inString copy]; }
	NSString*						GetField() const { return mField; }
	
	virtual CFStringRef				CopyTransformationResult( NSUInteger* errorPos = NULL, float* time = NULL ) const;

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

