//
//  ASetVariableToSafariField.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 13 Nov 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "ASetVariableToSafariField.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "AInsertText.h"
#import "TokenText.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "XClipboardDataSet.h"
#import "MTextResultsWindow.h"
#import "XClipboardHistory.h"
#import "KMNotifications.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ASetVariableToSafariFieldEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetVariableToSafariField::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> summary = AInsertText::CopySummaryText( mField );
	CFContainer<CFStringRef> format = CFSTR("aSetVariableToSafariField_DescriptionFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, CFStringRef(mVariable), GetWebBrowserName(), CFStringRef(summary) );
}

NSString*
ASetVariableToSafariField::GetNewActionTitle() const
{
	return [NSString stringWithFormat:[@"aSetVariableToSafariField_NewActionFormat" localized], GetWebBrowserName()];
}

CFStringRef
ASetVariableToSafariField::CopyTransformationResult( NSUInteger* errorPos, float* outTime ) const
{
	float time;
	NSString* title = TokenText::GetResult( mField, &time );
	NSString* result = mWebBrowserInterface->FieldValue( title );
	if ( !result ) {
		result = @"";
	}
	if ( outTime ) *outTime = time;
	return CFBridgingToCFString(result)->Copy();
}

#if KMF_IS_ENGINE

float
ASetVariableToSafariField::DoAction()
{
	NSUInteger errorPos;
	float time;

	NSString* result = GetTransformationResult( &errorPos, &time );
	check( result ); // cannot fail
	KMVariables::Set( mVariable->GetNS(), result, KMNotifications::kLoadVariableFromEngineAction );
	return 5 + time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetVariableToSafariField::ActionController() const
{
	return [ASetVariableToSafariFieldEditor class];
}

#endif

#pragma mark streaming

void
ASetVariableToSafariField::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Variable"), CFStringRef(mVariable) );
	dictionary->Set( CFSTR("Field"), CFBridgingToCFString(mField) );
	mWebBrowserInterface->SaveToDictionary( dictionary );
}

void
ASetVariableToSafariField::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Variable"), value, CFSTR("Variable") );
	check( value );
	mVariable = value->Copy();
	dictionary->Get( CFSTR("Field"), value, CFSTR(kWebBrowserInterfaceDefaultField) );
	check( value );
	mField = value->GetNS();
	mWebBrowserInterface = WebBrowserInterface::InitializeFromDictionary( dictionary );
}
