//
//  ASetVariableToText.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 15 Apr 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "ASetVariableToText.h"
#import "KMDefines.h"
#import "XCoreFoundation.h"
#import "AInsertText.h"
#import "TokenText.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "XClipboardDataSet.h"
#import "MTextResultsWindow.h"
#import "XClipboardHistory.h"
#import "KMNotifications.h"
#endif
#if KMF_EDITOR_WINDOW
#import "ASetVariableToTextEditor.h"
#import "TemplateEngine.h"
#endif

CFStringRef
ASetVariableToText::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> summary = AInsertText::CopySummaryText( mText );
	CFContainer<CFStringRef> format = CFSTR("aSetVariableToText_DescriptionFormat")->CopyLocalized();
	return CFString::CreateWithFormat( format, CFStringRef(mVariable), CFStringRef(summary) );
}

CFStringRef
ASetVariableToText::CopyTransformationResult( NSUInteger* errorPos, float* outTime ) const
{
	float time;
	CFContainer<CFStringRef> result = TokenText::CopyResult( CFBridgingToCFString(mText), time );
	if ( errorPos ) *errorPos = 0;
	if ( outTime ) *outTime = time;
	return result->Copy();
}


#if KMF_IS_ENGINE

float
ASetVariableToText::DoAction()
{
	NSUInteger errorPos;
	float time;

	NSString* result = GetTransformationResult( &errorPos, &time );
	check( result ); // TokenText cannot fail
	KMVariables::Set( mVariable->GetNS(), result, KMNotifications::kLoadVariableFromEngineAction );
	return time;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASetVariableToText::ActionController() const
{
	return [ASetVariableToTextEditor class];
}

NSString*
ASetVariableToText::GetActionHTMLDescriptionString() const
{
	return [NSString stringWithFormat:[@"aSetVariableToText_HTMLFormat" localized], mVariable->GetNS()];
}

static NSString* kActionDisplayHTML = @R"RAW(
{{ text }}
)RAW";

NSString*
ASetVariableToText::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"text" : UserTextForHTML( mText ),
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
ASetVariableToText::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Variable"), CFStringRef(mVariable) );
	dictionary->Set( CFSTR("Text"), CFBridgingToCFString(mText) );
}

void
ASetVariableToText::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef value;
	dictionary->Get( CFSTR("Variable"), value, CFSTR("Variable") );
	check( value );
	mVariable = value->Copy();
	dictionary->Get( CFSTR("Text"), value, CFSTR("") );
	check( value );
	mText = value->GetNS();
}
