//
//  ASpeakText.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Wed 18 May 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "ASpeakText.h"
#import "KMDefines.h"
#import "TokenText.h"
#if KMF_EDITOR_WINDOW
#import "ASpeakTextEditor.h"
#import "TemplateEngine.h"
#import "NSStringExtensions.h"
#endif

ASpeakText::ASpeakText()
	:	HBaseAction(),
		mText( CFSTR("kSpeakTextDefault")->CopyLocalized() ),
		mVoice( CFSTR("")->Copy() )
{
#if KMF_IS_ENGINE
	mSynthesizer = nil;
#endif
}

ASpeakText::ASpeakText( const ASpeakText& inAction )
	:	HBaseAction( inAction ),
		mText( inAction.mText ),
		mVoice( inAction.mVoice )
{
#if KMF_IS_ENGINE
	mSynthesizer = nil;
#endif
}

ASpeakText::~ASpeakText()
{
#if KMF_IS_ENGINE
	[mSynthesizer stopSpeaking];
#endif
}

CFStringRef
ASpeakText::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> theFormatStr = CFSTR("hkSpeakTextTitle")->CopyLocalized();
	
	return CFString::CreateWithFormat( theFormatStr, CFStringRef(mText) );
}

#if KMF_IS_ENGINE

float
ASpeakText::DoAction()
{
	NSString* text = TokenText::GetResult( mText->GetNS() );
	mSynthesizer = [[NSSpeechSynthesizer alloc] initWithVoice:mVoice->GetNS()];
	if ( !mSynthesizer ) {
		mSynthesizer = [[NSSpeechSynthesizer alloc] init];
	}
	[mSynthesizer startSpeakingString:text];
	
	return 10;
}

Boolean
ASpeakText::IsActionCompleted()
{
	return ![mSynthesizer isSpeaking];
}

void
ASpeakText::ActionAborted()
{
	[mSynthesizer stopSpeaking];
	mSynthesizer = nil;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASpeakText::ActionController() const
{
	return [ASpeakTextEditor class];
}

static NSString* NameForVoiceID( NSString* voiceid )
{
	return [[NSSpeechSynthesizer attributesForVoice:voiceid] objectForKey:NSVoiceName];
}

NSString*
ASpeakText::GetActionHTMLDescriptionString() const
{
	NSString* key = @"hkSpeakTextVoiceTitle";
	NSString* voice = NameForVoiceID( mVoice->GetNS() );
	if ( !voice ) {
		voice = NameForVoiceID( [NSSpeechSynthesizer defaultVoice] );
		key = @"hkSpeakTextDefaultVoiceTitle";

	}
	return [NSString stringWithFormat:[key localized], voice];
}

static NSString* kActionDisplayHTML = @R"RAW(
{{ text }}
)RAW";

NSString*
ASpeakText::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"text" : UserTextForHTML( mText->GetNS() ),
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
ASpeakText::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Text"), CFStringRef(mText) );
	dictionary->Set( CFSTR("Voice"), CFStringRef(mVoice) );
}

void
ASpeakText::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	CFStringRef newText;
	XAssert( dictionary->Get( CFSTR("Text"), newText ) );
	mText = newText->Copy();
	
	CFStringRef newVoice;
	dictionary->Get( CFSTR("Voice"), newVoice, CFSTR("") );
	check( newVoice );
	mVoice = newVoice->Copy();
}
