//
//  ASubstring.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 5 Apr 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"
#import "ClipboardReference.h"
#import "StringRange.h"

enum class SubstringSourceType { // WARNING: tag valued in ASubstring.xib match this enum
	kClipboard,
	kNamedClipboard,
	kVariable,
	kDEFAULT = kClipboard,
};
#define kSubstringSourceTypeNamesArray {CFSTR("Clipboard"), CFSTR("NamedClipboard"), CFSTR("Variable"), NULL}

class ASubstring
	:	public HBaseAction
{

private:

	SubstringSourceType				mSource = SubstringSourceType::kDEFAULT;
	ClipboardReference				mClipboardReference;
	VariableContainer				mVariable = CFSTR("Variable");
	StringRangeType					mStringRange = StringRangeType::kDEFAULT;
	Boolean							mDelete = false;
	NSString*						mFirst = @"2";
	NSString*						mSecond = @"4";

public:

    ASubstring() = default;
	ASubstring( const ASubstring& inAction ) = default;
    virtual ~ASubstring() = default;

	virtual ASubstring*				Clone() const { return new ASubstring( *this ); }
	virtual ASubstring*				Create() const { return new ASubstring(); }
	
	virtual HActionType				GetActionType() const { return HActionType::kSubstring; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mClipboardReference.UpdateClipboardReference(); }
#endif

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionExtraHTML() const;
#endif

	bool							NeedSecond() const { return StringRange::CountTypeValues( mStringRange ) > 1; }
	
	SubstringSourceType				GetSource() const { return mSource; }
	void							SetSource( SubstringSourceType inSource ) { mSource = inSource; }

	virtual void					SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	virtual NSString*				GetVariable() const { return mVariable->GetNS(); }
	
	NSString*						GetClipboardUID() const { return mClipboardReference.GetClipboardUID(); }
	void							SetClipboardUID( NSString* inUID ) { mClipboardReference.SetClipboardUID( inUID ); }
	
	StringRangeType					GetStringRange() const { return mStringRange; }
	void							SetStringRange( StringRangeType inStringRange ) { mStringRange = inStringRange; }

	NSString*						GetFirst() const { return mFirst; }
	void							SetFirst( NSString* inFirst ) { mFirst = [inFirst copy]; }

	Boolean							GetDelete() const { return mDelete; }
	void							SetDelete( Boolean inDelete ) { mDelete = inDelete; }

	NSString*						GetSecond() const { return mSecond; }
	void							SetSecond( NSString* inSecond ) { mSecond = [inSecond copy]; }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );
	
};

