/*
 *  ASystemAction.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Tue May 07 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"

enum class SystemActionType {
	kSleep,
	kRestart,
	kShutdown,
	kLogOut,
	kLoginWindow,
	kOpenCloseCDROM,
	kMuteVolume,
	kIncreaseVolume,
	kDecreaseVolume,
	kIncreaseBrightness,
	kDecreaseBrightness,
	kActivateScreenSaver,
	kStopScreenSaver,
	kSleepDisplay,
	kWakeDisplay,
	kSystemBeep,
	kShowMacroPalette,
	kHideMacroPalette,
	kToggleMacroPalette,
	kShowStatusMenu,
	kDEFAULT = kSystemBeep,
};
#define kSystemActionNamesArray {CFSTR("Sleep"), CFSTR("Restart"), CFSTR("Shutdown"), CFSTR("LogOut"), CFSTR("LoginWindow"), \
	CFSTR("OpenCloseCDROM"), CFSTR("MuteVolume"), CFSTR("IncreaseVolume"), CFSTR("DecreaseVolume"), CFSTR("IncreaseBrightness"), CFSTR("DecreaseBrightness"), \
	CFSTR("ActivateScreenSaver"), CFSTR("StopScreenSaver"), CFSTR("SleepDisplay"), CFSTR("WakeDisplay"), \
	CFSTR("SystemBeep"), CFSTR("ShowMacroPalette"), CFSTR("HideMacroPalette"), CFSTR("ToggleMacroPalette"), \
	CFSTR("ShowStatusMenu"), NULL}


class ASystemAction
	:	public HBaseAction
{
private:

	SystemActionType				mSystemAction = SystemActionType::kDEFAULT;

public:

    ASystemAction() = default;
	ASystemAction( const ASystemAction& inAction ) = default;
	ASystemAction( const SystemActionType& inActionType ) : mSystemAction(inActionType) { }
	virtual ~ASystemAction() = default;

	virtual ASystemAction*			Clone() const { return new ASystemAction( *this ); }
	virtual ASystemAction*			Create() const { return new ASystemAction(); }
	
	virtual	CFStringRef				CopyActionTitle() const;
	virtual	XIcon					GetActionDescriptionXIcon() const;

	virtual NSString*				GetNewActionTitle() const { return NSCS(CopyActionDescriptionString()); }

    virtual HActionType				GetActionType() const { return HActionType::kSystemAction; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	void							SetSystemAction( SystemActionType inValue ) { mSystemAction = inValue; }
	SystemActionType				GetSystemAction() const { return mSystemAction; }
};

