/*
 *  ASystemVolume.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Thu 27 Nov 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "ASystemVolume.h"
#import "KMDefines.h"
#import "KMIconRegistry.h"
#import "XIcon.h"
#import <unistd.h>
#import "XSystemProperties.h"
#import "NSAppleScriptExtensions.h"

#if KMF_EDITOR_WINDOW
#import "ASystemVolumeEditor.h"
#endif

const int kDefaultSystemVolume = 50;

ASystemVolume::ASystemVolume()
	:	HBaseAction(),
		mSystemVolume( kDefaultSystemVolume )
{
}

ASystemVolume::ASystemVolume( const ASystemVolume& inAction )
	:	HBaseAction( inAction ),
		mSystemVolume( inAction.mSystemVolume )
{
}

CFStringRef
ASystemVolume::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> theTitleFormat = CFSTR("hkSystemVolumeTitleFormat")->CopyLocalized();
	return CFString::CreateWithFormat( theTitleFormat, mSystemVolume );
}

#if KMF_IS_ENGINE

float
ASystemVolume::DoAction()
{
	(void)[NSAppleScript executeWithSource:@"set volume output volume %d" numericParameter:mSystemVolume];
	return 10;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ASystemVolume::ActionController() const
{
	return [ASystemVolumeEditor class];
}

#endif

#pragma mark streaming

void
ASystemVolume::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->SetNumber( CFSTR("Volume"), mSystemVolume );
}

void
ASystemVolume::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	if ( !dictionary->Get( CFSTR("Volume"), mSystemVolume ) ) mSystemVolume = kDefaultSystemVolume;
}

