//
//  ATransformImage.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Tue 1 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "ClipboardTarget.h"

enum class TransformImageActionType { // WARNING: tag valued in ATransformImage.xib match this enum
	kFlipHorizontal,
	kFlipVertical,
	kRotateCW,
	kRotate180,
	kRotateCCW,
	kFreeRotate,
	kDEFAULT = kFlipHorizontal,
};
#define kTransformImageActionTypeNamesArray {CFSTR("FlipHorizontal"), CFSTR("FlipVertical"), CFSTR("RotateCW"), CFSTR("Rotate180"), CFSTR("RotateCCW"), CFSTR("FreeRotate"), NULL}

#define kTransformImageDefaultRotation @"45"

class ATransformImage
	:	public HBaseAction
{
private:

	TransformImageActionType		mTransformAction = TransformImageActionType::kDEFAULT;
	ClipboardTarget					mTarget;
	NSString*						mRotation = kTransformImageDefaultRotation;

protected:

public:

    ATransformImage() = default;
    ATransformImage( TransformImageActionType inAction );
	ATransformImage( const ATransformImage& inAction ) = default;
	
    virtual ~ATransformImage() = default;

	virtual ATransformImage*		Clone() const { return new ATransformImage( *this ); }
	virtual ATransformImage*		Create() const { return new ATransformImage(); }
	
	virtual NSString*				GetNewActionTitle() const;
	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kTransformImage; }
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { mTarget.UpdateToDefaultValues(); }
#endif
	
	TransformImageActionType		GetTransformAction() const { return mTransformAction; }
	void							SetTransformAction( TransformImageActionType inTransformAction ) { mTransformAction = inTransformAction; }
	
	NSString*						GetGenericNamedClipboardUID() const { return mTarget.GetGenericNamedClipboardUID(); }
	void							SetGenericNamedClipboardUID( NSString* inUID ) { mTarget.SetGenericNamedClipboardUID( inUID ); }
	
	void							SetRotation( NSString* inRotation ) { mRotation = [inRotation copy]; }
	void							SetRotation( double inRotation ) { mRotation = [NSString stringWithFormat:@"%f",inRotation]; }
	NSString*						GetRotation() const { return mRotation; }

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	virtual	NSString*				GetActionHTMLDescriptionString() const;
#endif

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

