//
//  ATriggerByName.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 24 Sep 2009.
//  Copyright (c) 2009 Stairways Software. All rights reserved.
//

#import "ATriggerByName.h"
#import "NSColorExtensions.h"
#import "NSStringExtensions.h"

#if KMF_IS_ENGINE
#import "SearchWindowController.h"
#import "TriggerByNameController.h"
#import "TokenText.h"
#endif
#if KMF_IS_EDITOR
#import "ATriggerByNameEditor.h"
#import "TemplateEngine.h"
#import "NSImageExtensions.h"
#endif

static const int kDefaultAlpha = 90;
static const RGBColor kDefaultColor = { 0, 152, 255 };

static NSString* GetDefaultSearchTitle()
{
	static NSString* kDefaultSearchTitle = [@"DefaultSearchTitle" localized];
	return kDefaultSearchTitle;
}

static NSString* GetDefaultSearchURL()
{
	static NSString* kDefaultSearchURL = [@"DefaultSearchURL" localized];
	return kDefaultSearchURL;
}

ATriggerByName::ATriggerByName()
	:	HBaseAction()
{
	mColor = [NSColor colorWithCalibratedRed:kDefaultColor.red/255.0 green:kDefaultColor.green/255.0 blue:kDefaultColor.blue/255.0 alpha:kDefaultAlpha/100.0];
}

ATriggerByName::ATriggerByName( const ATriggerByName& inAction )
	:	HBaseAction( inAction ),
		mColor( [inAction.mColor copy] )
{
#if KMF_IS_ENGINE
	mTriggerByNameController = nil;
#endif
}

ATriggerByName::~ATriggerByName()
{
#if KMF_IS_ENGINE
	[mTriggerByNameController Kill];
#endif
}

void
ATriggerByName::SetColor( NSColor* inColor )
{
	mColor = [[inColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] copy];
}

#if KMF_IS_ENGINE

float
ATriggerByName::DoAction()
{
	float time = 0;
	mTriggerByNameController = [TriggerByNameController newTriggerWindow];
	[mTriggerByNameController setColor:mColor];
	[mTriggerByNameController Show];
	return 20 + time;
}

Boolean
ATriggerByName::IsActionCompleted()
{
	if ( [mTriggerByNameController IsComplete] ) {
		mTriggerByNameController = nil;
	}
	return mTriggerByNameController == nil;
}

void
ATriggerByName::ActionAborted()
{
	[mTriggerByNameController Kill];
	mTriggerByNameController = nil;
}

#endif

#if KMF_EDITOR_WINDOW

Class
ATriggerByName::ActionController() const
{
	return [ATriggerByNameEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
 <span class="IconTitle">Color: </span><img src={{ color | inlineimage: 16 }} alt="Swatch" width="16" height="16" /><br />
)RAW";

NSString*
ATriggerByName::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"color" : [NSImage imageWithSwatch:mColor],
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
ATriggerByName::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	[mColor saveWithAlphaToDictionary:dictionary];
}

void
ATriggerByName::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mColor = [[NSColor colorFromDictionary:dictionary defaultColor:kDefaultColor defaultAlpha:kDefaultAlpha] copy];
}

