//
//  ATrimImage.cpp
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 3 May 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "ATrimImage.h"

#if KMF_EDITOR_WINDOW
#import "ATrimImageEditor.h"
#import "NSStringExtensions.h"
#endif

#if KMF_IS_ENGINE
#import "XErrorHandler.h"
#import "NSImageExtensions.h"
#endif


ATrimImage::ATrimImage()
	:	HBaseAction()
{
}

ATrimImage::ATrimImage( const ATrimImage& inAction )
	:	HBaseAction( inAction ),
		mTarget( inAction.mTarget )

{
}
	
ATrimImage::~ATrimImage()
{
}

#if KMF_IS_ENGINE

float
ATrimImage::DoAction()
{	
	const XClipboardDataSet* clip = mTarget.GetClipboardDataSet();
	if ( clip ) {
		NSImage* image = clip->Image();
		if ( image ) {
			image = [image trimImage];
			if ( image ) {
				mTarget.WriteClipboardDataSet( XClipboardDataSet( image ) );
			}
		} else {
			XErrorHandlerMaybeLogMessage("always", CFSTR("Transform Image cannot get source image") );
		}
	}
	
	return 10;
}


#endif

#if KMF_EDITOR_WINDOW

Class
ATrimImage::ActionController() const
{
	return [ATrimImageEditor class];
}

NSString*
ATrimImage::GetActionHTMLDescriptionString() const
{
	return [NSString stringWithFormat:[@"hkTrimImageHTMLTitle" localized], mTarget.GetDisplayName()];
}

#endif

#pragma mark streaming

void
ATrimImage::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	mTarget.SaveToDictionary( dictionary, CFSTR("Target") );
}

void
ATrimImage::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mTarget.InitializeFromDictionary( dictionary, CFSTR("Target") );
}
