//
//  AUseVariable.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 21 Apr 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "KMVariables.h"

// WARNING: enum valued used in menu selection in action editor
enum class UseVariableActionType {
	kSetMouse,
	kSetWindowPosition,
	kSetWindowSize,
	kSetWindowFrame,
	kSetWindowByName,
	kSetWindowByNameContains,
	kSetWindowByNameMatches,
	kSetApplicationByName,
	kSetApplicationByNameContains,
	kSetApplicationByNameMatches,
	kSetSystemVolume,
	kDEFAULT = UseVariableActionType::kSetMouse, 
};
#define kUseVariableActionTypeNamesArray {CFSTR("SetMouse"), CFSTR("SetWindowPosition"), CFSTR("SetWindowSize"), CFSTR("SetWindowFrame"), CFSTR("SetWindowByName"), CFSTR("SetWindowByNameContains"), CFSTR("SetWindowByNameMatches"), CFSTR("SetApplicationByName"), CFSTR("SetApplicationByNameContains"), CFSTR("SetApplicationByNameMatches"), CFSTR("SetSystemVolume"), NULL}

class AUseVariable
	:	public HBaseAction
{
private:

	UseVariableActionType			mAction;
	VariableContainer				mVariable;
	
public:
	
    AUseVariable();
	AUseVariable( const AUseVariable& inAction );
    virtual ~AUseVariable();

	virtual AUseVariable*	Clone() const { return new AUseVariable( *this ); }
	virtual AUseVariable*	Create() const { return new AUseVariable(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kUseVariable; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
	
	void							SetAction( UseVariableActionType inValue ) { mAction = inValue; }
	UseVariableActionType			GetAction() const { return mAction; }
	
	void							SetVariable( NSString* inVariable ) { check( inVariable ); mVariable = [inVariable copy]; }
	NSString*						GetVariable() const { return mVariable->GetNS(); }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

