//
//  AWhile.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Fri 18 Mar 2011.
//  Copyright (c) 2011 Stairways Software. All rights reserved.
//

#import "HBaseAction.h"
#import "CConditionList.h"
#import "HActionList.h"

class AWhile
	:	public HBaseAction
{

private:

#if KMF_IS_ENGINE
	NSUUID*							mTrackingID;
	Boolean							mExecuting;
#endif
	
	CConditionList					mConditions;
	HActionList						mActionList;

public:

    AWhile();
	AWhile( const AWhile& inAction );
    virtual ~AWhile();

	virtual AWhile*					Clone() const { return new AWhile( *this ); }
	virtual AWhile*					Create() const { return new AWhile(); }
	
	virtual	CFStringRef				CopyActionDescriptionString() const;

    virtual HActionType				GetActionType() const { return HActionType::kWhile; }
	virtual Boolean					IsLoop() const { return true; }

	virtual Boolean					HasTimeOut() const { return true; }
	virtual EventTime				DefaultTimeOutPeriod() const { return kVeryLongDefaultTimeOutValue; }
	
#if KMF_IS_ENGINE
    virtual float					DoAction();
	virtual Boolean					IsActionCompleted();
	virtual void					ActionAborted();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
	
	virtual	NSString*				GetActionHTMLDescriptionString() const;
	virtual NSString*				GetActionExtraHTML() const;
#endif

	const CConditionList&			GetConditionList() const { return mConditions; }
	void							SetConditionList( const CConditionList& inConditions ) { mConditions = inConditions; }
	
	const HActionList&				GetActionList() const { return mActionList; }
	void							SetActionList( const HActionList& inActions ) { mActionList = inActions; }
	void							SetActionAtIndex( const CFIndex& inIndex, HBaseAction* inAction ) { mActionList.SetActionAtIndex( inIndex, inAction ); }
	
	virtual void					DoForEverySubAction( ActionBlock inApplier ) { mActionList.DoForEveryActionAndSubAction( inApplier ); }

	virtual BOOL					MatchesTextSearch( NSString* search ) const;
	
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

};

