//
//  AWindowSwitcher.cpp
//  Keyboard Maestro
//
//  Created by Michael Kamprath on Tue Jul 09 2002.
//  Copyright (c) 2004 Stairways Software. All rights reserved.
//

#import "AWindowSwitcher.h"
#import "XAXWindowElementList.h"
#import "KMIconRegistry.h"
#import "NSColorExtensions.h"

#if KMF_IS_ENGINE
#import "WSHUDController.h"
#import "HExecuteActionListManager.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AWindowSwitcherEditor.h"
#import "TemplateEngine.h"
#import "NSImageExtensions.h"
#endif

static const RGBColor kDefaultColor = {0, 128, 128};

enum {
	kDefaultAlpha = 70,
};

AWindowSwitcher::AWindowSwitcher()
	:	HBaseAction(), 
		mColor( [NSColor colorWithCalibratedRed:kDefaultColor.red/255.0 green:kDefaultColor.green/255.0 blue:kDefaultColor.blue/255.0 alpha:kDefaultAlpha/100.0] )
{
}

#if KMF_IS_ENGINE

float
AWindowSwitcher::DoAction()
{
	return [WSHUDController TriggerFired:this modifiers:HExecuteActionListManager::GetTrippedModifiers()] ? 5 : 0.5;
}


#endif

void AWindowSwitcher::SetSwitcherColor( NSColor* inColor )
{
	mColor = [[inColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] copy];
}

#if KMF_EDITOR_WINDOW

Class
AWindowSwitcher::ActionController() const
{
	return [AWindowSwitcherEditor class];
}

static NSString* kActionDisplayHTML = @R"RAW(
 <span class="IconTitle">Color: </span><img src={{ color | inlineimage: 16 }} alt="Swatch" width="16" height="16" /><br />
 {% if sort %}Sort alphabetically.<br />{% /if %}
 {% if placeundermouse %}Place switcher under mouse.<br />{% /if %}
 {% if rollover %}Mouse rollover enabled.<br />{% /if %}
)RAW";

NSString*
AWindowSwitcher::GetActionExtraHTML() const
{
	NSDictionary *variables = @{
									@"sort" : @(mSortAlphabetically),
									@"placeundermouse" : @(mSwitcherUnderMouse),
									@"rollover" : @(mMouseRollover),
									@"color" : [NSImage imageWithSwatch:mColor],
	};
	return [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
}

#endif

#pragma mark streaming

void
AWindowSwitcher::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	[mColor saveWithAlphaToDictionary:dictionary];
	dictionary->SetBoolean( CFSTR("SwitcherUnderMouse"), mSwitcherUnderMouse );
	dictionary->SetBoolean( CFSTR("MouseRollover"), mMouseRollover );
	dictionary->SetBoolean( CFSTR("SortAlphabetically"), mSortAlphabetically );
}

void
AWindowSwitcher::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	mColor = [[NSColor colorFromDictionary:dictionary defaultColor:kDefaultColor defaultAlpha:kDefaultAlpha] copy];
	dictionary->GetBoolean( CFSTR("SwitcherUnderMouse"), mSwitcherUnderMouse, false );
	dictionary->GetBoolean( CFSTR("MouseRollover"), mMouseRollover, true );
	dictionary->GetBoolean( CFSTR("SortAlphabetically"), mSortAlphabetically, false );
}

