/*
 *  AiTunesControl.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sat Oct 26 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"

enum class iTunesControlCommand {
	kPlayPause,
	kStop,
	kRewind,
	kFastForward,
	kPause,
	kNextTrack,
	kBackTrack,
	kPlayRandomTrack,
	kPlayNamedTrack,
	kPlayNamedPlaylist,
	kPlayRandomPlaylistTrack,
	kIncreaseVolume,
	kDecreaseVolume,
	kSetVolume,
	kIncreaseRating,
	kDecreaseRating,
	LASTENUM( kSetRating ),
	kDEFAULT = kPlayPause,
} ;
#define kiTunesControlCommandNamesArray {CFSTR("PlayPause"), CFSTR("Stop"), CFSTR("Rewind"), CFSTR("FastForward"), \
	CFSTR("Pause"), CFSTR("NextTrack"), CFSTR("BackTrack"), CFSTR("PlayRandomTrack"), CFSTR("PlayNamedTrack"), \
	CFSTR("PlayNamedPlaylist"), CFSTR("PlayRandomPlaylistTrack"), \
	CFSTR("IncreaseVolume"), CFSTR("DecreaseVolume"), CFSTR("SetVolume"), \
	CFSTR("IncreaseRating"), CFSTR("DecreaseRating"), CFSTR("SetRating"), \
	NULL}

class AiTunesControl
	:	public HBaseAction
{
private:

	iTunesControlCommand			mCommand;
	CFContainer<CFStringRef>		mArgumentString;

protected:

#if KMF_IS_ENGINE
    virtual float					DoAction();
#endif
#if KMF_EDITOR_WINDOW
	virtual Class					ActionController() const;
#endif

public:
    AiTunesControl();
 	AiTunesControl( const AiTunesControl& inAction );
 	AiTunesControl( const iTunesControlCommand& inCommand );
	virtual ~AiTunesControl();

	virtual AiTunesControl*			Clone() const { return new AiTunesControl( *this ); }
	virtual AiTunesControl*			Create() const { return new AiTunesControl(); }
	
    virtual HActionType				GetActionType() const { return HActionType::kiTunesControl; }
	virtual	CFStringRef				CopyActionDescriptionString() const;
    virtual XIcon					GetActionXIcon() const;

	virtual NSString*				GetNewActionTitle() const;

	void							SetCommand( iTunesControlCommand inCommand );
	iTunesControlCommand			GetCommand() const;

	void							SetArgumentString( CFStringRef inString );
	void							SetArgumentInteger( NSInteger inValue );
	CFStringRef						CopyArgumentString() const;
	NSString*						GetArgumentString() const { return CFBridgingToNS( mArgumentString ); }
	NSInteger						GetArgumentInteger() const { return mArgumentString->GetIntValue(); }

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const;
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException );

	static CFStringRef				CopyRatingString( int rating, bool full );

};

