/*
 *  AiTunesControl.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sat Oct 26 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "AiTunesControl.h"
#import "KMIconRegistry.h"
#import "XException.h"
#import "XCoreFoundation.h"
#import "NSStringExtensions.h"
#if KMF_IS_ENGINE
#import "NSAppleScriptExtensions.h"
#import "TokenText.h"
#import "Calculate.h"
#import "XErrorHandler.h"
#endif
#if KMF_EDITOR_WINDOW
#import "AiTunesControlEditor.h"
#endif


AiTunesControl::AiTunesControl()
	:	HBaseAction(),
		mCommand( iTunesControlCommand::kDEFAULT ),
		mArgumentString( CFSTR( "" )->Copy() )
{
}

AiTunesControl::AiTunesControl( const AiTunesControl& inAction )
	:	HBaseAction( inAction ),
		mCommand( inAction.mCommand ),
		mArgumentString( inAction.mArgumentString )
{
}

AiTunesControl::AiTunesControl( const iTunesControlCommand& inCommand )
	:	HBaseAction(),
		mCommand( inCommand )
{
	switch ( inCommand ) {
		case iTunesControlCommand::kPlayRandomPlaylistTrack:
		case iTunesControlCommand::kPlayNamedTrack:
		case iTunesControlCommand::kPlayNamedPlaylist:
			mArgumentString = CFSTR("iTunesCmd_Untitled")->CopyLocalized();
			break;
		case iTunesControlCommand::kSetVolume:
			mArgumentString = CFSTR("50");
			break;
		case iTunesControlCommand::kSetRating:
			mArgumentString = CFSTR("100");
			break;
		case iTunesControlCommand::kPlayPause:
		case iTunesControlCommand::kStop:
		case iTunesControlCommand::kRewind:
		case iTunesControlCommand::kFastForward:
		case iTunesControlCommand::kPause:
		case iTunesControlCommand::kNextTrack:
		case iTunesControlCommand::kBackTrack:
		case iTunesControlCommand::kPlayRandomTrack:
		case iTunesControlCommand::kIncreaseVolume:
		case iTunesControlCommand::kDecreaseVolume:
		case iTunesControlCommand::kIncreaseRating:
		case iTunesControlCommand::kDecreaseRating:
			mArgumentString = CFSTR("")->Copy();
			break;
	}
}

AiTunesControl::~AiTunesControl()
{
}

CFStringRef
AiTunesControl::CopyRatingString( int rating, bool full  )
{
	return (CFStringRef)CFBridgingRetain([NSString stringWithRatingStars:rating full:full]);
}

static CFStringRef
ActionDescriptionStringKey( iTunesControlCommand command, bool newaction )
{
	CFStringRef theTitleKey;
	switch ( command ) {
		case iTunesControlCommand::kPlayPause:
			theTitleKey = CFSTR("iTunesCmd_PlayPause_Title");
			break;
		case iTunesControlCommand::kStop:
			theTitleKey = CFSTR("iTunesCmd_Stop_Title");
			break;
		case iTunesControlCommand::kRewind:
			theTitleKey = CFSTR("iTunesCmd_Rewind_Title");
			break;
		case iTunesControlCommand::kFastForward:
			theTitleKey = CFSTR("iTunesCmd_FastForward_Title");
			break;
		case iTunesControlCommand::kPause:
			theTitleKey = CFSTR("iTunesCmd_Pause_Title");
			break;
		case iTunesControlCommand::kNextTrack:
			theTitleKey = CFSTR("iTunesCmd_NextTrack_Title");
			break;
		case iTunesControlCommand::kBackTrack:
			theTitleKey = CFSTR("iTunesCmd_BackTrack_Title");
			break;
		case iTunesControlCommand::kPlayRandomTrack:
			theTitleKey = CFSTR("iTunesCmd_PlayRandomTrack_Title");
			break;
		case iTunesControlCommand::kPlayRandomPlaylistTrack:
			theTitleKey = newaction ? CFSTR("iTunesCmd_PlayRandomPlaylistTrack_Title") : CFSTR("iTunesCmd_PlayRandomPlaylistTrack_TitleFormat");
			break;
		case iTunesControlCommand::kPlayNamedTrack:
			theTitleKey = newaction ? CFSTR("iTunesCmd_PlayNamedTrack_Title") : CFSTR("iTunesCmd_PlayNamedTrack_TitleFormat");
			break;
		case iTunesControlCommand::kPlayNamedPlaylist:
			theTitleKey = newaction ? CFSTR("iTunesCmd_PlayNamedPlaylist_Title") : CFSTR("iTunesCmd_PlayNamedPlaylist_TitleFormat");
			break;
		case iTunesControlCommand::kIncreaseVolume:
			theTitleKey = newaction ? CFSTR("iTunesCmd_IncreaseVolume_ShortTitle") : CFSTR("iTunesCmd_IncreaseVolume_Title");
			break;
		case iTunesControlCommand::kDecreaseVolume:
			theTitleKey = newaction ? CFSTR("iTunesCmd_DecreaseVolume_ShortTitle") : CFSTR("iTunesCmd_DecreaseVolume_Title");
			break;
		case iTunesControlCommand::kSetVolume:
			theTitleKey = newaction ? CFSTR("iTunesCmd_SetVolume_Title") : CFSTR("iTunesCmd_SetVolume_TitleFormat");
			break;
		case iTunesControlCommand::kIncreaseRating:
			theTitleKey = CFSTR("iTunesCmd_IncreaseRating_Title");
			break;
		case iTunesControlCommand::kDecreaseRating:
			theTitleKey = CFSTR("iTunesCmd_DecreaseRating_Title");
			break;
		case iTunesControlCommand::kSetRating:
			theTitleKey = newaction ? CFSTR("iTunesCmd_SetRating_Title") : CFSTR("iTunesCmd_SetRating_TitleFormat");
			break;
	}
	return theTitleKey;
}


CFStringRef AiTunesControl::CopyActionDescriptionString() const
{
	CFContainer<CFStringRef> theDisplayArgument = mArgumentString->Copy();
	if ( mCommand == iTunesControlCommand::kSetRating ) {
		theDisplayArgument = CopyRatingString( mArgumentString->GetIntValue(), false );
	}

	CFContainer<CFStringRef> theTitle = ActionDescriptionStringKey( mCommand, false )->CopyLocalized();
	if ( theTitle->Contains( CFSTR("%@") ) ) {
		theTitle = CFString::CreateWithFormat( theTitle, CFStringRef(theDisplayArgument) );
	} else if ( theTitle->Contains( CFSTR("%d") ) ) {
		theTitle = CFString::CreateWithFormat( theTitle, theDisplayArgument->GetIntValue() );
	}
	return theTitle->Copy();
}

NSString*
AiTunesControl::GetNewActionTitle() const
{
	return [CFBridgingToNS(ActionDescriptionStringKey( mCommand, true )) localized];
}

XIcon
AiTunesControl::GetActionXIcon() const
{
	KMIconRegistry iconSignature;
	
	switch ( mCommand ) {
		case iTunesControlCommand::kPlayPause:
			iconSignature = KMIconRegistry::kiTunesPlayIcon;
			break;
		case iTunesControlCommand::kStop:
			iconSignature = KMIconRegistry::kiTunesStopIcon;
			break;
		case iTunesControlCommand::kRewind:
			iconSignature = KMIconRegistry::kiTunesRewindIcon;
			break;
		case iTunesControlCommand::kFastForward:
			iconSignature = KMIconRegistry::kiTunesFastForwardIcon;
			break;
		case iTunesControlCommand::kPause:
			iconSignature = KMIconRegistry::kiTunesPauseIcon;
			break;
		case iTunesControlCommand::kNextTrack:
			iconSignature = KMIconRegistry::kiTunesNextIcon;
			break;
		case iTunesControlCommand::kBackTrack:
			iconSignature = KMIconRegistry::kiTunesPreviousIcon;
			break;
		case iTunesControlCommand::kPlayRandomTrack:
			iconSignature = KMIconRegistry::kiTunesPlayIcon;
			break;
		case iTunesControlCommand::kPlayNamedTrack:
			iconSignature = KMIconRegistry::kiTunesPlayIcon;
			break;
		case iTunesControlCommand::kPlayNamedPlaylist:
			iconSignature = KMIconRegistry::kiTunesPlayIcon;
			break;
		case iTunesControlCommand::kPlayRandomPlaylistTrack:
			iconSignature = KMIconRegistry::kiTunesPlayIcon;
			break;
		case iTunesControlCommand::kIncreaseVolume:
		case iTunesControlCommand::kDecreaseVolume:
		case iTunesControlCommand::kSetVolume:
			iconSignature = KMIconRegistry::kSoundIcon;
			break;
		case iTunesControlCommand::kIncreaseRating:
		case iTunesControlCommand::kDecreaseRating:
		case iTunesControlCommand::kSetRating:
			iconSignature = KMIconRegistry::kStarRatingIcon;
			break;
	}
	return XIcon( iconSignature );
}

void
AiTunesControl::SetCommand( iTunesControlCommand inCommand )
{
	mCommand = inCommand;
}

iTunesControlCommand
AiTunesControl::GetCommand() const
{
	return mCommand;
}

void
AiTunesControl::SetArgumentString( CFStringRef inString )
{
	check( inString );
	mArgumentString = inString->Copy();
}

void
AiTunesControl::SetArgumentInteger( NSInteger inValue )
{
	mArgumentString = CFCS( CFString::CreateWithNumber( inValue ) );
}

CFStringRef
AiTunesControl::CopyArgumentString() const
{
	return mArgumentString->Copy();
}

#if KMF_IS_ENGINE

#define kTelliTunesTo "tell application \"iTunes\" to "

float
AiTunesControl::DoAction()
{
	float time = 0;
	CFStringRef scriptBase;
	
	switch ( mCommand ) {
		case iTunesControlCommand::kPlayPause:
			scriptBase = CFSTR( kTelliTunesTo "playpause" );
			break;
		case iTunesControlCommand::kStop:
			scriptBase = CFSTR( kTelliTunesTo "stop" );
			break;
		case iTunesControlCommand::kRewind:
			scriptBase = CFSTR( kTelliTunesTo "rewind" );
			break;
		case iTunesControlCommand::kFastForward:
			scriptBase = CFSTR( kTelliTunesTo "fast forward" );
			break;
		case iTunesControlCommand::kPause:
			scriptBase = CFSTR( kTelliTunesTo "pause" );
			break;
		case iTunesControlCommand::kNextTrack:
			scriptBase = CFSTR( kTelliTunesTo "next track" );
			break;
		case iTunesControlCommand::kBackTrack:
			scriptBase = CFSTR( kTelliTunesTo "back track" );
			break;
		case iTunesControlCommand::kPlayRandomTrack:
			scriptBase = CFSTR( kTelliTunesTo "tell first source to tell first playlist to play some track" );
			break;
		case iTunesControlCommand::kPlayRandomPlaylistTrack:
			scriptBase = CFSTR( kTelliTunesTo "tell first source to tell playlist %@ to play some track" );
			break;
		case iTunesControlCommand::kPlayNamedTrack:
			scriptBase = CFSTR( kTelliTunesTo "tell first source to tell first playlist to play track %@" );
			break;
		case iTunesControlCommand::kPlayNamedPlaylist:
			scriptBase = CFSTR( kTelliTunesTo "tell first source to play playlist %@" );
			break;
		case iTunesControlCommand::kIncreaseVolume:
			scriptBase = CFSTR( kTelliTunesTo "set sound volume to sound volume + 10" );
			break;
		case iTunesControlCommand::kDecreaseVolume:
			scriptBase = CFSTR( kTelliTunesTo "set sound volume to sound volume - 10" );
			break;
		case iTunesControlCommand::kSetVolume:
			scriptBase = CFSTR( kTelliTunesTo "set sound volume to %d" );
			break;
		case iTunesControlCommand::kIncreaseRating:
			scriptBase = CFSTR( kTelliTunesTo "set rating of current track to ((rating of current track) + 10)" );
			break;
		case iTunesControlCommand::kDecreaseRating:
			scriptBase = CFSTR( kTelliTunesTo "set rating of current track to ((rating of current track) - 10)" );
			break;
		case iTunesControlCommand::kSetRating:
			scriptBase = CFSTR( kTelliTunesTo "set rating of current track to %d" );
			break;
	}
	if ( scriptBase->Contains( CFSTR("%@") ) ) {
		NSString* arg = TokenText::GetResult( mArgumentString->GetNS(), &time );
		(void)[NSAppleScript executeWithSource:scriptBase->GetNS() param:arg];
	} else if ( scriptBase->Contains( CFSTR("%d") ) ) {
		double arg;
		if ( !Calculate::CalculateValue( mArgumentString->GetNS(), arg ) ) {
			XErrorHandlerMaybeLogFormat1("always", @"iTunes Control %@ failed", mArgumentString->GetNS() );
		} else {
			(void)[NSAppleScript executeWithSource:scriptBase->GetNS() numericParameter:lround(arg)];
		}
	} else {
		(void)[NSAppleScript executeWithSource:scriptBase->GetNS()];
	}
	return time + 10;
}

#endif

#if KMF_EDITOR_WINDOW

Class
AiTunesControl::ActionController() const
{
	if ( [AiTunesControlEditor iTunesControlCommandNeedsEditor:mCommand] ) {
		return [AiTunesControlEditor class];
	}
	return HBaseAction::ActionController();
}

#endif

#pragma mark streaming

static const CFStringRef kiTunesControlCommandNames[] = kiTunesControlCommandNamesArray;

void
AiTunesControl::SaveToDictionary( CFMutableDictionaryRef dictionary ) const
{
	dictionary->Set( CFSTR("Command"), kiTunesControlCommandNames[int(mCommand)] );
	dictionary->Set( CFSTR("Argument"), CFStringRef(mArgumentString) );
}

void
AiTunesControl::InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException )
{
	bool good = dictionary->CanGetEnumClass( CFSTR("Command"), kiTunesControlCommandNames, mCommand );
	XAssert( good );

	CFStringRef newArgumentString;
	dictionary->Get( CFSTR("Argument"), newArgumentString, CFSTR("") );
	check( newArgumentString );
	mArgumentString = newArgumentString->Copy();
}
