//
//  ClipboardReference.h
//  Keyboard Maestro
//
//  Created by Peter N Lewis on Thu 20 Sep 2012.
//  Copyright (c) 2012 Stairways Software. All rights reserved.
//

#import "NSObjectExtensions.h"
#import "XException.h"

class CSClipboardDataSet;

class ClipboardReference final
{
private:
	NSString*						mClipboardUID;
	NSString*						mClipboardRedundantDisplayName;

public:

	ClipboardReference();
	ClipboardReference( const ClipboardReference& inObj );
	ClipboardReference( ClipboardReference&& inObj ); // NYI
	~ClipboardReference();

	void							UpdateClipboardReference();
	
	void							SetClipboardUID( NSString* uid );
	void							SetClipboardUID( CFStringRef uid ) { SetClipboardUID( CFBridgingToNS( uid ) ); }

	NSString*						GetClipboardUID() const { return mClipboardUID; }

	NSString*						GetClipboardName() const;
	CSClipboardDataSet*				GetClipboardDataSet() const;

	ClipboardReference&				operator=( const ClipboardReference& inObj );
	ClipboardReference&				operator=( ClipboardReference&& inObj ); // NYI

	bool							operator! () const { return IsEmpty( mClipboardUID ); }
	bool							Defined() const { return !*this; }

	void							SaveToDictionary( CFMutableDictionaryRef dictionary, NSString* uidKey = nil, NSString* redundantNameKey = nil ) const;
	void							InitializeFromDictionary( CFDictionaryRef dictionary, NSString* uidKey = nil, NSString* redundantNameKey = nil ) throw ( XException );

	static bool						ClipboardUIDInUse( NSString* uid );
};

