/*
 *  HActionList.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Tue Dec 11 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "XException.h"
#import "ClonePtr.h"

typedef ClonePtr<HBaseAction> HBaseActionClonePtr;
typedef std::list<HBaseActionClonePtr> HBaseActionClonePtrList;
typedef HBaseActionClonePtrList::iterator HBaseActionClonePtrListIterator;
typedef HBaseActionClonePtrList::const_iterator HBaseActionClonePtrListConstIterator;

class HActionList
{
private:

	HBaseActionClonePtrList mActionList;
		
public:

	HActionList() = default;
	HActionList( const HActionList& inObj ) = default;
	HActionList( HActionList&& inObj ) = default;
	HActionList( SafeCFPropertyListRef properties ) throw( XException );
	virtual ~HActionList();

	HActionList&				operator=( const HActionList& inObj ) = default;
	HActionList&				operator=( HActionList&& inObj ) = default;

	SafeCFPropertyListRef		CopyActionsToPropertyList() const;
	void						LoadActionsFromPropertyList( SafeCFPropertyListRef properties ) throw( XException );

	CFIndex						GetCount() const { return mActionList.size(); }
		
	void						AppendAction( HBaseAction* inAction );
	void						InsertActionAt( const CFIndex& inIndex, HBaseAction* inAction );
	void						ExchangeActionsAtIndices( const CFIndex& inIndex1, const CFIndex& inIndex2 );
	void						RelocateActionToIndex( const CFIndex& inCurIndex, const CFIndex& inNewIndex );
	
	void						DoForEveryAction( HBaseAction::ActionBlock inApplier );
	void						DoForEveryActionAndSubAction( HBaseAction::ActionBlock inApplier );

	HBaseAction*				GetAction( const CFIndex& inIndex ) const;
	HBaseAction*				operator[] ( const CFIndex& inIndex ) const { return GetAction( inIndex ); }
	void						SetActionAtIndex( const CFIndex& inIndex, HBaseAction* inAction );
	
	void						RemoveActionAtIndex( const CFIndex& inIndex );

	BOOL						MatchesTextSearch( NSString* searchString ) const;

#if KMF_IS_EDITOR
	void						Execute( NSString* displayName, NSString* trackingID = nil );
	static void					ExecuteAction( NSString* displayName, const HBaseAction* trialAction );
	NSArray*					GetDisplayContent() const;
#endif
	
	void						Empty();
	
	bool						DontRotateStatusMenu() const;
};

