/*
 *  HBaseAction.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Tue Dec 11 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "XException.h"
#import "XIcon.h"
#import "TrippedCaseBehaviour.h"

enum class HActionType {
	kProgramSwitcher,
	kWindowSwitcher,
	kApplicationLauncher,
	kClipboardSwitcherCopy,
	kClipboardSwitcherCut,
	kClipboardSwitcherPaste,
	kClipboardHistorySwitcher,
	kSendMessageToEditor,
	kDontRotateStatusMenu,
	kSearch,
	kTriggerByName,
	kBringWindowsForward,
	kSwitchToLastApplication,
	kSwitchToNextApplication,
	kActivateApplication,
	kQuitOthers,
	kQuitAll,
	kQuitSpecificApp,
	kHideOthers,
	kHideAll,
	kHideCurrent,
	kHideSpecificApp,
	kShowAll,
	kShowSpecificApp,
	kFile,
	kReadFile,
	kWriteFile,
	kNewFolder,
	kPlugIn,
	kGetFileAttribute,
	kSetFileAttribute,
	kOpen1File,
	kOpenFinderSelection,
	kSystemPrefPane,
	kOpenURL,
	kExecuteAppleScript,
	kExecuteShellScript,
	kExecuteJavaScript,
	kExecuteWorkflow,
	kExecuteMacro,
	kSetMacroEnable,
	kInsertText,
	kSetFindPasteboard,
	kSetNetworkLocation,
	kFastUserSwitch,
	kQuickMacro,
	kSimulateKeystroke,
	kMouseMoveAndClick,
	kScrollWheelEvent,
	kSelectMenuItem,
	kPressButton,
	kManipulateWindow,
	kHiliteLocation,
	kScreenCapture,
	kFindImage,
	kTransformImage,
	kResizeImage,
	kCompositeImage,
	kCopyClipboard,
	kDrawImage,
	kGetImageSize,
	kTrimImage,
	kDisplayImage,
	kNewImage,
	kPause,
	kPauseUntil,
	kFor,
	kWhile,
	kUntil,
	kRepeat,
	kIfThenElse,
	kAlert,
	kComment,
	kCancel,
	kMacroGroupToggle,
	kiTunesControl,
	kQuickTimePlayerControl,
	kSafariControl,
	kClipboardSwitcher,
	kSetClipboardToText,
	kSetClipboardToPastClipboard,
	kSetClipboardToNamedClipboard,
	kDeletePastClipboard,
	kSaveClipboardToNamedClipboard,
	kFilterClipboard,
	kSearchReplaceClipboard,
	kApplyStyle,
	kApplyTextFactory,
	kSetVariableToText,
	kSetVariableToKeychainPassword,
	kSetKeychainPasswordToText,
	kSetVariableToSafariField,
	kSetSafariFieldToText,
	kSetSafariCheckbox,
	kSetSafariRadio,
	kSetVariableToCalculation,
	kSearchReplaceVariable,
	kSearchVariable,
	kSearchClipboard,
	kSubstring,
	kUseVariable,
	kFilterVariable,
	kPromptForUserInput,
	kSpeakText,
	kPlaySound,
	kSystemAction,
	kDebuggerAction,
	kSystemVolume,
	kSendMIDIMessage,
	kGrowl,
	kLog,
	kNotification,
// LAST HActionType
};
#define kHActionTypeNamesArray {\
	CFSTR("ProgramSwitcher"), \
	CFSTR("WindowSwitcher"), \
	CFSTR("ApplicationLauncher"), \
	CFSTR("ClipboardSwitcherCopy"), \
	CFSTR("ClipboardSwitcherCut"), \
	CFSTR("ClipboardSwitcherPaste"), \
	CFSTR("ClipboardHistorySwitcher"), \
	CFSTR("SendMessageToEditor"), \
	CFSTR("DontRotateStatusMenu"), \
	CFSTR("Search"), \
	CFSTR("TriggerByName"), \
	CFSTR("BringWindowsForward"), \
	CFSTR("SwitchToLastApplication"), \
	CFSTR("SwitchToNextApplication"), \
	CFSTR("ActivateApplication"), \
	CFSTR("QuitOthers"), \
	CFSTR("QuitAll"), \
	CFSTR("QuitSpecificApp"), \
	CFSTR("HideOthers"), \
	CFSTR("HideAll"), \
	CFSTR("HideCurrent"), \
	CFSTR("HideSpecificApp"), \
	CFSTR("ShowAll"), \
	CFSTR("ShowSpecificApp"), \
	CFSTR("File"), \
	CFSTR("ReadFile"), \
	CFSTR("WriteFile"), \
	CFSTR("NewFolder"), \
	CFSTR("PlugIn"), \
	CFSTR("GetFileAttribute"), \
	CFSTR("SetFileAttribute"), \
	CFSTR("Open1File"), \
	CFSTR("OpenFinderSelection"), \
	CFSTR("SystemPrefPane"), \
	CFSTR("OpenURL"), \
	CFSTR("ExecuteAppleScript"), \
	CFSTR("ExecuteShellScript"), \
	CFSTR("ExecuteJavaScript"), \
	CFSTR("ExecuteWorkflow"), \
	CFSTR("ExecuteMacro"), \
	CFSTR("SetMacroEnable"), \
	CFSTR("InsertText"), \
	CFSTR("SetFindPasteboard"), \
	CFSTR("SetNetworkLocation"), \
	CFSTR("FastUserSwitch"), \
	CFSTR("QuickMacro"), \
	CFSTR("SimulateKeystroke"), \
	CFSTR("MouseMoveAndClick"), \
	CFSTR("ScrollWheelEvent"), \
	CFSTR("SelectMenuItem"), \
	CFSTR("PressButton"), \
	CFSTR("ManipulateWindow"), \
	CFSTR("HiliteLocation"), \
	CFSTR("ScreenCapture"), \
	CFSTR("FindImage"), \
	CFSTR("TransformImage"), \
	CFSTR("ResizeImage"), \
	CFSTR("CompositeImage"), \
	CFSTR("CopyClipboard"), \
	CFSTR("DrawImage"), \
	CFSTR("GetImageSize"), \
	CFSTR("TrimImage"), \
	CFSTR("DisplayImage"), \
	CFSTR("NewImage"), \
	CFSTR("Pause"), \
	CFSTR("PauseUntil"), \
	CFSTR("For"), \
	CFSTR("While"), \
	CFSTR("Until"), \
	CFSTR("Repeat"), \
	CFSTR("IfThenElse"), \
	CFSTR("Alert"), \
	CFSTR("Comment"), \
	CFSTR("Cancel"), \
	CFSTR("MacroGroupToggle"), \
	CFSTR("iTunesControl"), \
	CFSTR("QuickTimePlayerControl"), \
	CFSTR("SafariControl"), \
	CFSTR("ClipboardSwitcherMacroAction"), \
	CFSTR("SetClipboardToText"), \
	CFSTR("SetClipboardToPastClipboard"), \
	CFSTR("SetClipboardToNamedClipboard"), \
	CFSTR("DeletePastClipboard"), \
	CFSTR("SaveClipboardToNamedClipboard"), \
	CFSTR("FilterClipboard"), \
	CFSTR("SearchReplaceClipboard"), \
	CFSTR("ApplyStyle"), \
	CFSTR("ApplyTextFactory"), \
	CFSTR("SetVariableToText"), \
	CFSTR("SetVariableToKeychainPassword"), \
	CFSTR("SetKeychainPasswordToText"), \
	CFSTR("SetVariableToSafariField"), \
	CFSTR("SetSafariFieldToText"), \
	CFSTR("SetSafariCheckbox"), \
	CFSTR("SetSafariRadio"), \
	CFSTR("SetVariableToCalculation"), \
	CFSTR("SearchReplaceVariable"), \
	CFSTR("SearchVariable"), \
	CFSTR("SearchClipboard"), \
	CFSTR("Substring"), \
	CFSTR("UseVariable"), \
	CFSTR("FilterVariable"), \
	CFSTR("PromptForUserInput"), \
	CFSTR("SpeakText"), \
	CFSTR("PlaySound"), \
	CFSTR("SystemAction"), \
	CFSTR("DebuggerAction"), \
	CFSTR("SystemVolume"), \
	CFSTR("SendMIDIMessage"), \
	CFSTR("Growl"), \
	CFSTR("Log"), \
	CFSTR("Notification"), \
/* LAST kHActionTypeNamesArray */ \
	NULL}

#if KMF_IS_SWITCHERMAESTRO
#define hkKMSMProgramSwitcher CFSTR("hkSMApplicationSwitcher")
#else
#define hkKMSMProgramSwitcher CFSTR("hkProgramSwitcher")
#endif

#define kHotKeyActionNamesArray {\
	hkKMSMProgramSwitcher, \
	CFSTR("hkWindowSwitcher"), \
	CFSTR("hkApplicationLauncher"), \
	CFSTR("hkClipboardSwitcherCopy"), \
	CFSTR("hkClipboardSwitcherCut"), \
	CFSTR("hkClipboardSwitcherPaste"), \
	CFSTR("hkClipboardHistorySwitcher"), \
	CFSTR("hkSendMessageToEditor"), \
	CFSTR("hkDontRotateStatusMenu"), \
	CFSTR("hkSearch"), \
	CFSTR("hkTriggerByName"), \
	CFSTR("hkBringWindowsForward"), \
	CFSTR("hkSwitchToLastApplication"), \
	CFSTR("hkSwitchToNextApplication"), \
	CFSTR("hkActivateApplication"), \
	CFSTR("hkQuitOthers"), \
	CFSTR("hkQuitAll"), \
	CFSTR("hkQuitSpecificApp"), \
	CFSTR("hkHideOthers"), \
	CFSTR("hkHideAll"), \
	CFSTR("hkHideCurrent"), \
	CFSTR("hkHideSpecificApp"), \
	CFSTR("hkShowAll"), \
	CFSTR("hkShowSpecificApp"), \
	CFSTR("hkFile"), \
	CFSTR("hkReadFile"), \
	CFSTR("hWriteFileFormatType::k"), \
	CFSTR("hkNewFolder"), \
	CFSTR("hkPlugIn"), \
	CFSTR("hkGetFileAttribute"), \
	CFSTR("hkSetFileAttribute"), \
	CFSTR("hkOpen1File"), \
	CFSTR("hkOpenFinderSelection"), \
	CFSTR("hkSystemPrefPane"), \
	CFSTR("hkOpenURL"), \
	CFSTR("hkExecuteAppleScript"), \
	CFSTR("hkExecuteShellScript"), \
	CFSTR("hkExecuteJavaScript"), \
	CFSTR("hkExecuteWorkflow"), \
	CFSTR("hkExecuteMacro"), \
	CFSTR("hkSetMacroEnable"), \
	CFSTR("hkInsertText"), \
	CFSTR("hkSetFindPasteboard"), \
	CFSTR("hkSetNetworkLocation"), \
	CFSTR("hkFastUserSwitch"), \
	CFSTR("hkQuickMacro"), \
	CFSTR("hkSimulateKeystroke"), \
	CFSTR("hkMouseMoveAndClick"), \
	CFSTR("hkScrollWheelEvent"), \
	CFSTR("hkSelectMenuItem"), \
	CFSTR("hkPressButtonItem"), \
	CFSTR("hkManipulateWindow"), \
	CFSTR("hkHiliteLocation"), \
	CFSTR("hkScreenCapture"), \
	CFSTR("hkFindImage"), \
	CFSTR("hkTransformImage"), \
	CFSTR("hkResizeImage"), \
	CFSTR("hkCompositeImage"), \
	CFSTR("hkCopyClipboard"), \
	CFSTR("hkDrawImage"), \
	CFSTR("hkGetImageSize"), \
	CFSTR("hkTrimImage"), \
	CFSTR("hkDisplayImage"), \
	CFSTR("hkNewImage"), \
	CFSTR("hkPause"), \
	CFSTR("hkPauseUntil"), \
	CFSTR("hkFor"), \
	CFSTR("hkWhile"), \
	CFSTR("hkUntil"), \
	CFSTR("hkRepeat"), \
	CFSTR("hkIfThenElse"), \
	CFSTR("hkAlert"), \
	CFSTR("hkComment"), \
	CFSTR("hkCancel"), \
	CFSTR("hkMacroGroupToggle"), \
	CFSTR("hkiTunesControl"), \
	CFSTR("hkQuickTimePlayerControl"), \
	CFSTR("hkSafariControl"), \
	CFSTR("hkClipboardSwitcher"), \
	CFSTR("hkSetClipboardToText"), \
	CFSTR("hkSetClipboardToPastClipboard"), \
	CFSTR("hkSetClipboardToNamedClipboard"), \
	CFSTR("hkDeletePastClipboard"), \
	CFSTR("hkSaveClipboardToNamedClipboard"), \
	CFSTR("hkFilterClipboard"), \
	CFSTR("hkSearchReplaceClipboard"), \
	CFSTR("hkApplyStyle"), \
	CFSTR("hkApplyTextFactory"), \
	CFSTR("hkSetVariableToText"), \
	CFSTR("hkSetVariableToKeychainPassword"), \
	CFSTR("hkSetKeychainPasswordToText"), \
	CFSTR("hkSetVariableToSafariField"), \
	CFSTR("hkSetSafariFieldToText"), \
	CFSTR("hkSetSafariCheckbox"), \
	CFSTR("hkSetSafariRadio"), \
	CFSTR("hkSetVariableToCalculation"), \
	CFSTR("hkSearchReplaceVariable"), \
	CFSTR("hkSearchVariable"), \
	CFSTR("hkSearchClipboard"), \
	CFSTR("hkSubstring"), \
	CFSTR("hkUseVariable"), \
	CFSTR("hkFilterVariable"), \
	CFSTR("hkPromptForUserInput"), \
	CFSTR("hkSpeakText"), \
	CFSTR("hkPlaySound"), \
	CFSTR("hkSystemAction"), \
	CFSTR("hkDebuggerAction"), \
	CFSTR("hkSystemVolume"), \
	CFSTR("hkSendMIDIMessage"), \
	CFSTR("hkGrowl"), \
	CFSTR("hkLog"), \
	CFSTR("hkNotification"), \
/* LAST kHotKeyActionNamesArray */ \
	NULL}

const FlavorType kActionDragFlavor = 'aCTN';
const Boolean kDefaultTimeOutAbortsMacro = true;
const EventTime kVeryLongDefaultTimeOutValue = 99*kEventDurationHour;

class TBaseTrigger;

class HBaseAction
{
private:
	
	Boolean							mIsActive;
	Boolean							mIsDisclosed;
	EventTime						mTimeOutPeriod;
	Boolean							mTimeOutAbortsMacro;
	
protected:

	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary ) const = PURE_VIRTUAL;
	virtual void					SaveToDictionary( CFMutableDictionaryRef dictionary, BOOL includeRecordedDetails ) const { SaveToDictionary( dictionary ); }
	virtual void					InitializeFromDictionary( CFDictionaryRef dictionary ) throw ( XException ) = PURE_VIRTUAL;
	
	virtual HBaseAction*			CleanActionAfterInitialization() { return this; }

#if KMF_IS_EDITOR
	static NSString*				UserTextForHTML( NSString* text );
#endif

public:

	typedef void (^ActionBlock)( HBaseAction* inAction );

	HBaseAction();
	HBaseAction( const HBaseAction& inAction );
	virtual ~HBaseAction();

	virtual Boolean					IsActive() const { return mIsActive; }
#if KMF_IS_EDITOR
	virtual void					SetIsActive( const Boolean& inValue ) { mIsActive = inValue; }
#endif
	
	virtual Boolean					IsDisclosed() const { return mIsDisclosed; }
#if KMF_IS_EDITOR
	virtual void					SetIsDisclosed( const Boolean& inValue ) { mIsDisclosed = inValue; }
#endif
	
	static HBaseAction*				CreateActionOfType( const HActionType& inType ) throw( XException );
	static HBaseAction*				CreateFromPropertyList( SafeCFPropertyListRef properties ) throw( XException );

	virtual HBaseAction*			Clone() const = PURE_VIRTUAL;
	virtual HBaseAction*			Create() const = PURE_VIRTUAL;
	
#if KMF_EDITOR_WINDOW
	virtual void					UpdateToDefaultValues() { ; }
	virtual void					UpdateForPasteOrAdd() { ; }
#endif

	virtual bool					IsA( HBaseAction* action ) const { return GetActionType() == action->GetActionType(); }
	
	virtual HBaseAction*			Merge( HBaseAction* action ) { return NULL; }

	static XIcon					GetActionTypeXIcon( HActionType inType ); // Remove/make protected
	static CFStringRef				CopyActionTypeTitle( HActionType inType );
	
	virtual	CFStringRef				CopyActionDescriptionString() const { return CopyActionTitle(); }
	virtual	XIcon					GetActionDescriptionXIcon() const { return GetActionXIcon(); }
//	NSImage*						GetActionDescriptionImage() const { return GetActionDescriptionXIcon().GetImage(); };
	virtual	XIcon					GetActionDescriptionXIconWithDisabledBadge() const;
//	NSImage*						GetActionDescriptionImageWithDisabledBadge() const { return GetActionDescriptionXIconWithDisabledBadge().GetImage(); }

	virtual NSString*				GetToolTip() const { return nil; }

	virtual Boolean					IsLoop() const { return false; }

	virtual HActionType				GetActionType() const = PURE_VIRTUAL;
	virtual CFStringRef				CopyActionTitle() const { return CopyActionTypeTitle( GetActionType() ); }
	virtual XIcon					GetActionXIcon() const { return HBaseAction::GetActionTypeXIcon( GetActionType() ); }
//	NSImage*						GetActionImage() const { return GetActionXIcon().GetImage(); };

	virtual NSString*				GetActionTypeTitle( HActionType inType ) const { return NSCS(CopyActionTypeTitle( inType )); }
	virtual	NSString*				GetActionDescriptionString() const { return NSCS(CopyActionDescriptionString()); }
	virtual NSString*				GetActionTitle() const { return NSCS(CopyActionTitle()); }

	virtual NSString*				GetNewActionTitle() const { return NSCS(CopyActionTitle()); } // Displayed in Category/Actions list

	virtual void					DoForEverySubAction( ActionBlock inApplier ) { }
	
#if KMF_IS_ENGINE
	virtual float					DoAction() = PURE_VIRTUAL;
	
	virtual Boolean					IsActionCompleted() { return true; }

	virtual void					ActionAborted() { ; }
	
	// Sequence is always:
	// * DoAction
	// * IsActionCompleted repeatedly while false
	// * either IsActionCompleted returns true
	// * or ActionAborted
	// * or destroyed
#endif
	
	virtual Boolean					HasTimeOut() const { return false; }
	virtual EventTime				DefaultTimeOutPeriod() const { return 0; }
	
	EventTime						TimeOutPeriod() const { check( HasTimeOut() ); return mTimeOutPeriod ? mTimeOutPeriod : DefaultTimeOutPeriod(); }
	Boolean							TimeOutAbortsMacro() const { return mTimeOutAbortsMacro; }

#if KMF_EDITOR_WINDOW
	void							ResetDefaultTimeOut() { check( HasTimeOut() ); mTimeOutPeriod = 0; check( DefaultTimeOutPeriod() > 0 ); mTimeOutAbortsMacro = kDefaultTimeOutAbortsMacro; }
	void							SetTimeOutPeriod( EventTime inTimeOutPeriod ) { check( HasTimeOut() ); mTimeOutPeriod = inTimeOutPeriod; check( mTimeOutPeriod > 0 ); }
	void							SetTimeOutAbortsMacro( Boolean inTimeOutAbortsMacro ) { check( HasTimeOut() ); mTimeOutAbortsMacro = inTimeOutAbortsMacro; }

	virtual Class					ActionController() const;
	
	virtual	NSString*				GetActionHTMLDescriptionString() const { return GetActionDescriptionString(); }
	virtual	NSString*				GetActionExtraHTML() const { return @""; }

	virtual NSString*				ActionDisplayHTML() const;
#endif

	virtual BOOL					IsClickAction() const { return NO; }
	virtual BOOL					CanBeInstigatedByClickAction() const { return NO; }

	virtual BOOL					MatchesTextSearch( NSString* search ) const;

	virtual SafeCFPropertyListRef	CopyToPropertyList( BOOL includeRecordedDetails ) const;
	id								GetPropertyList( BOOL includeRecordedDetails ) const { return CFBridgingSafeRelease(CopyToPropertyList( includeRecordedDetails )); }

};
