/*
 *  HBaseAction.cpp
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Tue Dec 11 2001.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#import "HBaseAction.h"
#import "KMDefines.h"
#import "KMIconRegistry.h"
#import "XIcon.h"
#import "NSStringExtensions.h"
#import "NSStringRegularExpressions.h"

#import "ALastApplication.h"
#import "ABringWindowsForward.h"
#import "AOpenURL.h"
#import "AQuitApplications.h"
#import "AHideApplications.h"
#import "AShowApplications.h"
#import "ANextApplication.h"
#import "AActivateApplication.h"
#import "AInsertText.h"
#import "ASetFindPasteboard.h"
#import "ASetNetworkLocation.h"
#import "AFastUserSwitch.h"
#import "AQuickMacro.h"
#import "ASimulateKeystroke.h"
#import "AExecuteAppleScript.h"
#import "AExecuteShellScript.h"
#import "AExecuteJavaScript.h"
#import "AExecuteWorkflow.h"
#import "AExecuteMacro.h"
#import "ASetMacroEnable.h"
#import "AOpenPrefPane.h"
#import "AFile.h"
#import "AReadFile.h"
#import "AWriteFile.h"
#import "ANewFolder.h"
#import "APlugIn.h"
#import "AGetFileAttribute.h"
#import "ASetFileAttribute.h"
#import "AOpen1File.h"
#import "AOpenFinderSelection.h"
#import "AMouseMoveAndClick.h"
#import "ASystemAction.h"
#import "ADebuggerAction.h"
#import "ASpeakText.h"
#import "APlaySound.h"
#import "ASystemVolume.h"
#import "ASendMIDIMessage.h"
#import "APause.h"
#import "APauseUntil.h"
#import "AWhile.h"
#import "AFor.h"
#import "AUntil.h"
#import "ARepeat.h"
#import "AIfThenElse.h"
#import "AAlert.h"
#import "AComment.h"
#import "AGrowl.h"
#import "ALog.h"
#import "ANotification.h"
#import "AMacroGroupToggle.h"
#import "AQuitSpecificApplication.h"
#import "AHideSpecificApplication.h"
#import "AShowSpecificApplication.h"
#import "ASelectMenuItem.h"
#import "AiTunesControl.h"
#import "AQuickTimePlayerControl.h"
#import "ASafariControl.h"
#import "AScrollWheelEvent.h"
#import "AManipulateWindow.h"
#import "AHiliteLocation.h"
#import "AScreenCapture.h"
#import "AFindImage.h"
#import "ATransformImage.h"
#import "ACompositeImage.h"
#import "ACopyClipboard.h"
#import "ADrawImage.h"
#import "AResizeImage.h"
#import "AGetImageSize.h"
#import "ATrimImage.h"
#import "ADisplayImage.h"
#import "ANewImage.h"
#import "APressButton.h"
#import "AClipboardSwitcherMacroAction.h"
#import "ASetClipboardToText.h"
#import "ASetClipboardToPastClipboard.h"
#import "ASetClipboardToNamedClipboard.h"
#import "ADeletePastClipboard.h"
#import "ASaveClipboardToNamedClipboard.h"
#import "AFilterClipboard.h"
#import "ASearchReplaceClipboard.h"
#import "AApplyStyle.h"
#import "AApplyTextFactory.h"
#import "ASetVariableToText.h"
#import "ASetVariableToKeychainPassword.h"
#import "ASetKeychainPasswordToText.h"
#import "ASetVariableToSafariField.h"
#import "ASetSafariFieldToText.h"
#import "ASetSafariCheckbox.h"
#import "ASetSafariRadio.h"
#import "ASetVariableToCalculation.h"
#import "ASearchReplaceVariable.h"
#import "ASearchVariable.h"
#import "ASearchClipboard.h"
#import "ASubstring.h"
#import "AUseVariable.h"
#import "AFilterVariable.h"
#import "APromptForUserInput.h"
#import "AProgramSwitcher.h"
#import "AClipboardSwitcher.h"
#import "AWindowSwitcher.h"
#import "AApplicationLauncher.h"
#import "AClipboardHistorySwitcher.h"
#import "ACancel.h"
#import "ActionViewListItemController.h"
#import "ASearch.h"
#import "ATriggerByName.h"
#import "ASendMessageToEditor.h"
#import "ADontRotateStatusMenu.h"
#import "KMPreferences.h"
#import "TemplateEngine.h"
#import "XErrorHandler.h"

HBaseAction::HBaseAction()
	:	mIsActive( true ),
		mIsDisclosed( false ),
		mTimeOutPeriod( 0 ),
		mTimeOutAbortsMacro( kDefaultTimeOutAbortsMacro )
{
}

HBaseAction::HBaseAction( const HBaseAction& inAction )
	:	mIsActive( inAction.mIsActive ),
		mIsDisclosed( inAction.mIsDisclosed ),
		mTimeOutPeriod( inAction.mTimeOutPeriod ),
		mTimeOutAbortsMacro( inAction.mTimeOutAbortsMacro )
{
}


HBaseAction::~HBaseAction()
{

}

HBaseAction*
HBaseAction::CreateActionOfType( const HActionType& inType ) throw(XException)
{
	HBaseAction* pAction = NULL;

	switch (inType) {
		case HActionType::kBringWindowsForward:
			pAction = new ABringWindowsForward();
			break;
		case HActionType::kSwitchToLastApplication:
			pAction = new ALastApplication();
			break;
		case HActionType::kSwitchToNextApplication:
			pAction = new ANextApplication();
			break;
		case HActionType::kActivateApplication:
			pAction = new AActivateApplication();
			break;
		case HActionType::kQuitOthers:
			pAction = new AQuitApplications( EQuitApplicationsActionType::kOtherApplications );
			break;
		case HActionType::kQuitAll:
			pAction = new AQuitApplications( EQuitApplicationsActionType::kAllApplications );
			break;
		case HActionType::kQuitSpecificApp:
			pAction = new AQuitSpecificApplication();
			break;
		case HActionType::kHideOthers:
			pAction = new AHideApplications( EHideApplicationsActionType::kOtherApplications );
			break;
		case HActionType::kHideAll:
			pAction = new AHideApplications( EHideApplicationsActionType::kAllApplications );
			break;
		case HActionType::kHideCurrent:
			pAction = new AHideApplications( EHideApplicationsActionType::kCurrentApplication );
			break;
		case HActionType::kHideSpecificApp:
			pAction = new AHideSpecificApplication();
			break;
		case HActionType::kShowSpecificApp:
			pAction = new AShowSpecificApplication();
			break;
		case HActionType::kShowAll:
			pAction = new AShowApplications();
			break;
		case HActionType::kOpenURL:
			pAction = new AOpenURL();
			break;
		case HActionType::kExecuteAppleScript:
			pAction = new AExecuteAppleScript();
			break;
		case HActionType::kExecuteShellScript:
			pAction = new AExecuteShellScript();
			break;
		case HActionType::kExecuteJavaScript:
			pAction = new AExecuteJavaScript();
			break;
		case HActionType::kExecuteWorkflow:
			pAction = new AExecuteWorkflow();
			break;
		case HActionType::kExecuteMacro:
			pAction = new AExecuteMacro();
			break;
		case HActionType::kSetMacroEnable:
			pAction = new ASetMacroEnable();
			break;
		case HActionType::kInsertText:
			pAction = new AInsertText();
			break;
		case HActionType::kSetFindPasteboard:
			pAction = new ASetFindPasteboard();
			break;
		case HActionType::kSetNetworkLocation:
			pAction = new ASetNetworkLocation();
			break;
		case HActionType::kFastUserSwitch:
			pAction = new AFastUserSwitch();
			break;
		case HActionType::kQuickMacro:
			pAction = new AQuickMacro();
			break;
		case HActionType::kSimulateKeystroke:
			pAction = new ASimulateKeystroke();
			break;
		case HActionType::kSystemPrefPane:
			pAction = new AOpenPrefPane();
			break;
		case HActionType::kFile:
			pAction = new AFile();
			break;
		case HActionType::kReadFile:
			pAction = new AReadFile();
			break;
		case HActionType::kWriteFile:
			pAction = new AWriteFile();
			break;
		case HActionType::kNewFolder:
			pAction = new ANewFolder();
			break;
		case HActionType::kPlugIn:
			pAction = new APlugIn();
			break;
		case HActionType::kGetFileAttribute:
			pAction = new AGetFileAttribute();
			break;
		case HActionType::kSetFileAttribute:
			pAction = new ASetFileAttribute();
			break;
		case HActionType::kOpen1File:
			pAction = new AOpen1File();
			break;
		case HActionType::kOpenFinderSelection:
			pAction = new AOpenFinderSelection();
			break;
		case HActionType::kMouseMoveAndClick:
			pAction = new AMouseMoveAndClick();
			break;
		case HActionType::kSystemAction:
			pAction = new ASystemAction();
			break;
		case HActionType::kDebuggerAction:
			pAction = new ADebuggerAction();
			break;
		case HActionType::kSpeakText:
			pAction = new ASpeakText();
			break;
		case HActionType::kPlaySound:
			pAction = new APlaySound();
			break;
		case HActionType::kSystemVolume:
			pAction = new ASystemVolume();
			break;
		case HActionType::kSendMIDIMessage:
			pAction = new ASendMIDIMessage();
			break;
		case HActionType::kPause:
			pAction = new APause();
			break;
		case HActionType::kPauseUntil:
			pAction = new APauseUntil();
			break;
		case HActionType::kWhile:
			pAction = new AWhile();
			break;
		case HActionType::kUntil:
			pAction = new AUntil();
			break;
		case HActionType::kRepeat:
			pAction = new ARepeat();
			break;
		case HActionType::kIfThenElse:
			pAction = new AIfThenElse();
			break;
		case HActionType::kFor:
			pAction = new AFor();
			break;
		case HActionType::kAlert:
			pAction = new AAlert();
			break;
		case HActionType::kComment:
			pAction = new AComment();
			break;
		case HActionType::kCancel:
			pAction = new ACancel();
			break;
		case HActionType::kMacroGroupToggle:
			pAction = new AMacroGroupToggle();
			break;
		case HActionType::kSelectMenuItem:
			pAction = new ASelectMenuItem();
			break;
		case HActionType::kiTunesControl:
			pAction = new AiTunesControl();
			break;
		case HActionType::kQuickTimePlayerControl:
			pAction = new AQuickTimePlayerControl();
			break;
		case HActionType::kSafariControl:
			pAction = new ASafariControl();
			break;
		case HActionType::kScrollWheelEvent:
			pAction = new AScrollWheelEvent();
			break;
		case HActionType::kManipulateWindow:
			pAction = new AManipulateWindow();
			break;
		case HActionType::kHiliteLocation:
			pAction = new AHiliteLocation();
			break;
		case HActionType::kScreenCapture:
			pAction = new AScreenCapture();
			break;
		case HActionType::kFindImage:
			pAction = new AFindImage();
			break;
		case HActionType::kTransformImage:
			pAction = new ATransformImage();
			break;
		case HActionType::kCompositeImage:
			pAction = new ACompositeImage();
			break;
		case HActionType::kCopyClipboard:
			pAction = new ACopyClipboard();
			break;
		case HActionType::kDrawImage:
			pAction = new ADrawImage();
			break;
		case HActionType::kResizeImage:
			pAction = new AResizeImage();
			break;
		case HActionType::kGetImageSize:
			pAction = new AGetImageSize();
			break;
		case HActionType::kTrimImage:
			pAction = new ATrimImage();
			break;
		case HActionType::kDisplayImage:
			pAction = new ADisplayImage();
			break;
		case HActionType::kNewImage:
			pAction = new ANewImage();
			break;
		case HActionType::kPressButton:
			pAction = new APressButton();
			break;
		case HActionType::kClipboardSwitcher:
			pAction = new AClipboardSwitcherMacroAction();
			break;

		case HActionType::kSetClipboardToText:
			pAction = new ASetClipboardToText();
			break;
		case HActionType::kSetClipboardToPastClipboard:
			pAction = new ASetClipboardToPastClipboard();
			break;
		case HActionType::kSetClipboardToNamedClipboard:
			pAction = new ASetClipboardToNamedClipboard();
			break;
		case HActionType::kSaveClipboardToNamedClipboard:
			pAction = new ASaveClipboardToNamedClipboard();
			break;
		case HActionType::kDeletePastClipboard:
			pAction = new ADeletePastClipboard();
			break;
		case HActionType::kFilterClipboard:
			pAction = new AFilterClipboard();
			break;
		case HActionType::kSearchReplaceClipboard:
			pAction = new ASearchReplaceClipboard();
			break;
		case HActionType::kApplyStyle:
			pAction = new AApplyStyle();
			break;
		case HActionType::kApplyTextFactory:
			pAction = new AApplyTextFactory();
			break;

		case HActionType::kSetVariableToText:
			pAction = new ASetVariableToText();
			break;
		case HActionType::kSetVariableToKeychainPassword:
			pAction = new ASetVariableToKeychainPassword();
			break;
		case HActionType::kSetKeychainPasswordToText:
			pAction = new ASetKeychainPasswordToText();
			break;
		case HActionType::kSetVariableToSafariField:
			pAction = new ASetVariableToSafariField();
			break;
		case HActionType::kSetSafariFieldToText:
			pAction = new ASetSafariFieldToText();
			break;
		case HActionType::kSetSafariCheckbox:
			pAction = new ASetSafariCheckbox();
			break;
		case HActionType::kSetSafariRadio:
			pAction = new ASetSafariRadio();
			break;
		case HActionType::kSetVariableToCalculation:
			pAction = new ASetVariableToCalculation();
			break;
		case HActionType::kSearchReplaceVariable:
			pAction = new ASearchReplaceVariable();
			break;
		case HActionType::kSearchVariable:
			pAction = new ASearchVariable();
			break;
		case HActionType::kSearchClipboard:
			pAction = new ASearchClipboard();
			break;
		case HActionType::kSubstring:
			pAction = new ASubstring();
			break;
		case HActionType::kUseVariable:
			pAction = new AUseVariable();
			break;
		case HActionType::kFilterVariable:
			pAction = new AFilterVariable();
			break;
		case HActionType::kPromptForUserInput:
			pAction = new APromptForUserInput();
			break;
			
		case HActionType::kSendMessageToEditor:
			pAction = new ASendMessageToEditor();
			break;
		case HActionType::kDontRotateStatusMenu:
			pAction = new ADontRotateStatusMenu();
			break;
		case HActionType::kSearch:
			pAction = new ASearch();
			break;
		case HActionType::kTriggerByName:
			pAction = new ATriggerByName();
			break;
		case HActionType::kProgramSwitcher:
			pAction = new AProgramSwitcher();
			break;
		case HActionType::kWindowSwitcher:
			pAction = new AWindowSwitcher();
			break;
		case HActionType::kApplicationLauncher:
			pAction = new AApplicationLauncher();
			break;
		case HActionType::kClipboardSwitcherCopy:
			pAction = new AClipboardSwitcher( ClipboardSwitcherType::kCopy );
			break;
		case HActionType::kClipboardSwitcherCut:
			pAction = new AClipboardSwitcher( ClipboardSwitcherType::kCut );
			break;
		case HActionType::kClipboardSwitcherPaste:
			pAction = new AClipboardSwitcher( ClipboardSwitcherType::kPaste );
			break;
		case HActionType::kClipboardHistorySwitcher:
			pAction = new AClipboardHistorySwitcher();
			break;
		case HActionType::kGrowl:
			pAction = new AGrowl();
			break;
		case HActionType::kLog:
			pAction = new ALog();
			break;
		case HActionType::kNotification:
			pAction = new ANotification();
			break;
// LAST CreateActionOfType
	}

	return pAction;
}

XIcon HBaseAction::GetActionTypeXIcon( HActionType inType )
{
	XIcon result;

	switch ( inType ) {
		case HActionType::kSwitchToLastApplication: {
			XIcon backgroundIcon( KMIconRegistry::kGenericApplicationIcon );
			XIcon foregroundIcon( KMIconRegistry::kPreviousBadgeIcon );
			result = XIcon( backgroundIcon, foregroundIcon );
			break;
		}
		case HActionType::kSwitchToNextApplication: {
			XIcon backgroundIcon( KMIconRegistry::kGenericApplicationIcon );
			XIcon foregroundIcon( KMIconRegistry::kNextBadgeIcon );
			result = XIcon( backgroundIcon, foregroundIcon );
			break;
		}
		case HActionType::kBringWindowsForward:
			result = XIcon( KMIconRegistry::kWindowIcon );// kGenericWindowIcon
			break;
		case HActionType::kOpenFinderSelection:
			result = XIcon( KMIconRegistry::kFinderIcon );
			break;
		case HActionType::kActivateApplication:
			result = XIcon( KMIconRegistry::kGenericApplicationIcon );
			break;
		case HActionType::kFile:
		case HActionType::kReadFile:
		case HActionType::kWriteFile:
		case HActionType::kGetFileAttribute:
		case HActionType::kSetFileAttribute:
		case HActionType::kOpen1File:
			result = XIcon( KMIconRegistry::kGenericDocumentIcon );
			break;
		case HActionType::kNewFolder:
			result = XIcon( KMIconRegistry::kGenericFolderIcon );
			break;
		case HActionType::kFastUserSwitch:
			result = XIcon( KMIconRegistry::kUserIcon );
			break;
		case HActionType::kOpenURL:
			result = XIcon( KMIconRegistry::kInternetLocationGenericIcon );
			break;
		case HActionType::kQuitSpecificApp:
		case HActionType::kQuitOthers:
		case HActionType::kQuitAll: {
			XIcon backgroundIcon( KMIconRegistry::kGenericApplicationIcon );
			result = XIcon( backgroundIcon, XIcon(KMIconRegistry::kQuitBadgeIcon) );
			break;
		}
		case HActionType::kExecuteAppleScript: {
			result = XIcon( KMIconRegistry::kAppleScript );
			break;
		}
		case HActionType::kSendMessageToEditor:
		case HActionType::kDontRotateStatusMenu:
			result = XIcon( KMIconRegistry::kKeyCapsIcon );
			break;
		case HActionType::kSetNetworkLocation:
		case HActionType::kSearch:
			result = XIcon( KMIconRegistry::kGenericNetworkIcon );
			break;
		case HActionType::kHideAll:
		case HActionType::kHideOthers:
		case HActionType::kHideCurrent:
		case HActionType::kHideSpecificApp:
		case HActionType::kShowAll:
		case HActionType::kShowSpecificApp:
		case HActionType::kProgramSwitcher:
		case HActionType::kApplicationLauncher:
			result = XIcon( KMIconRegistry::kGenericApplicationIcon );
			break;
		case HActionType::kLog:
		case HActionType::kPlugIn:
			result = XIcon( KMIconRegistry::kKeyboardMaestroIcon );
			break;
		case HActionType::kClipboardSwitcher:
		case HActionType::kClipboardSwitcherCopy:
		case HActionType::kClipboardSwitcherCut:
		case HActionType::kClipboardSwitcherPaste:
		case HActionType::kClipboardHistorySwitcher:
		case HActionType::kInsertText:
		case HActionType::kSetFindPasteboard:
		case HActionType::kSetClipboardToPastClipboard:
		case HActionType::kSetClipboardToNamedClipboard:
		case HActionType::kSaveClipboardToNamedClipboard:
		case HActionType::kDeletePastClipboard:
		case HActionType::kSetClipboardToText:
		case HActionType::kFilterClipboard:
		case HActionType::kSearchReplaceClipboard:
		case HActionType::kApplyStyle:
		case HActionType::kApplyTextFactory:
		case HActionType::kCopyClipboard:
		case HActionType::kSearchClipboard:
			result = XIcon( KMIconRegistry::kClipboardAction );
			break;
		case HActionType::kSetVariableToText:
		case HActionType::kSetVariableToKeychainPassword:
		case HActionType::kSetKeychainPasswordToText:
		case HActionType::kSetVariableToCalculation:
		case HActionType::kSearchReplaceVariable:
		case HActionType::kSearchVariable:
		case HActionType::kSubstring:
		case HActionType::kUseVariable:
		case HActionType::kFilterVariable:
			result = XIcon( KMIconRegistry::kVariableIcon );
			break;
		case HActionType::kPromptForUserInput:
			result = XIcon( KMIconRegistry::kWindowIcon );
			break;
		case HActionType::kQuickMacro:
			result = XIcon( KMIconRegistry::kRecordQuickMacroAction );
			break;
		case HActionType::kSimulateKeystroke:
			result = XIcon( KMIconRegistry::kKeyCapsIcon );
			break;
		case HActionType::kExecuteShellScript:
			result = XIcon( KMIconRegistry::kExecuteShellScriptIcon );
			break;
		case HActionType::kExecuteJavaScript:
			result = XIcon( KMIconRegistry::kSafari );
			break;
		case HActionType::kExecuteWorkflow:
			result = XIcon( KMIconRegistry::kWorkflow );
			break;
		case HActionType::kDebuggerAction:
		case HActionType::kTriggerByName:
		case HActionType::kExecuteMacro:
			result = XIcon( KMIconRegistry::kKeyCapsIcon );
			break;
		case HActionType::kSetMacroEnable:
			result = XIcon( KMIconRegistry::kKeyCapsIcon );
			break;
		case HActionType::kSystemPrefPane:
			result = XIcon( KMIconRegistry::kPreferencePane );
			break;
		case HActionType::kMouseMoveAndClick:
			result = XIcon( KMIconRegistry::kMoveClickMouseIcon );
			break;
		case HActionType::kSystemAction:
			result = XIcon( KMIconRegistry::kFinderIcon );
			break;
		case HActionType::kSpeakText:
		case HActionType::kPlaySound:
			result = XIcon( KMIconRegistry::kSoundIcon );
			break;
		case HActionType::kSystemVolume:
			result = XIcon( KMIconRegistry::kSoundIcon );
			break;
		case HActionType::kSendMIDIMessage:
			result = XIcon( KMIconRegistry::kMIDIIcon );
			break;
		case HActionType::kPause:
		case HActionType::kPauseUntil:
			result = XIcon( KMIconRegistry::kPauseActionIcon );
			break;
		case HActionType::kWhile:
		case HActionType::kUntil:
		case HActionType::kRepeat:
		case HActionType::kIfThenElse:
		case HActionType::kFor:
			result = XIcon( KMIconRegistry::kControlFlowIcon );
			break;
	case HActionType::kAlert:
			result = XIcon( KMIconRegistry::kAlertCautionIcon );
			break;
		case HActionType::kComment:
			result = XIcon( KMIconRegistry::kAlertNoteIcon );
			break;
		case HActionType::kCancel:
			result = XIcon( KMIconRegistry::kAlertStopIcon );
			break;
		case HActionType::kMacroGroupToggle:
			result = XIcon( KMIconRegistry::kGenericFolderIcon );
			break;
		case HActionType::kSelectMenuItem:
			result = XIcon( KMIconRegistry::kSelectMenuItemIcon );
			break;
		case HActionType::kiTunesControl:
			result = XIcon( KMIconRegistry::kiTunes );
			break;
		case HActionType::kSetVariableToSafariField:
		case HActionType::kSetSafariFieldToText:
		case HActionType::kSetSafariCheckbox:
		case HActionType::kSetSafariRadio:
		case HActionType::kSafariControl:
			result = XIcon( KMIconRegistry::kSafari );
			break;
		case HActionType::kQuickTimePlayerControl:
			result = XIcon( KMIconRegistry::kQuickTimePlayer );
			break;
		case HActionType::kManipulateWindow:
		case HActionType::kHiliteLocation:
		case HActionType::kWindowSwitcher:
		case HActionType::kDisplayImage:
		case HActionType::kScreenCapture:
		case HActionType::kFindImage:
			result = XIcon( KMIconRegistry::kWindowIcon );
			break;
		case HActionType::kTransformImage:
		case HActionType::kCompositeImage:
		case HActionType::kDrawImage:
		case HActionType::kResizeImage:
		case HActionType::kGetImageSize:
		case HActionType::kTrimImage:
		case HActionType::kNewImage:
			result = XIcon( KMIconRegistry::kPreview );
			break;
		case HActionType::kPressButton:
			result = XIcon( KMIconRegistry::kPressButtonAction );
			break;
		case HActionType::kScrollWheelEvent:
			result = XIcon( KMIconRegistry::kScrollWheelIcon );
			break;
		case HActionType::kGrowl:
			result = XIcon( KMIconRegistry::kGrowl );
			break;
		case HActionType::kNotification:
			result = XIcon( KMIconRegistry::kNotifications );
			break;
// LAST GetActionTypeXIcon
	}
	check( result.Defined() );
	return result;
}

BOOL
HBaseAction::MatchesTextSearch( NSString* searchString ) const
{
	if ( [searchString length] == 0 ) return YES;
	if ( [GetActionDescriptionString() rangeOfString:searchString options:NSCaseInsensitiveSearch+NSDiacriticInsensitiveSearch].location != NSNotFound ) return YES;
	if ( [GetActionTitle() rangeOfString:searchString options:NSCaseInsensitiveSearch+NSDiacriticInsensitiveSearch].location != NSNotFound ) return YES;
	return NO;
}

XIcon
HBaseAction::GetActionDescriptionXIconWithDisabledBadge() const
{
	if ( !mIsActive ) {
		return GetActionDescriptionXIcon().DimmedIcon();
	}
	return GetActionDescriptionXIcon();
	
}

static const CFStringRef kHotKeyActionName[] = kHotKeyActionNamesArray;

CFStringRef
HBaseAction::CopyActionTypeTitle( HActionType inType )
{
	CFStringRef		theString = NULL;
	CFStringRef		theKey = kHotKeyActionName[int(inType)];
	
	if ( theKey ) {
		theString =	theKey->CopyLocalized();
	} else {
		check( false );
		theString = CFSTR("Unknown Action");
	}
	return theString;
}

#if KMF_EDITOR_WINDOW

Class
HBaseAction::ActionController() const
{
	return [ActionViewListItemController class];
}

static NSString* kActionDisplayHTML = @R"RAW(
<img src={{ icon | inlineimage: 16 }} alt="Action icon" width="16" height="16" /><span class="IconTitle{% if disabled  %} Disabled{% /if %}">{{ description | htmlencode }}</span><br />
 {% if extra %}<div class="ActionExtra{% if disabled  %} Disabled{% /if %}">
	{{ extra }}
 </div>
{% /if %}
)RAW";

#if KMF_IS_EDITOR
NSString*
HBaseAction::UserTextForHTML( NSString* text )
{
	return [text isMatchedByRegex:@"\\S"] ? [NSString stringWithFormat:@"<div class=\"UserText\">%@</div>", [text HTMLString]] : @"";

}
#endif

NSString*
HBaseAction::ActionDisplayHTML() const
{
	NSString* result = @"<div><strong>Exception happeneded while trying to get display information.<br />Please report to <a href=\"support@stairways.com\">support@stairways.com</a></strong></div>";
	@try {
		NSImage* icon = GetActionDescriptionXIconWithDisabledBadge().GetImageForDisplay();
		NSString* extra = GetActionExtraHTML();
		if ( ![extra isMatchedByRegex:@"\\S"] ) {
			extra = @"";
		}
		NSDictionary *variables = @{
		@"icon" : icon,
		@"description" : GetActionHTMLDescriptionString(),
		@"disabled" : @(!IsActive()),
		@"extra" : extra
		};
		result = [[TemplateEngine sharedDetailViewTemplateEngine] processTemplate:kActionDisplayHTML withVariables:variables];
	}
	@catch (NSException* exception) {
		XErrorHandlerMaybeLogFormat1( "always", @"ActionDisplayHTML exception %@", exception );
	}
	@catch (...) {
		XErrorHandlerMaybeLogMessage( "always", @"ActionDisplayHTML unknown exception" );
	}
	return result;
}

#endif


#pragma mark streaming

static const CFStringRef kHActionTypeNames[] = kHActionTypeNamesArray;

SafeCFPropertyListRef
HBaseAction::CopyToPropertyList( BOOL includeRecordedDetails ) const
{
	CFMutableDictionaryRef actionDictionary = CFDictionary::CreateMutable();

	CFBooleanRef isActive = mIsActive ? kCFBooleanTrue : kCFBooleanFalse;
	actionDictionary->Set( CFSTR("IsActive"), isActive );
	
	actionDictionary->Set( CFSTR("IsDisclosed"), mIsDisclosed ? kCFBooleanTrue : kCFBooleanFalse );
	if ( HasTimeOut() ) {
		if ( lround(mTimeOutPeriod) != 0 ) {
			actionDictionary->SetDouble( CFSTR("TimeOutPeriod"), mTimeOutPeriod );
		}
		actionDictionary->Set( CFSTR("TimeOutAbortsMacro"), mTimeOutAbortsMacro ? kCFBooleanTrue : kCFBooleanFalse );
	}
	
	actionDictionary->Set( CFSTR("MacroActionType"), kHActionTypeNames[int(GetActionType())] );
	
	SaveToDictionary( actionDictionary, includeRecordedDetails );
	
	return (SafeCFPropertyListRef)actionDictionary;
}

HBaseAction*
HBaseAction::CreateFromPropertyList( SafeCFPropertyListRef properties ) throw( XException )
{
	HBaseAction* action = NULL;
	CFDictionaryRef actionDictionary = (CFDictionaryRef)properties;
	
	XAssert( actionDictionary && actionDictionary->IsCorrectTypeID() );
	HActionType actionType;
	bool good = actionDictionary->CanGetEnumClass( CFSTR("MacroActionType"), kHActionTypeNames, actionType );
	XAssert( good );
	action = CreateActionOfType( actionType );
	XAssert( action );

	actionDictionary->GetBoolean( CFSTR("IsActive"), action->mIsActive, action->mIsActive );
	actionDictionary->GetBoolean( CFSTR("IsDisclosed"), action->mIsDisclosed, action->mIsDisclosed );
	action->mTimeOutPeriod = 0;
	action->mTimeOutAbortsMacro = kDefaultTimeOutAbortsMacro;
	if ( !actionDictionary->Get( CFSTR("TimeOutPeriod"), action->mTimeOutPeriod ) ) action->mTimeOutPeriod = 0;
	actionDictionary->GetBoolean( CFSTR("TimeOutAbortsMacro"), action->mTimeOutAbortsMacro, action->mTimeOutAbortsMacro );
	
	action->InitializeFromDictionary( actionDictionary );
	action = action->CleanActionAfterInitialization();
	
	return action;
}


