/*
 *  HExecuteActionListManager.h
 *  Keyboard Maestro
 *
 *  Created by Michael Kamprath on Sun Apr 21 2002.
 *  Copyright (c) 2004 Stairways Software. All rights reserved.
 */

#if KMF_IS_EDITOR
#error Engine only
#endif

enum class ExecuteActionListDisplay {
	kDisplay,
	kHide
};

#import "TrippedCaseBehaviour.h"

class MMacro;
class HActionList;
class HBaseAction;
class TBaseTrigger;

enum class ExecutingEntryDebugState {
	kRunning,
	kPaused,
	kStepOver,
	kStepInto,
	kStepOut,
};

@interface HExecutionEntry : NSObject

@property (nonatomic, copy) NSString* name;
@property (nonatomic, assign) NSUInteger order;
@property (nonatomic, copy) NSUUID* executionGroup;
@property (nonatomic, assign) ExecutingEntryDebugState debugState;
@property (nonatomic, copy) NSArray* actions; // current or next

- (NSComparisonResult) compare: (HExecutionEntry*) other;

- (void) cancelExecutionGroup;
- (void) setExecutionDebugState: (ExecutingEntryDebugState) state;

@end

enum class ExecutingListSortOrder {
	kAlphabetical,
	kExecution,
};

namespace HExecuteActionListManager {
	
	void					Execute( ExecuteActionListDisplay display, const MMacro* inMacro, TBaseTrigger* inTrippedTrigger, NSUUID* trackingID );
	void					Execute( ExecuteActionListDisplay display, NSString* displayName, const HActionList& inActionList, NSUUID* trackingID = NULL );
	void					Execute( ExecuteActionListDisplay display, NSString* displayName, const HBaseAction* inTrialAction, NSUUID* trackingID = NULL );

	NSArray*				ExecutingList( ExecutingListSortOrder sort ); // Sorted List or HExecutionEntry. Currently executing displayed macro groups

	bool					IsExecuting();
	bool					IsExecuting( NSUUID* trackingID );
	void					CancelExecution( NSUUID* trackingID );
	void					CancelAllExecution();
	void					CancelExecutionGroup( NSUUID* executionGroup );
	void					SetExecutionDebugState( NSUUID* executionGroup, ExecutingEntryDebugState state );
	void					SetExecutionDebugStateAll( ExecutingEntryDebugState state, NSUUID* execeptExecutionGroup = nil );
	void					SetExecutionPausedAll( BOOL paused, NSUUID* execeptExecutionGroup = nil );
	NSUUID*					GetThisExecutionGroup();			// Valid only from an executing action
	void					CancelAllOtherExecutionGroups();	// Valid only from an executing action
	void					CancelThisExecutionGroup();			// Valid only from an executing action
	void					CancelJustThisMacro();				// Valid only from an executing action
	void					CancelJustThisLoop();				// Valid only from an executing action
	UInt32					GetTrippedKeyCode();				// Valid only from an executing action
	UInt32					GetTrippedModifiers();				// Valid only from an executing action
	TrippedCaseBehaviour	GetTrippedCase();					// Valid only from an executing action	
	NSString*				ExecutingMacroName();				// Valid only from an executing action	
	NSString*				ExecutingMacroGroupName();			// Valid only from an executing action	
};

