/*
 *  HTimeSaved.cpp
 *  Keyboard Maestro
 *
 *  Created by Peter N Lewis on Thu 31 Jan 2008.
 *  Copyright (c) 2008 Stairways Software. All rights reserved.
 */

#import "HTimeSaved.h"
#import "KMPreferences.h"
#import "XEventLoopTimer.h"
#import "KMNotifications.h"
#import "XNotificationCenter.h"

using namespace HTimeSaved;

#if KMF_IS_ENGINE

class TimeSavedHolder
	:	public XEventLoopTimer
{
private:

	float			mTimeAll;
	float			mTimeV4;
	float			mTimeV5;
	float			mTimeV6;
	Boolean			mDirty;

protected:
	
	virtual void	HandleTimerEvent();

public:

	TimeSavedHolder();
	virtual ~TimeSavedHolder();
	
	void			Save();
	void			Add( float seconds );
};

TimeSavedHolder* gTimeSavedHolder = NULL;

TimeSavedHolder::TimeSavedHolder()
	:	XEventLoopTimer( GetMainEventLoop(), kEventDurationSecond*10, kEventDurationSecond*10 ),
		mDirty( false )
{
	KMPreferences::Get( KMPreferences::kTimeSavedAllID, &mTimeAll, 0.0 );
	KMPreferences::Get( KMPreferences::kTimeSavedV4ID, &mTimeV4, 0.0 );
	KMPreferences::Get( KMPreferences::kTimeSavedV5ID, &mTimeV5, 0.0 );
	KMPreferences::Get( KMPreferences::kTimeSavedV6ID, &mTimeV6, 0.0 );
}

TimeSavedHolder::~TimeSavedHolder()
{
	Save();
}

void
TimeSavedHolder::Add( float seconds )
{
	mTimeAll += seconds;
	mTimeV4 += seconds;
	mTimeV5 += seconds;
	mTimeV6 += seconds;
	mDirty = true;
}

void
TimeSavedHolder::Save()
{
	KMPreferences::Set( KMPreferences::kTimeSavedAllID, mTimeAll );
	KMPreferences::Set( KMPreferences::kTimeSavedV4ID, mTimeV4 );
	KMPreferences::Set( KMPreferences::kTimeSavedV5ID, mTimeV5 );
	KMPreferences::Set( KMPreferences::kTimeSavedV6ID, mTimeV6 );
	mDirty = false;
	
	XNotificationCenter::PostNotification( KMNotifications::kTimeSavedUpdated, @{ KMNotifications::kTimeSavedKey : @(mTimeAll) } );
}

void
TimeSavedHolder::HandleTimerEvent()
{
	if ( mDirty ) {
		Save();
	}
}

void
HTimeSaved::Add( float seconds )
{
	if ( !gTimeSavedHolder ) {
		gTimeSavedHolder = new TimeSavedHolder();
	}
	gTimeSavedHolder->Add( seconds );
}

void
HTimeSaved::Save()
{
	if ( gTimeSavedHolder ) {
		gTimeSavedHolder->Save();
	}
}

#endif

static CFAbsoluteTime GetFirstUse6Date()
{
	float firstUse;
	if ( KMPreferences::Get( KMPreferences::kFirstUseDateV4ID, &firstUse ) != noErr ) {
		firstUse = CFAbsoluteTimeGetCurrent();
		KMPreferences::Set( KMPreferences::kFirstUseDateV4ID, firstUse );
	}
	if ( KMPreferences::Get( KMPreferences::kFirstUseDateV5ID, &firstUse ) != noErr ) {
		firstUse = CFAbsoluteTimeGetCurrent();
		KMPreferences::Set( KMPreferences::kFirstUseDateV5ID, firstUse );
	}
	if ( KMPreferences::Get( KMPreferences::kFirstUseDateV6ID, &firstUse ) != noErr ) {
		firstUse = CFAbsoluteTimeGetCurrent();
		KMPreferences::Set( KMPreferences::kFirstUseDateV6ID, firstUse );
	}
	return firstUse;
}

void
HTimeSaved::SetFirstUseDate()
{
	(void) GetFirstUse6Date();
}

NSInteger
HTimeSaved::DaysOfUse6()
{
	return lroundf((CFAbsoluteTimeGetCurrent() - GetFirstUse6Date())/86400)+1;
}

float
HTimeSaved::GetAll()
{
	float time;
	KMPreferences::Get( KMPreferences::kTimeSavedAllID, &time, 0.0 );
	return time;
}

float
HTimeSaved::GetV6()
{
	float time;
	KMPreferences::Get( KMPreferences::kTimeSavedV6ID, &time, 0.0 );
	return time;
}

CFStringRef
HTimeSaved::CopyAboutString( float timesaved )
{
	CFStringRef formatKey;
	long int value;
	if ( timesaved < 60 ) {
		formatKey = CFSTR("TimeSavedFormat_None");
		value = 0;
	} else if ( timesaved < 8000 ) {
		formatKey = CFSTR("TimeSavedFormat_Minutes");
		value = lroundf( timesaved / 60.0 );
	} else {
		formatKey = CFSTR("TimeSavedFormat_Hours");
		value = lroundf( timesaved / 3600.0 );
	}
	CFContainer<CFStringRef> text = CFString::CreateWithFormat( CFCS( formatKey->CopyLocalized() ), value );
	return text->Copy();
}

CFStringRef
HTimeSaved::CopyTimeString( float timesaved )
{
	CFStringRef formatKey;
	long int value;
	if ( timesaved < 60*2.6 ) {
		formatKey = CFSTR("TimeSavedTimeFormat_Seconds");
		value = lroundf( timesaved );
	} else if ( timesaved < 3600*2.6 ) {
		formatKey = CFSTR("TimeSavedTimeFormat_Minutes");
		value = lroundf( timesaved / 60.0 );
	} else {
		formatKey = CFSTR("TimeSavedTimeFormat_Hours");
		value = lroundf( timesaved / 3600.0 );
	}
	CFContainer<CFStringRef> text = CFString::CreateWithFormat( CFCS( formatKey->CopyLocalized() ), value );
	return text->Copy();
}
